@@ -3,7 +3,7 @@
 
 #include <antlr/config.hpp>
 #include "GDLInterpreterTokenTypes.hpp"
-/* $ANTLR 2.7.6 (20071205): "gdlc.i.g" -> "GDLInterpreter.hpp"$ */
+/* $ANTLR 2.7.7 (20110618): "gdlc.i.g" -> "GDLInterpreter.hpp"$ */
 #include <antlr/TreeParser.hpp>
 
 
@@ -67,7 +67,6 @@ private:
     friend class FOREACH_INDEX_LOOPNode;
     friend class FOR_STEPNode;
     friend class FOR_STEP_LOOPNode;
-    
     friend class KEYDEFNode;
     friend class KEYDEF_REFNode;
     friend class KEYDEF_REF_CHECKNode;
@@ -76,6 +75,10 @@ private:
     friend class REF_CHECKNode;
     friend class REF_EXPRNode;
     friend class ParameterNode;
+    friend class REFVNNode;
+    friend class REF_CHECKVNNode;
+    friend class REF_EXPRVNNode;
+    friend class ParameterVNNode;
 
 public: 
 
@@ -87,6 +90,10 @@ public:
     {
         this->_retTree = rT;
     }
+    ProgNodeP GetRetTree() const
+    {
+        return this->_retTree;
+    }
 //     void SetReturnCode( RetCode rC)
 //     {
 //         this->returnCode = rC;
@@ -101,7 +108,8 @@ public:
 //     };  
 
     // code in: dinterpreter.cpp
-    static bool SearchCompilePro(const std::string& pro);
+    // procedure (searchForPro == true) or function (searchForPro == false)
+    static bool SearchCompilePro(const std::string& pro, bool searchForPro); 
     static int GetFunIx( ProgNodeP);
     static int GetFunIx( const std::string& subName);
     static int GetProIx( ProgNodeP);//const std::string& subName);
@@ -157,7 +165,10 @@ protected:
     bool interruptEnable;
 
 public:
-    static bool CompileFile(const std::string& f, const std::string& untilPro=""); 
+    // procedure (searchForPro == true) or function (searchForPro == false)
+    static bool CompileFile(const std::string& f, 
+                            const std::string& untilPro="",
+                            bool searchForPro=true); 
 
     typedef RefHeap<BaseGDL> RefBaseGDL;
     typedef RefHeap<DStructGDL> RefDStructGDL;
@@ -268,33 +279,33 @@ public:
     }
 
    static void DecRef( DPtr id)
-    {
-        if( id != 0)
-            {
+   {
+       if( id != 0)
+           {
 #ifdef GDL_DEBUG_HEAP
-                std::cout << "-- <PtrHeapVar" << id << ">" << std::endl; 
+               std::cout << "-- <PtrHeapVar" << id << ">" << std::endl; 
 #endif
-                HeapT::iterator it=heap.find( id);
-                if( it != heap.end()) 
-                    { 
-                        if( (*it).second.Dec())
-                            {
+               HeapT::iterator it=heap.find( id);
+               if( it != heap.end()) 
+                   { 
+                       if( (*it).second.Dec())
+                           {
 #ifdef GDL_DEBUG_HEAP
-                                std::cout << "Out of scope (garbage collected): <PtrHeapVar" << id 
-                                          << ">"
-                                          << " at: " << callStack.back()->GetProName()
-                                          << "  line: " << callStack.back()->GetLineNumber()
-                                          << std::endl; 
+                               std::cout << "Out of scope (garbage collected): <PtrHeapVar" << id 
+                                         << ">"
+                                         << " at: " << callStack.back()->GetProName()
+                                         << "  line: " << callStack.back()->GetLineNumber()
+                                         << std::endl; 
 #endif
-                                FreeHeapDirect( id, it);
-                            }
+                               FreeHeapDirect( id, it);
+                           }
 #ifdef GDL_DEBUG_HEAP
-                        else
-std::cout << "<PtrHeapVar" << id << "> = " << (*it).second.Count() << std::endl; 
+                       else
+                           std::cout << "<PtrHeapVar" << id << "> = " << (*it).second.Count() << std::endl; 
 #endif
-                    }
-            }
-    }
+                   }
+           }
+   }
    static void DecRef( DPtrGDL* p)
     {
         SizeT nEl=p->N_Elements();
@@ -318,10 +329,10 @@ std::cout << "-- <ObjHeapVar" << id << ">" << std::endl;
                            {
 #ifdef GDL_DEBUG_HEAP
                                std::cout << "Out of scope (garbage collected): <ObjHeapVar" << id 
-                                          << ">"
-                                          << " at: " << callStack.back()->GetProName()
-                                          << "  line: " << callStack.back()->GetLineNumber()
-                                          << std::endl; 
+                                         << ">"
+                                         << " at: " << callStack.back()->GetProName()
+                                         << "  line: " << callStack.back()->GetLineNumber()
+                                         << std::endl; 
 #endif
                                callStack.back()->ObjCleanup( id);
 //                             FreeObjHeapDirect( id, it);
@@ -722,25 +733,14 @@ public:
 	public:  BaseGDL*  call_fun(ProgNodeP _t);
 	public:  BaseGDL**  call_lfun(ProgNodeP _t);
 	public: void call_pro(ProgNodeP _t);
-	public:  RetCode  block(ProgNodeP _t);
-	public:  RetCode  switch_statement(ProgNodeP _t);
-	public: BaseGDL*  expr(ProgNodeP _t);
-	public:  RetCode  case_statement(ProgNodeP _t);
-	public:  RetCode  repeat_statement(ProgNodeP _t);
-	public:  RetCode  repeat_loop_statement(ProgNodeP _t);
-	public:  RetCode  while_statement(ProgNodeP _t);
-	public:  RetCode  for_statement(ProgNodeP _t);
-	public:  RetCode  foreach_statement(ProgNodeP _t);
-	public:  RetCode  if_statement(ProgNodeP _t);
-	public:  RetCode  if_else_statement(ProgNodeP _t);
 	public: BaseGDL**  l_deref(ProgNodeP _t);
 	public: BaseGDL**  l_ret_expr(ProgNodeP _t);
+	public: BaseGDL*  expr(ProgNodeP _t);
 	public:  BaseGDL**  l_arrayexpr_mfcall_as_mfcall(ProgNodeP _t);
 	public:  BaseGDL**  l_function_call(ProgNodeP _t);
 	public: BaseGDL*  tmp_expr(ProgNodeP _t);
 	public:  BaseGDL*  lib_function_call(ProgNodeP _t);
 	public: BaseGDL*  r_expr(ProgNodeP _t);
-	public: BaseGDL*  constant_nocopy(ProgNodeP _t);
 	public: BaseGDL*  l_decinc_indexable_expr(ProgNodeP _t,
 		int dec_inc
 	);
@@ -772,13 +772,12 @@ public:
 		EnvBaseT* actEnv
 	);
 	public: BaseGDL**  l_indexable_expr(ProgNodeP _t);
-	public: BaseGDL**  l_array_expr(ProgNodeP _t,
+	public: BaseGDL**  unused_l_array_expr(ProgNodeP _t,
 		BaseGDL* right
 	);
 	public: BaseGDL**  l_arrayexpr_mfcall(ProgNodeP _t,
 		BaseGDL* right
 	);
-	public: BaseGDL*  array_expr(ProgNodeP _t);
 	public: void tag_expr(ProgNodeP _t,
 		DotAccessDescT* aD
 	);
@@ -791,20 +790,22 @@ public:
 	);
 	public: BaseGDL*  dot_expr(ProgNodeP _t);
 	public: BaseGDL*  assign_expr(ProgNodeP _t);
-	public:  BaseGDL*  function_call(ProgNodeP _t);
+	public:  BaseGDL*  unused_function_call(ProgNodeP _t);
 	public:  BaseGDL*  lib_function_call_retnew(ProgNodeP _t);
 	public: BaseGDL*  constant(ProgNodeP _t);
 	public: BaseGDL*  simple_var(ProgNodeP _t);
 	public: BaseGDL*  sys_var(ProgNodeP _t);
+	public: BaseGDL*  unused_constant_nocopy(ProgNodeP _t);
 	public: BaseGDL**  l_arrayexpr_mfcall_as_arrayexpr(ProgNodeP _t,
 		BaseGDL* right
 	);
-	public:  BaseGDL**  ref_parameter(ProgNodeP _t,
-		EnvBaseT* actEnv
-	);
 	public: void parameter_def_n_elements(ProgNodeP _t,
 		EnvBaseT* actEnv
 	);
+	public: void parameter_def_nocheck(ProgNodeP _t,
+		EnvBaseT* actEnv
+	);
+	public: ArrayIndexListT*  arrayindex_list_noassoc(ProgNodeP _t);
 public:
 	antlr::RefAST getAST()
 	{
@@ -817,10 +818,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 227;
+	static const int NUM_TOKENS = 232;
 #else
 	enum {
-		NUM_TOKENS = 227
+		NUM_TOKENS = 232
 	};
 #endif
 	
