@@ -2,7 +2,7 @@
 #define INC_GDLLexer_hpp_
 
 #include <antlr/config.hpp>
-/* $ANTLR 2.7.6 (20071205): "gdlc.g" -> "GDLLexer.hpp"$ */
+/* $ANTLR 2.7.7 (20110618): "gdlc.g" -> "GDLLexer.hpp"$ */
 #include <antlr/CommonToken.hpp>
 #include <antlr/InputBuffer.hpp>
 #include <antlr/BitSet.hpp>
@@ -58,7 +58,7 @@ class CUSTOM_API GDLLexer : public antlr::CharScanner, public GDLTokenTypes
 
     // main lexer constructor
     GDLLexer( std::istream& in, const std::string f, unsigned int compileOptIn,
-        const std::string pro="") 
+        const std::string pro="", bool searchForPro=true) 
     : antlr::CharScanner(new antlr::CharBuffer(in),false),
       lineContinuation( 0)
 //    : antlr::CharScanner(in)
@@ -68,7 +68,7 @@ class CUSTOM_API GDLLexer : public antlr::CharScanner, public GDLTokenTypes
   
         selector=     new antlr::TokenStreamSelector();
         mainLexerPtr= this;
-        parserPtr=    new GDLParser( *selector, pro, compileOptIn);
+        parserPtr=    new GDLParser( *selector, pro, searchForPro, compileOptIn);
 
         parserPtr->setFilename(f);
         parserPtr->initializeASTFactory( DNodeFactory);
@@ -254,6 +254,7 @@ public:
 	protected: void mSKIP_LINES(bool _createToken);
 	public: void mCONT_STATEMENT(bool _createToken);
 	public: void mEND_OF_LINE(bool _createToken);
+	protected: void mMAX_TOKEN_NUMBER(bool _createToken);
 private:
 	
 	static const unsigned long _tokenSet_0_data_[];
