@@ -2,7 +2,7 @@
 #define INC_GDLParser_hpp_
 
 #include <antlr/config.hpp>
-/* $ANTLR 2.7.6 (20071205): "gdlc.g" -> "GDLParser.hpp"$ */
+/* $ANTLR 2.7.7 (20110618): "gdlc.g" -> "GDLParser.hpp"$ */
 #include <antlr/TokenStream.hpp>
 #include <antlr/TokenBuffer.hpp>
 #include "GDLTokenTypes.hpp"
@@ -65,7 +65,8 @@ class CUSTOM_API GDLParser : public antlr::LLkParser, public GDLTokenTypes
     }
 
     std::string subName; // name of procedure function to be compiled ("" -> all file)
-    bool   subReached;
+    bool   searchForPro; // true -> procedure subName, false -> function subName 
+    bool   subReached; 
     unsigned int compileOpt;
 
     bool ConstantExprNode( int t)
@@ -75,8 +76,11 @@ class CUSTOM_API GDLParser : public antlr::LLkParser, public GDLTokenTypes
     }
 
     public:
-    GDLParser(antlr::TokenStream& selector, const std::string& sName, unsigned int compileOptIn):
-    antlr::LLkParser(selector,2), subName(sName), 
+    GDLParser(antlr::TokenStream& selector, 
+              const std::string& sName, 
+              bool searchPro, // true -> search for procedure sName, false -> for function
+              unsigned int compileOptIn):
+    antlr::LLkParser(selector,2), subName(sName), searchForPro( searchPro), 
     subReached(false), compileOpt(compileOptIn)
     { 
         //        setTokenNames(_tokenNames);
@@ -210,10 +214,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 227;
+	static const int NUM_TOKENS = 232;
 #else
 	enum {
-		NUM_TOKENS = 227
+		NUM_TOKENS = 232
 	};
 #endif
 	
