@@ -3,7 +3,7 @@
 
 #include <antlr/config.hpp>
 #include "GDLTreeParserTokenTypes.hpp"
-/* $ANTLR 2.7.6 (20071205): "gdlc.tree.g" -> "GDLTreeParser.hpp"$ */
+/* $ANTLR 2.7.7 (20110618): "gdlc.tree.g" -> "GDLTreeParser.hpp"$ */
 #include <antlr/TreeParser.hpp>
 
 
@@ -22,38 +22,6 @@ class CUSTOM_API GDLTreeParser : public antlr::TreeParser, public GDLTreeParserT
     
     IDList          loopVarStack;
     
-//     // called after structure is fixed
-//     ArrayIndexListT* MakeArrayIndex( ArrayIndexVectorT* ixList)
-//     {
-//         assert( ixList->size() != 0); // must be, from compiler
-        
-//         if( ixList->size() == 1)
-//         {
-//             if( dynamic_cast< CArrayIndexScalar*>((*ixList)[0]))
-//                 return new ArrayIndexListOneConstScalarT( ixList);
-
-//             if( dynamic_cast< ArrayIndexScalar*>((*ixList)[0]))
-//                 return new ArrayIndexListOneScalarT( ixList);
-
-//             if( dynamic_cast< ArrayIndexScalarVP*>((*ixList)[0]))
-//                 return new ArrayIndexListOneScalarVPT( ixList);
-
-//             return new ArrayIndexListOneT( ixList);
-//         }
-
-//         SizeT nScalar  = 0;
-//         for( SizeT i=0; i<ixList->size(); ++i)
-//         {
-//             if( dynamic_cast< ArrayIndexScalar*>((*ixList)[i]) ||
-//                 dynamic_cast< ArrayIndexScalarVP*>((*ixList)[i]) ||
-//                 dynamic_cast< CArrayIndexScalar*>((*ixList)[i])) ++nScalar;
-//         }
-//         if( nScalar == ixList->size())
-//         return new ArrayIndexListScalarT( ixList);
-        
-//         return new ArrayIndexListMultiT( ixList);
-//     }
-
     bool LoopVar( RefDNode& lN)
     {
         int lT = lN->getType();
@@ -179,9 +147,13 @@ public:
 	public: void while_statement(RefDNode _t);
 	public: void jump_statement(RefDNode _t);
 	public: void if_statement(RefDNode _t);
-	public: void parameter_def(RefDNode _t);
+	public: void parameter_def(RefDNode _t,
+		bool varNum
+	);
 	public: void key_parameter(RefDNode _t);
-	public: void pos_parameter(RefDNode _t);
+	public: void pos_parameter(RefDNode _t,
+		bool varNum
+	);
 	public: int  array_def(RefDNode _t);
 	public: void struct_def(RefDNode _t);
 	public: void tag_def(RefDNode _t);
@@ -193,7 +165,9 @@ public:
 	public: void assign_expr(RefDNode _t);
 	public: void sysvar(RefDNode _t);
 	public: void var(RefDNode _t);
-	public: void arrayindex_list_to_parameter_list(RefDNode _t);
+	public: void arrayindex_list_to_parameter_list(RefDNode _t,
+		bool varNum
+	);
 	public: void arrayexpr_fn(RefDNode _t);
 	public: void arrayexpr_mfcall(RefDNode _t);
 	public: void primary_expr(RefDNode _t);
@@ -219,10 +193,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 227;
+	static const int NUM_TOKENS = 232;
 #else
 	enum {
-		NUM_TOKENS = 227
+		NUM_TOKENS = 232
 	};
 #endif
 	
