@@ -36,7 +36,10 @@ BaseGDL* BaseGDL::AssocVar( int, SizeT)
   throw GDLException("BaseGDL::AssocVar(...) called.");
 }
 
-SizeT BaseGDL::N_Elements() const { return dim.N_Elements();}
+SizeT BaseGDL::N_Elements() const 
+{
+  return dim.NDimElementsConst();
+}
 SizeT BaseGDL::Size() const { return 0;}
 SizeT BaseGDL::ToTransfer() const { return 0;}
 SizeT BaseGDL::Sizeof() const { return 0;}
@@ -49,6 +52,11 @@ BaseGDL& BaseGDL::operator=(const BaseGDL& right)
   return *this;
 }
 
+void  BaseGDL::InitFrom(const BaseGDL& right)
+{
+  throw GDLException("BaseGDL::InitFrom(const BaseGDL& right) called.");
+}
+
 
 // BaseGDL* BaseGDL::Abs() const
 // {
@@ -146,10 +154,14 @@ void* BaseGDL::DataAddr()// SizeT elem)
 }
   
 // make same type on the heap
-BaseGDL* BaseGDL::New( const dimension& dim_, InitType noZero)
+BaseGDL* BaseGDL::New( const dimension& dim_, InitType noZero) const
 {
   throw GDLException("BaseGDL::New(...) called.");
 }
+BaseGDL* BaseGDL::NewResult() const
+{
+  throw GDLException("BaseGDL::NewResult() called.");
+}
 
 BaseGDL* BaseGDL::Dup() const
 { 
@@ -196,6 +208,15 @@ int BaseGDL::Scalar2RangeT(RangeT& ret) const
   throw GDLException("Operation not defined for UNDEF 1a.");
 }
 
+SizeT BaseGDL::GetAsIndex( SizeT i) const
+{ 
+  throw GDLException("BaseGDL::GetAsIndex called.");
+}
+SizeT BaseGDL::GetAsIndexStrict( SizeT i) const
+{ 
+  throw GDLException("BaseGDL::GetAsIndexStrict called.");
+}
+
 bool BaseGDL::BaseGDL::True()
 {
   throw GDLException("Operation not defined for UNDEF 3.");
@@ -391,7 +412,7 @@ BaseGDL* BaseGDL::GtMark( BaseGDL* r)
 
 BaseGDL* BaseGDL::Mult( BaseGDL* r)   
 {
-  throw GDLException("Operation not defined for UNDEF 19.");
+  throw GDLException("Operation not defined for UNDEF 19a.");
 }
 
 BaseGDL* BaseGDL::Div( BaseGDL* r)      
@@ -424,10 +445,6 @@ BaseGDL* BaseGDL::PowInt( BaseGDL* r)
 {
   throw GDLException("Operation not defined for UNDEF 170.");
 }
-BaseGDL* BaseGDL::PowIntNew( BaseGDL* r)   
-{
-  throw GDLException("Operation not defined for UNDEF 171.");
-}
 
 BaseGDL* BaseGDL::MatrixOp( BaseGDL* r, bool rtranspose, bool transposeResult, bool strassen)
 {
@@ -508,80 +525,6 @@ PyObject* BaseGDL::ToPython()
 
 
 
-BaseGDL* BaseGDL::AndOpNew( BaseGDL* r)    
-{
-  throw GDLException("Operation not defined for UNDEF 13.");
-}
-
-// BaseGDL* BaseGDL::AndOpInvNew( BaseGDL* r) 
-// {
-//   throw GDLException("Operation not defined for UNDEF 14.");
-// }
-
-BaseGDL* BaseGDL::OrOpNew( BaseGDL* r)    
-{
-  throw GDLException("Operation not defined for UNDEF 13.");
-}
-
-// BaseGDL* BaseGDL::OrOpInvNew( BaseGDL* r) 
-// {
-//   throw GDLException("Operation not defined for UNDEF 14.");
-// }
-
-BaseGDL* BaseGDL::XorOpNew( BaseGDL* r)    
-{
-  throw GDLException("Operation not defined for UNDEF 13.");
-}
-BaseGDL* BaseGDL::AddNew( BaseGDL* r)      
-{
-  throw GDLException("Operation not defined for UNDEF 15.");
-}
-// BaseGDL* BaseGDL::AddInvNew( BaseGDL* r)      
-// {
-//   throw GDLException("Operation not defined for UNDEF 15.");
-// }
-
-BaseGDL* BaseGDL::SubNew( BaseGDL* r)      
-{
-  throw GDLException("Operation not defined for UNDEF 16.");
-}
-// BaseGDL* BaseGDL::SubInvNew( BaseGDL* r)   
-// {
-//   throw GDLException("Operation not defined for UNDEF 17.");
-// }
-BaseGDL* BaseGDL::MultNew( BaseGDL* r)   
-{
-  throw GDLException("Operation not defined for UNDEF 19.");
-}
-
-BaseGDL* BaseGDL::DivNew( BaseGDL* r)      
-{
-  throw GDLException("Operation not defined for UNDEF 16.");
-}
-// BaseGDL* BaseGDL::DivInvNew( BaseGDL* r)   
-// {
-//   throw GDLException("Operation not defined for UNDEF 17.");
-// }
-
-BaseGDL* BaseGDL::ModNew( BaseGDL* r)      
-{
-  throw GDLException("Operation not defined for UNDEF 16.");
-}
-// BaseGDL* BaseGDL::ModInvNew( BaseGDL* r)   
-// {
-//   throw GDLException("Operation not defined for UNDEF 17.");
-// }
-
-BaseGDL* BaseGDL::PowNew( BaseGDL* r)      
-{
-  throw GDLException("Operation not defined for UNDEF 16.");
-}
-// BaseGDL* BaseGDL::PowInvNew( BaseGDL* r)   
-// {
-//   throw GDLException("Operation not defined for UNDEF 17.");
-// }
-
-
 
 
 BaseGDL* BaseGDL::AndOpS( BaseGDL* r)    
@@ -634,10 +577,10 @@ BaseGDL* BaseGDL::GtMarkS( BaseGDL* r)
 
 BaseGDL* BaseGDL::MultS( BaseGDL* r)   
 {
-  throw GDLException("Operation not defined for UNDEF 19.");
+  throw GDLException("Operation not defined for UNDEF 19a.");
 }
 
-BaseGDL* BaseGDL::DivS( BaseGDL* r)      
+BaseGDL* BaseGDL::DivS( BaseGDL* r)
 {
   throw GDLException("Operation not defined for UNDEF 16.");
 }
@@ -672,7 +615,7 @@ BaseGDL* BaseGDL::NewIx( BaseGDL* ix, bool strict)
 {
   throw GDLException("Operation not defined for UNDEF 21.");
 }
-BaseGDL* BaseGDL::NewIx( AllIxT* ix, dimension* dIn)
+BaseGDL* BaseGDL::NewIx( AllIxBaseT* ix, const dimension* dIn)
 {
   throw GDLException("Operation not defined for UNDEF 22.");
 }
@@ -714,7 +657,7 @@ void BaseGDL::AssignAtIx( RangeT ix, BaseGDL* srcIn)
   throw GDLException("Operation not defined for UNDEF 28.");
 }
 
-SizeT BaseGDL::LoopIndex() const
+RangeT BaseGDL::LoopIndex() const
 { 
   throw GDLException("Operation not defined for UNDEF 29.");
 }
@@ -734,6 +677,52 @@ BaseGDL* BaseGDL::DupReverse( DLong dim)
   throw GDLException("Operation not defined for UNDEF 32.");
 }
 
+  BaseGDL* BaseGDL::AndOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 40.");}
+  BaseGDL* BaseGDL::AndOpInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 41.");}
+  BaseGDL* BaseGDL::OrOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 42.");}
+  BaseGDL* BaseGDL::OrOpInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 43.");}
+  BaseGDL* BaseGDL::XorOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 44.");}
+//   BaseGDL* BaseGDL::EqOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 45.");}
+//   BaseGDL* BaseGDL::NeOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 46.");}
+//   BaseGDL* BaseGDL::LeOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 47.");}
+//   BaseGDL* BaseGDL::GeOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 48.");}
+//   BaseGDL* BaseGDL::LtOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 49.");}
+//   BaseGDL* BaseGDL::GtOpNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 50.");}
+  BaseGDL* BaseGDL::AddNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 51.");}       // implemented
+  BaseGDL* BaseGDL::AddInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 52.");}       // implemented
+  BaseGDL* BaseGDL::SubNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 53.");}
+  BaseGDL* BaseGDL::SubInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 54.");}
+  BaseGDL* BaseGDL::LtMarkNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 55.");}
+  BaseGDL* BaseGDL::GtMarkNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 56.");}
+  BaseGDL* BaseGDL::MultNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 57.");}    // implemented
+  BaseGDL* BaseGDL::DivNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 58.");}
+  BaseGDL* BaseGDL::DivInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 59.");}
+  BaseGDL* BaseGDL::ModNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 60.");}
+  BaseGDL* BaseGDL::ModInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 61.");}
+  BaseGDL* BaseGDL::PowNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 62.");}
+  BaseGDL* BaseGDL::PowInvNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 63.");}
+  BaseGDL* BaseGDL::PowIntNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 64.");}    // implemented
+
+  BaseGDL* BaseGDL::AndOpSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 65.");}
+  BaseGDL* BaseGDL::AndOpInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 66.");}
+  BaseGDL* BaseGDL::OrOpSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 67.");}
+  BaseGDL* BaseGDL::OrOpInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 68.");}
+  BaseGDL* BaseGDL::XorOpSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 69.");}
+  BaseGDL* BaseGDL::AddSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 70.");}          // implemented
+  BaseGDL* BaseGDL::AddInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 71.");}     // implemented
+  BaseGDL* BaseGDL::SubSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 73.");}
+  BaseGDL* BaseGDL::SubInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 74.");}
+  BaseGDL* BaseGDL::LtMarkSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 75.");}
+  BaseGDL* BaseGDL::GtMarkSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 76.");}
+  BaseGDL* BaseGDL::MultSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 77.");}       // implemented
+  BaseGDL* BaseGDL::DivSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 78.");}
+  BaseGDL* BaseGDL::DivInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 79.");}
+  BaseGDL* BaseGDL::ModSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 80.");}
+  BaseGDL* BaseGDL::ModInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 81.");}
+  BaseGDL* BaseGDL::PowSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 82.");}
+  BaseGDL* BaseGDL::PowInvSNew( BaseGDL* r) { throw GDLException("Operation not defined for UNDEF 83.");}
+
+
 // --- SA: MemStats stuff
 
 SizeT MemStats::NumAlloc = 0;
