@@ -212,7 +212,7 @@ inline bool ConvertableType( DType t) // everything except Struct, Ptr, Obj
 
 class   BaseGDL;
 class   ArrayIndexListT;
-class   ExprListT;
+//class   ExprListT;
 
 struct SpDULong;
 template<class> class Data_;
@@ -311,6 +311,7 @@ public:
 };
 
 // ---
+class AllIxBaseT;
 
 class BaseGDL: private MemStats
 {
@@ -360,7 +361,7 @@ public:
 
   // provide access to dim member
   inline const dimension& Dim()   const      { return dim;}
-  inline SizeT    Dim(SizeT d)    const      { return dim[d];}
+   inline SizeT    Dim(SizeT d)    const      { return dim[d];}
 //   inline SizeT*   Dim0Address()              { return dim.Dim0Address();}
   inline SizeT    Stride(SizeT d) const      { return dim.Stride(d);}
   inline void     Purge()                    { dim.Purge();}
@@ -374,7 +375,11 @@ public:
     return i;
   }
 
+//  private:
  virtual BaseGDL& operator=(const BaseGDL& right);
+//  public:
+ 
+ virtual void InitFrom(const BaseGDL& right); // for structs
 
   // virtual functions
   virtual bool IsAssoc() const;
@@ -420,8 +425,9 @@ public:
   virtual const std::string& TypeStr() const;
   virtual bool          EqType( const BaseGDL*) const;
   virtual void* DataAddr();// SizeT elem=0);
-  virtual BaseGDL* New( const dimension& dim_, InitType noZero=ZERO);
-  virtual BaseGDL* Dup() const; 
+  virtual BaseGDL* New( const dimension& dim_, InitType noZero=ZERO) const;
+  virtual BaseGDL* NewResult() const;
+  virtual BaseGDL* Dup() const;
 //   virtual BaseGDL* Dup( char*) const; 
   virtual BaseGDL* Convert2( DType destTy, Convert2Mode mode=CONVERT);
   virtual BaseGDL* GetTag() const; 
@@ -431,7 +437,9 @@ public:
   virtual void     SetBufferSize( SizeT s);
   virtual int Scalar2index(SizeT& ret) const;
   virtual int Scalar2RangeT(RangeT& ret) const;
-  virtual SizeT LoopIndex() const; 
+  virtual SizeT GetAsIndex( SizeT i) const;
+  virtual SizeT GetAsIndexStrict( SizeT i) const;
+  virtual RangeT LoopIndex() const;
   virtual bool True();
   virtual bool False();
   virtual bool LogTrue();
@@ -460,7 +468,7 @@ public:
   // return a new type of itself
   virtual BaseGDL* NewIx( SizeT ix);
   virtual BaseGDL* NewIx( BaseGDL* ix, bool strict);
-  virtual BaseGDL* NewIx( AllIxT* ix, dimension* dIn);
+  virtual BaseGDL* NewIx( AllIxBaseT* ix, const dimension* dIn);
   virtual BaseGDL* NewIxFrom( SizeT s);
   virtual BaseGDL* NewIxFrom( SizeT s, SizeT e);
   virtual BaseGDL* NewIxFromStride( SizeT s, SizeT stride);
@@ -478,9 +486,11 @@ public:
   virtual BaseGDL* Log10();              
   virtual void     Log10This();
 
+  // defined in basic_op.cpp
   // used in r_expr
   virtual BaseGDL* UMinus();              
-  virtual BaseGDL* NotOp();               
+  virtual BaseGDL* NotOp();
+             
   virtual BaseGDL* AndOp( BaseGDL* r);    
   virtual BaseGDL* AndOpInv( BaseGDL* r); 
   virtual BaseGDL* OrOp( BaseGDL* r);    
@@ -494,20 +504,18 @@ public:
   virtual BaseGDL* GtOp( BaseGDL* r);    
   virtual BaseGDL* Add( BaseGDL* r);      
   virtual BaseGDL* AddInv( BaseGDL* r);      
-  virtual BaseGDL* Sub( BaseGDL* r);      
+  virtual BaseGDL* Sub( BaseGDL* r);
   virtual BaseGDL* SubInv( BaseGDL* r);   
   virtual BaseGDL* LtMark( BaseGDL* r);   
   virtual BaseGDL* GtMark( BaseGDL* r);   
   virtual BaseGDL* Mult( BaseGDL* r);   
-  virtual BaseGDL* Div( BaseGDL* r);      
+  virtual BaseGDL* Div( BaseGDL* r);
   virtual BaseGDL* DivInv( BaseGDL* r);   
   virtual BaseGDL* Mod( BaseGDL* r);      
   virtual BaseGDL* ModInv( BaseGDL* r);   
   virtual BaseGDL* Pow( BaseGDL* r);      
   virtual BaseGDL* PowInv( BaseGDL* r);   
-
-  virtual BaseGDL* PowInt( BaseGDL* r);      
-  virtual BaseGDL* PowIntNew( BaseGDL* r);   
+  virtual BaseGDL* PowInt( BaseGDL* r);
 
   virtual BaseGDL* AndOpS( BaseGDL* r);    
   virtual BaseGDL* AndOpInvS( BaseGDL* r); 
@@ -515,13 +523,13 @@ public:
   virtual BaseGDL* OrOpInvS( BaseGDL* r); 
   virtual BaseGDL* XorOpS( BaseGDL* r);    
   virtual BaseGDL* AddS( BaseGDL* r);      
-  virtual BaseGDL* AddInvS( BaseGDL* r);      
-  virtual BaseGDL* SubS( BaseGDL* r);      
+  virtual BaseGDL* AddInvS( BaseGDL* r);
+  virtual BaseGDL* SubS( BaseGDL* r);
   virtual BaseGDL* SubInvS( BaseGDL* r);   
   virtual BaseGDL* LtMarkS( BaseGDL* r);   
   virtual BaseGDL* GtMarkS( BaseGDL* r);   
   virtual BaseGDL* MultS( BaseGDL* r);   
-  virtual BaseGDL* DivS( BaseGDL* r);      
+  virtual BaseGDL* DivS( BaseGDL* r);
   virtual BaseGDL* DivInvS( BaseGDL* r);   
   virtual BaseGDL* ModS( BaseGDL* r);      
   virtual BaseGDL* ModInvS( BaseGDL* r);   
@@ -529,22 +537,61 @@ public:
   virtual BaseGDL* PowInvS( BaseGDL* r);   
 
 
-  virtual BaseGDL* AndOpNew( BaseGDL* r);    // create new result var      
-  //  virtual BaseGDL* AndOpInvNew( BaseGDL* r); // create new result var      
-  virtual BaseGDL* OrOpNew( BaseGDL* r);    
-  //  virtual BaseGDL* OrOpInvNew( BaseGDL* r); 
-  virtual BaseGDL* XorOpNew( BaseGDL* r);    
-  virtual BaseGDL* AddNew( BaseGDL* r);      
-  //  virtual BaseGDL* AddInvNew( BaseGDL* r);      
-  virtual BaseGDL* SubNew( BaseGDL* r);      
-  //  virtual BaseGDL* SubInvNew( BaseGDL* r);   
-  virtual BaseGDL* MultNew( BaseGDL* r);   
-  virtual BaseGDL* DivNew( BaseGDL* r);      
-  //  virtual BaseGDL* DivInvNew( BaseGDL* r);   
-  virtual BaseGDL* ModNew( BaseGDL* r);      
-  //  virtual BaseGDL* ModInvNew( BaseGDL* r);   
-  virtual BaseGDL* PowNew( BaseGDL* r);     
-  //  virtual BaseGDL* PowInvNew( BaseGDL* r);  
+  
+  
+  
+// the New functions
+// defined in basic_op_new.cpp
+// results go into a new DataT
+
+  virtual BaseGDL* AndOpNew( BaseGDL* r);
+  virtual BaseGDL* AndOpInvNew( BaseGDL* r);
+  virtual BaseGDL* OrOpNew( BaseGDL* r);
+  virtual BaseGDL* OrOpInvNew( BaseGDL* r);
+  virtual BaseGDL* XorOpNew( BaseGDL* r);
+//   virtual BaseGDL* EqOpNew( BaseGDL* r);
+//   virtual BaseGDL* NeOpNew( BaseGDL* r);
+//   virtual BaseGDL* LeOpNew( BaseGDL* r);
+//   virtual BaseGDL* GeOpNew( BaseGDL* r);
+//   virtual BaseGDL* LtOpNew( BaseGDL* r);
+//   virtual BaseGDL* GtOpNew( BaseGDL* r);
+  virtual BaseGDL* AddNew( BaseGDL* r);      // implemented
+  virtual BaseGDL* AddInvNew( BaseGDL* r);      // implemented
+  virtual BaseGDL* SubNew( BaseGDL* r);
+  virtual BaseGDL* SubInvNew( BaseGDL* r);
+  virtual BaseGDL* LtMarkNew( BaseGDL* r);
+  virtual BaseGDL* GtMarkNew( BaseGDL* r);
+  virtual BaseGDL* MultNew( BaseGDL* r);   // implemented
+  virtual BaseGDL* DivNew( BaseGDL* r);
+  virtual BaseGDL* DivInvNew( BaseGDL* r);
+  virtual BaseGDL* ModNew( BaseGDL* r);
+  virtual BaseGDL* ModInvNew( BaseGDL* r);
+  virtual BaseGDL* PowNew( BaseGDL* r);
+  virtual BaseGDL* PowInvNew( BaseGDL* r);
+  virtual BaseGDL* PowIntNew( BaseGDL* r);   // implemented
+
+  virtual BaseGDL* AndOpSNew( BaseGDL* r);
+  virtual BaseGDL* AndOpInvSNew( BaseGDL* r);
+  virtual BaseGDL* OrOpSNew( BaseGDL* r);
+  virtual BaseGDL* OrOpInvSNew( BaseGDL* r);
+  virtual BaseGDL* XorOpSNew( BaseGDL* r);
+  virtual BaseGDL* AddSNew( BaseGDL* r);         // implemented
+  virtual BaseGDL* AddInvSNew( BaseGDL* r);    // implemented
+  virtual BaseGDL* SubSNew( BaseGDL* r);
+  virtual BaseGDL* SubInvSNew( BaseGDL* r);
+  virtual BaseGDL* LtMarkSNew( BaseGDL* r);
+  virtual BaseGDL* GtMarkSNew( BaseGDL* r);
+  virtual BaseGDL* MultSNew( BaseGDL* r);      // implemented
+  virtual BaseGDL* DivSNew( BaseGDL* r);
+  virtual BaseGDL* DivInvSNew( BaseGDL* r);
+  virtual BaseGDL* ModSNew( BaseGDL* r);
+  virtual BaseGDL* ModInvSNew( BaseGDL* r);
+  virtual BaseGDL* PowSNew( BaseGDL* r);
+  virtual BaseGDL* PowInvSNew( BaseGDL* r);
+
+
+  
+  //  virtual BaseGDL* PowInvNew( BaseGDL* r);
   virtual BaseGDL* MatrixOp( BaseGDL* r, bool rtranspose = false, bool transposeResult =false, bool strassen = false);
   virtual void AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT offset);
   virtual void AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList);
@@ -575,28 +622,69 @@ public:
 #endif
 };
 
-const int ExprListDefaultLength = 128;
+// {
+// public:
+// typedef BaseGDL** iterator;
+// 
+// private:
+// BaseGDL** eArr;
+// BaseGDL* buf[ExprListDefaultLength];
+// SizeT sz;
+// SizeT actLen;
+// 
+// public:
+// ExprListT(): eArr(buf), sz(0) {}
+// ~ExprListT()
+// {
+// 	BaseGDL** pEnd = &eArr[sz];
+// 	for( BaseGDL** p = &eArr[0]; p!=pEnd;++p)
+// 		delete *p;
+// 	if( eArr != buf)
+// 		delete[] eArr;
+// }
+// void push_back( BaseGDL* p)
+// {
+// 	if( sz < ExprListDefaultLength)
+// 	{
+// 		eArr[ sz++] = p;
+// 		return;
+// 	}
+// 	if( sz == ExprListDefaultLength)
+// 		actLen =ExprListDefaultLength; // only init here
+// 	if( sz == actLen)
+// 	{
+// 		actLen *= 2;
+// 		BaseGDL** newArr = new BaseGDL*[ actLen];
+// 		for( SizeT i=0; i<sz; ++i)
+// 			newArr[i] = eArr[i];
+// 		if( eArr != buf)
+// 			delete[] eArr;
+// 		eArr = newArr;
+// 	}
+// 	eArr[ sz++] = p;
+// }
+// BaseGDL* operator[]( SizeT i) const { assert( i<sz);  return eArr[i];}
+// SizeT size() const { return sz;}
+// iterator begin()  { return &eArr[0];}
+// iterator end()  { return &eArr[sz];}
+// };
 
 // ExprListT deletes all members upon own destruction
-// class ExprListT: public std::list<BaseGDL*> 
-class ExprListT: public std::vector<BaseGDL*> 
-{
-public:
-ExprListT()
-{
-this->reserve(ExprListDefaultLength);
-}
-  ~ExprListT()
-  {
-    for( ExprListT::iterator i=this->begin(); i!=this->end(); ++i)
-      delete *i;
-  }
-};
-
-
-
-typedef ExprListT::iterator ExprListIterT;
 
+// old slow version (up to GDL 0.9.1):
+// class ExprListT: public std::vector<BaseGDL*>
+// {
+// public:
+// 	ExprListT()
+// 	{
+// 	this->reserve(ExprListDefaultLength);
+// 	}
+// 	~ExprListT()
+// 	{
+// 	for( ExprListT::iterator i=this->begin(); i!=this->end(); ++i)
+// 		delete *i;
+// 	}
+// };
 
 
 struct ForLoopInfoT
@@ -615,6 +703,24 @@ struct ForLoopInfoT
 		delete endLoopVar;
 		delete loopStepVar;
 	}
+	void Init()
+	{
+		endLoopVar = NULL;
+		loopStepVar = NULL;
+		foreachIx = -1;
+	}
+	void Clear()
+	{
+		delete endLoopVar;
+		delete loopStepVar;
+	}
+	void ClearInit()
+	{
+		delete endLoopVar;
+		delete loopStepVar;
+		endLoopVar = NULL;
+		loopStepVar = NULL;
+	}
 };
 
 #endif
