@@ -402,7 +402,7 @@ namespace lib {
 	  }
       }
 
-    // BYTE
+    // BYTEDComplexDblGDL
     if (e->KeywordSet(6) || type == BYTE) {
 
       return make_array_template< DByteGDL>( e, dimKey, value);
@@ -455,18 +455,25 @@ namespace lib {
       // STRING (added by MS 29.10.2005)
     } else if (e->KeywordSet(17) || type == STRING) {
 
+      if (!e->KeywordSet(indexix))
+		return make_array_template< DStringGDL>( e, dimKey, value);
+      
       // 'true' for ignoring /INDEX keyword 
-      BaseGDL* ret = make_array_template< DStringGDL>( e, dimKey, value, true);
-      if (e->KeywordSet(indexix))
-      {
-        for (DLong i = 0; i < ret->N_Elements(); ++i) 
-        {
-          char tmp[13];
-          assert(sprintf(tmp, "%12d", i) == 12);
-          (*static_cast<DStringGDL*>(ret))[i] = tmp;
-        }
-      }
-      return ret;
+	 // BaseGDL* ret = make_array_template< DStringGDL>( e, dimKey, value, true);
+//       if (e->KeywordSet(indexix))
+//       {
+
+	  BaseGDL* ret = make_array_template< DULongGDL>( e, dimKey, value); //, true);
+      return ret->Convert2( STRING);
+
+//         for (DLong i = 0; i < ret->N_Elements(); ++i)
+//         {
+//           char tmp[13];
+//           assert(sprintf(tmp, "%12d", i) == 12);
+//           (*static_cast<DStringGDL*>(ret))[i] = tmp;
+//         }
+//       }
+//       return ret;
 
       // PTR (added by SA 15.08.2009)
     } else if (e->KeywordSet(18) || type == PTR) {
@@ -544,7 +551,7 @@ namespace lib {
     } else arr( e, dim, 1);
 
 
-    if (dim.N_Elements() != nEl) 
+    if (dim.NDimElements() != nEl) 
       e->Throw( "New subscripts must not change the number of elements in " 
 		+ e->GetParString( 0));
 
@@ -646,7 +653,7 @@ namespace lib {
 	s = s.substr(4);  // Remove "res="
 
 	size_t left_paren = s.find_first_of("(");
-	subList.push_back( s.substr( 0, left_paren-1));
+	subList.push_back( s.substr( 0, left_paren));
       }
 
       sort( subList.begin(), subList.end());
@@ -666,7 +673,7 @@ namespace lib {
 	DString s = libProList[ i]->ToString();
 
 	size_t comma_brac = s.find_first_of(",[");
-	subList.push_back( s.substr( 0, comma_brac-1));
+	subList.push_back( s.substr( 0, comma_brac));
       }
 
       sort( subList.begin(), subList.end());
