@@ -37,7 +37,7 @@
 #pragma interface
 #endif
 
-const size_t multiAlloc = 100;
+const size_t multiAlloc = 256;
 
 template<class Sp>
 class Data_: public Sp
@@ -92,6 +92,9 @@ static	void operator delete( void *ptr);
   // assignment. 
   Data_& operator=(const BaseGDL& right);
 
+  // for structs
+  void InitFrom(const BaseGDL& right);
+
   // one dim array access (unchecked)
   inline Ty& operator[] (const SizeT d1) { return dd[d1];}
   inline const Ty& operator[] (const SizeT d1) const { return dd[d1];}
@@ -152,7 +155,11 @@ static	void operator delete( void *ptr);
   // used by the interpreter
   int Scalar2index( SizeT& st) const;
   int Scalar2RangeT( RangeT& st) const;
-  SizeT LoopIndex() const;
+  RangeT LoopIndex() const;
+
+  // used for indexing of arrays
+  SizeT GetAsIndex( SizeT i) const;
+  SizeT GetAsIndexStrict( SizeT i) const;
   
   // make a duplicate on the heap
   Data_* Dup() const;
@@ -171,7 +178,8 @@ static	void operator delete( void *ptr);
     s=dd[0];
     return true; }
 
-  Data_* New( const dimension& dim_, BaseGDL::InitType noZero=BaseGDL::ZERO);
+  Data_* New( const dimension& dim_, BaseGDL::InitType noZero=BaseGDL::ZERO) const;
+  Data_* NewResult() const;
 
   // convert *this to other 'destTy'
   BaseGDL* Convert2( DType destTy, 
@@ -235,6 +243,7 @@ static	void operator delete( void *ptr);
   Data_*   Log10();
   void     Log10This();
 
+  // operators
   BaseGDL* UMinus(); // UMinus for SpDString returns float
   Data_*   NotOp();
   Data_*   AndOp( BaseGDL* r);
@@ -244,11 +253,14 @@ static	void operator delete( void *ptr);
   Data_*   XorOp( BaseGDL* r);
   Data_*   Add( BaseGDL* r);
   Data_*   AddInv( BaseGDL* r);
+//   Data_*   AddNew( BaseGDL* r);
+//   Data_*   AddInvNew( BaseGDL* r);
   Data_*   Sub( BaseGDL* r);
   Data_*   SubInv( BaseGDL* r);
   Data_*   GtMark( BaseGDL* r);
   Data_*   LtMark( BaseGDL* r);
   Data_*   Mult( BaseGDL* r);
+//   Data_*   MultNew( BaseGDL* r);
   Data_*   Div( BaseGDL* r);
   Data_*   DivInv( BaseGDL* r);
   Data_*   Mod( BaseGDL* r);
@@ -256,9 +268,10 @@ static	void operator delete( void *ptr);
   Data_*   Pow( BaseGDL* r);
   Data_*   PowInv( BaseGDL* r);
   Data_*   PowInt( BaseGDL* r);      
-  Data_*   PowIntNew( BaseGDL* r);   
+//   Data_*   PowIntNew( BaseGDL* r);   
   Data_*   MatrixOp( BaseGDL* r, bool transpose, bool transposeResult, bool strassen);
 
+  // operators with scalar
   Data_*   AndOpS( BaseGDL* r);
   Data_*   AndOpInvS( BaseGDL* r);
   Data_*   OrOpS( BaseGDL* r);
@@ -266,11 +279,14 @@ static	void operator delete( void *ptr);
   Data_*   XorOpS( BaseGDL* r);
   Data_*   AddS( BaseGDL* r);
   Data_*   AddInvS( BaseGDL* r);
+//   Data_*   AddSNew( BaseGDL* r);
+//   Data_*   AddInvSNew( BaseGDL* r);
   Data_*   SubS( BaseGDL* r);
   Data_*   SubInvS( BaseGDL* r);
   Data_*   GtMarkS( BaseGDL* r);
   Data_*   LtMarkS( BaseGDL* r);
   Data_*   MultS( BaseGDL* r);
+//   Data_*   MultSNew( BaseGDL* r);
   Data_*   DivS( BaseGDL* r);
   Data_*   DivInvS( BaseGDL* r);
   Data_*   ModS( BaseGDL* r);
@@ -278,22 +294,53 @@ static	void operator delete( void *ptr);
   Data_*   PowS( BaseGDL* r);
   Data_*   PowInvS( BaseGDL* r);
   
-  Data_* AndOpNew( BaseGDL* r);    // create new result var      
-//   Data_* AndOpInvNew( BaseGDL* r); // create new result var      
-  Data_* OrOpNew( BaseGDL* r);    
-//   Data_* OrOpInvNew( BaseGDL* r); 
-  Data_* XorOpNew( BaseGDL* r);    
-  Data_* AddNew( BaseGDL* r);      
-//   Data_* AddInvNew( BaseGDL* r);      
-  Data_* SubNew( BaseGDL* r);      
-//   Data_* SubInvNew( BaseGDL* r);   
-  Data_* MultNew( BaseGDL* r);   
-  Data_* DivNew( BaseGDL* r);      
-//   Data_* DivInvNew( BaseGDL* r);   
-  Data_* ModNew( BaseGDL* r);      
-//   Data_* ModInvNew( BaseGDL* r);   
-  Data_* PowNew( BaseGDL* r);     
-//   Data_* PowInvNew( BaseGDL* r);  
+  // operators returning a new value
+  Data_* AndOpNew( BaseGDL* r);
+  Data_* AndOpInvNew( BaseGDL* r);
+  Data_* OrOpNew( BaseGDL* r);
+  Data_* OrOpInvNew( BaseGDL* r);
+  Data_* XorOpNew( BaseGDL* r);
+//   Data_* EqOpNew( BaseGDL* r);
+//   Data_* NeOpNew( BaseGDL* r);
+//   Data_* LeOpNew( BaseGDL* r);
+//   Data_* GeOpNew( BaseGDL* r);
+//   Data_* LtOpNew( BaseGDL* r);
+//   Data_* GtOpNew( BaseGDL* r);
+  Data_* AddNew( BaseGDL* r);      // implemented
+  Data_* AddInvNew( BaseGDL* r);      // implemented
+  Data_* SubNew( BaseGDL* r);
+  Data_* SubInvNew( BaseGDL* r);
+  Data_* LtMarkNew( BaseGDL* r);
+  Data_* GtMarkNew( BaseGDL* r);
+  Data_* MultNew( BaseGDL* r);   // implemented
+  Data_* DivNew( BaseGDL* r);
+  Data_* DivInvNew( BaseGDL* r);
+  Data_* ModNew( BaseGDL* r);
+  Data_* ModInvNew( BaseGDL* r);
+  Data_* PowNew( BaseGDL* r);
+  Data_* PowInvNew( BaseGDL* r);
+  Data_* PowIntNew( BaseGDL* r);   // implemented
+
+  // operators with scalar returning a new value
+  Data_* AndOpSNew( BaseGDL* r);
+  Data_* AndOpInvSNew( BaseGDL* r);
+  Data_* OrOpSNew( BaseGDL* r);
+  Data_* OrOpInvSNew( BaseGDL* r);
+  Data_* XorOpSNew( BaseGDL* r);
+  Data_* AddSNew( BaseGDL* r);         // implemented
+  Data_* AddInvSNew( BaseGDL* r);    // implemented
+  Data_* SubSNew( BaseGDL* r);
+  Data_* SubInvSNew( BaseGDL* r);
+  Data_* LtMarkSNew( BaseGDL* r);
+  Data_* GtMarkSNew( BaseGDL* r);
+  Data_* MultSNew( BaseGDL* r);      // implemented
+  Data_* DivSNew( BaseGDL* r);
+  Data_* DivInvSNew( BaseGDL* r);
+  Data_* ModSNew( BaseGDL* r);
+  Data_* ModInvSNew( BaseGDL* r);
+  Data_* PowSNew( BaseGDL* r);
+  Data_* PowInvSNew( BaseGDL* r);
+
 
   Data_<SpDByte>* EqOp( BaseGDL* r);
   Data_<SpDByte>* NeOp( BaseGDL* r);
@@ -330,7 +377,7 @@ static	void operator delete( void *ptr);
   // return a new type of itself
   Data_* NewIx( SizeT ix);
   Data_* NewIx( BaseGDL* ix, bool strict);
-  Data_* NewIx( AllIxT* ix, dimension* dIn);
+  Data_* NewIx( AllIxBaseT* ix, const dimension* dIn);
   Data_* NewIxFrom( SizeT s);
   Data_* NewIxFrom( SizeT s, SizeT e);
   Data_* NewIxFromStride( SizeT s, SizeT stride);
