@@ -312,7 +312,7 @@ void Data_<SpDPtr>::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT off
 	{
 	  SizeT nCp=ixList->N_Elements();
 	  
-	  AllIxT* allIx = ixList->BuildIx();
+	  AllIxBaseT* allIx = ixList->BuildIx();
 	  
 	  GDLInterpreter::AddRef( scalar, nCp);
 
@@ -376,7 +376,7 @@ void Data_<SpDPtr>::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT off
 		    throw GDLException("Array subscript must have same size as"
 				       " source expression.");
 		  
-		  AllIxT* allIx = ixList->BuildIx();
+		  AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -396,7 +396,7 @@ void Data_<SpDPtr>::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT off
 		    throw GDLException("Array subscript must have same size as"
 				       " source expression.");
 		  
-		  AllIxT* allIx = ixList->BuildIx();
+		  AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -448,7 +448,7 @@ void Data_<SpDObj>::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT off
 	{
 	  SizeT nCp=ixList->N_Elements();
 	  
-	  AllIxT* allIx = ixList->BuildIx();
+	  AllIxBaseT* allIx = ixList->BuildIx();
 	  
 	  GDLInterpreter::AddRefObj( scalar, nCp);
 
@@ -512,7 +512,7 @@ void Data_<SpDObj>::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT off
 		    throw GDLException("Array subscript must have same size as"
 				       " source expression.");
 		  
-		  AllIxT* allIx = ixList->BuildIx();
+		  AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -532,7 +532,7 @@ void Data_<SpDObj>::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT off
 		    throw GDLException("Array subscript must have same size as"
 				       " source expression.");
 		  
-		  AllIxT* allIx = ixList->BuildIx();
+		  AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -608,7 +608,7 @@ if( isScalar)
 	else
 	{
 	Ty scalar=(*src)[0];
-	AllIxT* allIx = ixList->BuildIx();
+	AllIxBaseT* allIx = ixList->BuildIx();
 	
 	GDLInterpreter::AddRef( scalar, nCp);
 
@@ -640,7 +640,7 @@ else
 		throw GDLException("Array subscript must have same size as"
 				" source expression.");
 
-	AllIxT* allIx = ixList->BuildIx();
+	AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -736,7 +736,7 @@ if( isScalar)
 	else
 	{
 	Ty scalar=(*src)[0];
-	AllIxT* allIx = ixList->BuildIx();
+	AllIxBaseT* allIx = ixList->BuildIx();
 	
 	GDLInterpreter::AddRefObj( scalar, nCp);
 
@@ -767,7 +767,7 @@ else
 		throw GDLException("Array subscript must have same size as"
 				" source expression.");
 
-	AllIxT* allIx = ixList->BuildIx();
+	AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -854,7 +854,7 @@ SizeT nCp=ixList->N_Elements();
 //  cout << "dim = " << this->dim << endl;
 
 //  DataT& res_dd = res->dd;
-AllIxT* allIx = ixList->BuildIx();
+AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -883,7 +883,7 @@ SizeT nCp=ixList->N_Elements();
 //  cout << "dim = " << this->dim << endl;
 
 //  DataT& res_dd = res->dd;
-AllIxT* allIx = ixList->BuildIx();
+AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -929,7 +929,7 @@ else
 	{
 	SizeT nCp=ixList->N_Elements();
 
-	AllIxT* allIx = ixList->BuildIx();
+	AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -1035,7 +1035,7 @@ else
 	{
 	SizeT nCp=ixList->N_Elements();
 
-	AllIxT* allIx = ixList->BuildIx();
+	AllIxBaseT* allIx = ixList->BuildIx();
 //#pragma omp parallel if (nCp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nCp))
 {
 //#pragma omp for
@@ -1186,7 +1186,7 @@ return new Data_( (*this)[ ix]);
 
 
 template<>
-Data_<SpDPtr>* Data_<SpDPtr>::NewIx( AllIxT* ix, dimension* dIn)
+Data_<SpDPtr>* Data_<SpDPtr>::NewIx( AllIxBaseT* ix, const dimension* dIn)
 {
 SizeT nCp = ix->size();
 Data_* res=Data_::New( *dIn, BaseGDL::NOZERO);
@@ -1203,7 +1203,7 @@ for( SizeT c=0; c<nCp; ++c)
 return res;
 }
 template<>
-Data_<SpDObj>* Data_<SpDObj>::NewIx( AllIxT* ix, dimension* dIn)
+Data_<SpDObj>* Data_<SpDObj>::NewIx( AllIxBaseT* ix, const dimension* dIn)
 {
 SizeT nCp = ix->size();
 Data_* res=Data_::New( *dIn, BaseGDL::NOZERO);
@@ -1350,20 +1350,6 @@ return res;
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #undef NEWIX_SIGNEDINT
 #undef NEWIX_UNSIGNEDINT
 #define NEWIX_UNSIGNEDINT \
@@ -1421,272 +1407,48 @@ for(; i < nElem; ++i)\
 template<>
 Data_<SpDPtr>* Data_<SpDPtr>::NewIx( BaseGDL* ix, bool strict)
 {
-SizeT nElem = ix->N_Elements();
+ 	
+ 	assert( ix->Type() != UNDEF);
 
-Data_* res = New( ix->Dim(), BaseGDL::NOZERO);
-auto_ptr<Data_> guard( res);
+// no type checking needed here: GetAsIndex() will fail with grace
+//     int typeCheck = DTypeOrder[ dType];
+// 	if( typeCheck >= 100)
+// 	  throw GDLException("Type "+ix->TypeStr()+" not allowed as subscript.");
+  
+	SizeT nElem = ix->N_Elements();
 
-SizeT upper = dd.size() - 1;
-Ty    upperVal = (*this)[ upper];
-Ty    zeroVal  = (*this)[ 0];
-switch( ix->Type())
-	{
-	case BYTE:
-	{
-	DByteGDL* src = static_cast<DByteGDL*>( ix);
+	Data_* res = New( ix->Dim(), BaseGDL::NOZERO);
+	auto_ptr<Data_> guard( res);
 
-	NEWIX_UNSIGNEDINT
-	}
-	case INT:
+	SizeT upper = dd.size() - 1;
+	Ty    upperVal = (*this)[ upper];
+	if( strict)
 	{
-	DIntGDL* src = static_cast<DIntGDL*>( ix);
-
-	NEWIX_SIGNEDINT
-	}
-	case UINT:
-	{
-	DUIntGDL* src = static_cast<DUIntGDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case LONG: // typical type (returned from WHERE)
-	{
-	DLongGDL* src = static_cast<DLongGDL*>( ix);
-
-	NEWIX_SIGNEDINT
-	}
-	case ULONG:
-	{
-	DULongGDL* src = static_cast<DULongGDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case LONG64:
-	{
-	DLong64GDL* src = static_cast<DLong64GDL*>( ix);
-
-	NEWIX_SIGNEDINT
-	}
-	case ULONG64:
-	{
-	DULong64GDL* src = static_cast<DULong64GDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case FLOAT:
-	{
-	DFloat maxF = upper; 
-	DFloatGDL* src = static_cast<DFloatGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( (*src)[i] <= 0.0)
+		for(SizeT i = 0 ; i < nElem; ++i)
 		{
-		Ty b = zeroVal;
-		GDLInterpreter::IncRef( b);
-
-		(*res)[i] = zeroVal;
-		if( (*src)[i] <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
+			SizeT actIx = ix->GetAsIndexStrict( i);
+			if( actIx > upper)
+				throw GDLException("Array used to subscript array "
+						"contains out of range (>) subscript (at index: "+i2s(i)+").");
+			(*res)[i]= (*this)[ actIx];
 		}
-	else if( (*src)[i] > maxF)
-		{
-		if( (*src)[i] >= (maxF + 1.0) && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		Ty b = upperVal;
-		GDLInterpreter::IncRef( b);
-		(*res)[i] = upperVal;
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<SizeT,float>((*src)[i])];
-		}
-	GDLInterpreter::IncRef( res);
-	return guard.release();
 	}
-	case DOUBLE:
+	else // not strict
 	{
-	DDouble maxF = upper; 
-	DDoubleGDL* src = static_cast<DDoubleGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( (*src)[i] <= 0.0)
+		for(SizeT i = 0 ; i < nElem; ++i)
 		{
-		(*res)[i] = zeroVal;
-		if( (*src)[i] <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
-		}
-	else if( (*src)[i] > maxF)
-		{
-		(*res)[i] = upperVal;
-		if( (*src)[i] >= (maxF + 1.0) && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<SizeT,double>((*src)[i])];
-		}
-	GDLInterpreter::IncRef( res);
-	return guard.release();
-	}
-	case STRING:
-	{
-	DStringGDL* src = static_cast<DStringGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	{
-		const char* cStart=(*src)[i].c_str();
-		char* cEnd;
-		long l=strtol(cStart,&cEnd,10);
-		if( cEnd == cStart)
-		{
-		Warning("Type conversion error: "
-			"Unable to convert given STRING to LONG.");
-		(*res)[i] = zeroVal;
-		}
-		else if( l < 0)
-		{
-		if( strict)
-		throw GDLException("Array used to subscript array "
-					"contains out of range (<0) subscript.");
-		(*res)[i] = zeroVal;
-		}
-		else if( l > upper)
-		{
-		if( strict)
-		throw GDLException("Array used to subscript array "
-					"contains out of range (>) subscript.");
-		(*res)[i] = upperVal;
-		}
-		else
-		{
-		(*res)[i] = (*this)[ l];
+			SizeT actIx = ix->GetAsIndex( i);
+			if( actIx >= upper)
+				(*res)[i] = upperVal;
+			else
+				(*res)[i]= (*this)[ actIx];
 		}
 	}
 	GDLInterpreter::IncRef( res);
 	return guard.release();
-	}
-	case COMPLEX:
-	{
-	DFloat maxF = upper; 
-	DComplexGDL* src = static_cast<DComplexGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( real((*src)[i]) <= 0.0)
-		{
-		if( real((*src)[i]) <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
-		(*res)[i] = zeroVal;
-		}
-	else if( real((*src)[i]) > upper)
-		{
-		if( real((*src)[i]) >= upper+1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		(*res)[i] = upperVal;
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<DLong,float>(real((*src)[i]))];
-		}
-	GDLInterpreter::IncRef( res);
-	return guard.release();
-	}
-	case COMPLEXDBL:
-	{
-	DDouble maxF = upper; 
-	DComplexDblGDL* src = static_cast<DComplexDblGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( real((*src)[i]) <= 0.0)
-		{
-		if( real((*src)[i]) <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
-		(*res)[i] = zeroVal;
-		}
-	else if( real((*src)[i]) > upper)
-		{
-		if( real((*src)[i]) >= upper+1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		(*res)[i] = upperVal;
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<DLong,float>(real((*src)[i]))];
-		}
-	GDLInterpreter::IncRef( res);
-	return guard.release();
-	}
-	default:
-	{
-	DType dType = ix->Type();
-	assert( dType != UNDEF);
-	
-	int typeCheck = DTypeOrder[ dType];
-	if( typeCheck >= 100)
-	throw GDLException("Type not allowed as subscript.");
-
-	assert( 0);
-	}
-	}
-assert( 0);
 }
 
-#undef NEWIX_SIGNEDINT
-#undef NEWIX_UNSIGNEDINT
 
-#define NEWIX_UNSIGNEDINT \
-SizeT i = 0;\
-for( ; i < nElem; ++i)\
-  if( (*src)[i] > upper)\
-    {\
-      if( strict)\
-	throw GDLException("Array used to subscript array "\
-			   "contains out of range (>) subscript.");\
-      (*res)[i++]= upperVal;\
-      break;\
-    }\
-  else\
-    (*res)[i]= (*this)[ (*src)[i]];\
-for(; i < nElem; ++i)\
-  if( (*src)[i] > upper)\
-    (*res)[i] = upperVal;\
-  else\
-    (*res)[i]= (*this)[ (*src)[i]];\
-GDLInterpreter::IncRefObj( res);\
-return guard.release();
-
-#define NEWIX_SIGNEDINT \
-  SizeT i = 0;\
-for(; i < nElem; ++i)\
-	  if( (*src)[i] < 0)\
-	    {\
-	      if( strict)\
-		throw GDLException("Array used to subscript array "\
-				   "contains out of range (<0) subscript.");\
-	      (*res)[i++]= zeroVal;\
-	      break;\
-	    }\
-	  else if( (*src)[i] > upper)\
-	    {\
-	      if( strict)\
-		throw GDLException("Array used to subscript array "\
-				   "contains out of range (>) subscript.");\
-	      (*res)[i++]= upperVal;\
-	      break;\
-	    }\
-	  else\
-	    (*res)[ i] = (*this)[ (*src)[ i]];\
-	for(; i < nElem; ++i)\
-	  if( (*src)[i] < 0)\
-	    (*res)[i]= zeroVal;\
-	  else if( (*src)[i] > upper)\
-	    (*res)[i]= upperVal;\
-	  else\
-	    (*res)[ i] = (*this)[ (*src)[ i]];\
-    GDLInterpreter::IncRefObj( res);\
-	return guard.release();
 
 template<>
 Data_<SpDObj>* Data_<SpDObj>::NewIx( BaseGDL* ix, bool strict)
@@ -1698,209 +1460,30 @@ auto_ptr<Data_> guard( res);
 
 SizeT upper = dd.size() - 1;
 Ty    upperVal = (*this)[ upper];
-Ty    zeroVal  = (*this)[ 0];
-switch( ix->Type())
+	if( strict)
 	{
-	case BYTE:
-	{
-	DByteGDL* src = static_cast<DByteGDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case INT:
-	{
-	DIntGDL* src = static_cast<DIntGDL*>( ix);
-
-	NEWIX_SIGNEDINT
-	}
-	case UINT:
-	{
-	DUIntGDL* src = static_cast<DUIntGDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case LONG: // typical type (returned from WHERE)
-	{
-	DLongGDL* src = static_cast<DLongGDL*>( ix);
-
-	NEWIX_SIGNEDINT
-	}
-	case ULONG:
-	{
-	DULongGDL* src = static_cast<DULongGDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case LONG64:
-	{
-	DLong64GDL* src = static_cast<DLong64GDL*>( ix);
-
-	NEWIX_SIGNEDINT
-	}
-	case ULONG64:
-	{
-	DULong64GDL* src = static_cast<DULong64GDL*>( ix);
-
-	NEWIX_UNSIGNEDINT
-	}
-	case FLOAT:
-	{
-	DFloat maxF = upper; 
-	DFloatGDL* src = static_cast<DFloatGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( (*src)[i] <= 0.0)
+		for(SizeT i = 0 ; i < nElem; ++i)
 		{
-		Ty b = zeroVal;
-		GDLInterpreter::IncRefObj( b);
-
-		(*res)[i] = zeroVal;
-		if( (*src)[i] <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
+			SizeT actIx = ix->GetAsIndexStrict( i);
+			if( actIx > upper)
+				throw GDLException("Array used to subscript array "
+						"contains out of range (>) subscript (at index: "+i2s(i)+").");
+			(*res)[i]= (*this)[ actIx];
 		}
-	else if( (*src)[i] > maxF)
+	}
+	else // not strict
+	{
+		for(SizeT i = 0 ; i < nElem; ++i)
 		{
-		if( (*src)[i] >= (maxF + 1.0) && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		Ty b = upperVal;
-		GDLInterpreter::IncRefObj( b);
-		(*res)[i] = upperVal;
+			SizeT actIx = ix->GetAsIndex( i);
+			if( actIx >= upper)
+				(*res)[i] = upperVal;
+			else
+				(*res)[i]= (*this)[ actIx];
 		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<SizeT,float>((*src)[i])];
-		}
-	GDLInterpreter::IncRefObj( res);
+	}
+    GDLInterpreter::IncRefObj( res);
 	return guard.release();
-	}
-	case DOUBLE:
-	{
-	DDouble maxF = upper; 
-	DDoubleGDL* src = static_cast<DDoubleGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( (*src)[i] <= 0.0)
-		{
-		(*res)[i] = zeroVal;
-		if( (*src)[i] <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
-		}
-	else if( (*src)[i] > maxF)
-		{
-		(*res)[i] = upperVal;
-		if( (*src)[i] >= (maxF + 1.0) && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<SizeT,double>((*src)[i])];
-		}
-	GDLInterpreter::IncRefObj( res);
-	return guard.release();
-	}
-	case STRING:
-	{
-	DStringGDL* src = static_cast<DStringGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	{
-		const char* cStart=(*src)[i].c_str();
-		char* cEnd;
-		long l=strtol(cStart,&cEnd,10);
-		if( cEnd == cStart)
-		{
-		Warning("Type conversion error: "
-			"Unable to convert given STRING to LONG.");
-		(*res)[i] = zeroVal;
-		}
-		else if( l < 0)
-		{
-		if( strict)
-		throw GDLException("Array used to subscript array "
-					"contains out of range (<0) subscript.");
-		(*res)[i] = zeroVal;
-		}
-		else if( l > upper)
-		{
-		if( strict)
-		throw GDLException("Array used to subscript array "
-					"contains out of range (>) subscript.");
-		(*res)[i] = upperVal;
-		}
-		else
-		{
-		(*res)[i] = (*this)[ l];
-		}
-	}
-	GDLInterpreter::IncRefObj( res);
-	return guard.release();
-	}
-	case COMPLEX:
-	{
-	DFloat maxF = upper; 
-	DComplexGDL* src = static_cast<DComplexGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( real((*src)[i]) <= 0.0)
-		{
-		if( real((*src)[i]) <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
-		(*res)[i] = zeroVal;
-		}
-	else if( real((*src)[i]) > upper)
-		{
-		if( real((*src)[i]) >= upper+1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		(*res)[i] = upperVal;
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<DLong,float>(real((*src)[i]))];
-		}
-	GDLInterpreter::IncRefObj( res);
-	return guard.release();
-	}
-	case COMPLEXDBL:
-	{
-	DDouble maxF = upper; 
-	DComplexDblGDL* src = static_cast<DComplexDblGDL*>( ix);
-	for( SizeT i=0; i < nElem; ++i)
-	if( real((*src)[i]) <= 0.0)
-		{
-		if( real((*src)[i]) <= -1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (<0) subscript.");
-		(*res)[i] = zeroVal;
-		}
-	else if( real((*src)[i]) > upper)
-		{
-		if( real((*src)[i]) >= upper+1.0 && strict)
-		throw GDLException("Array used to subscript array "
-				"contains out of range (>) subscript.");
-		(*res)[i] = upperVal;
-		}
-	else
-		{
-		(*res)[i]= (*this)[ Real2Int<DLong,float>(real((*src)[i]))];
-		}
-	GDLInterpreter::IncRefObj( res);
-	return guard.release();
-	}
-	default:
-	{
-	DType dType = ix->Type();
-	assert( dType != UNDEF);
-	
-	int typeCheck = DTypeOrder[ dType];
-	if( typeCheck >= 100)
-	throw GDLException("Type not allowed as subscript.");
-
-	assert( 0);
-	}
-	}
-assert( 0);
 }
 
 #endif
