@@ -105,7 +105,7 @@ DStructGDL::DStructGDL( const string& name_)
     { 
       SizeT nTags=NTags();
 
-      dd.resize( Desc()->NBytes());
+      dd.SetSize( Desc()->NBytes());
 
       typeVar.resize( nTags);
 
@@ -140,7 +140,7 @@ DStructGDL::DStructGDL(const DStructGDL& d_)
       ConstructTag( t);
 
       for( SizeT i=0; i < nEl; ++i)
-	*GetTag( t, i) = *d_.GetTag( t, i);
+		GetTag( t, i)->InitFrom( *d_.GetTag( t, i));
     }
 }
 
@@ -220,6 +220,7 @@ DStructGDL* DStructGDL::CShift( DLong d)
   return sh;
 }
 
+// TODO: Implement fast 2D version (see datatypes.cpp)
 DStructGDL* DStructGDL::CShift( DLong s[MAXRANK])
 {
   DStructGDL* sh = new DStructGDL( Desc(), dim, BaseGDL::NOZERO);
@@ -319,7 +320,7 @@ void DStructGDL::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList,
 	{
 	  SizeT nCp=ixList->N_Elements();
 
-	  AllIxT* allIx = ixList->BuildIx();
+	  AllIxBaseT* allIx = ixList->BuildIx();
 	  for( SizeT c=0; c<nCp; c++)
 	    {
 // 	      SizeT cTag=(*allIx)[ c]*nTags;
@@ -377,7 +378,7 @@ void DStructGDL::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList,
 		throw GDLException("Array subscript must have"
 				   " same size as source expression.");
 
-	      AllIxT* allIx = ixList->BuildIx();
+	      AllIxBaseT* allIx = ixList->BuildIx();
 	      for( SizeT c=0; c<nCp; c++)
 		{
 // 		  SizeT cTag= (*allIx)[ c]*nTags;
@@ -395,6 +396,42 @@ void DStructGDL::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList,
  	}
     }
 }
+void DStructGDL::AssignAtIx( RangeT ixR, BaseGDL* srcIn)
+{
+  if( srcIn->Type() != this->Type())
+    throw 
+      GDLException( "Conflicting data structures.");
+
+  DStructGDL* src=static_cast<DStructGDL*>(srcIn);
+  // check struct compatibility
+  if( src->Desc() != this->Desc() && (*src->Desc()) != (*this->Desc()))
+    throw 
+      GDLException( "Conflicting data structures.");
+
+  SizeT nTags=NTags();
+
+  if( ixR < 0)
+  {
+    SizeT nEl = this->N_Elements();
+  
+    if( -ixR > nEl)
+      throw GDLException("Subscript out of range: " + i2s(ixR));
+
+    SizeT ix = nEl + ixR;
+  
+    //(*this)[ix] = (*static_cast<Data_*>(srcIn))[0];
+    for( SizeT tagIx=0; tagIx<nTags; ++tagIx)
+      {
+	*GetTag( tagIx, ix) = *src->GetTag( tagIx);
+      }
+    return;
+  } // ixR >= 0
+  //(*this)[ixR] = (*static_cast<Data_*>(srcIn))[0];
+  for( SizeT tagIx=0; tagIx<nTags; ++tagIx)
+    {
+      *GetTag( tagIx, ixR) = *src->GetTag( tagIx);
+    }
+}
 void DStructGDL::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList) 
 {
   DStructGDL* src=static_cast<DStructGDL*>(srcIn);
@@ -428,7 +465,7 @@ void DStructGDL::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList)
 	{
 	  SizeT nCp=ixList->N_Elements();
 
-	  AllIxT* allIx = ixList->BuildIx();
+	  AllIxBaseT* allIx = ixList->BuildIx();
 	  for( SizeT c=0; c<nCp; c++)
 	    {
 // 	      SizeT cTag=(*allIx)[ c]*nTags;
@@ -479,7 +516,7 @@ void DStructGDL::AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList)
 		throw GDLException("Array subscript must have"
 				   " same size as source expression.");
 
-	      AllIxT* allIx = ixList->BuildIx();
+	      AllIxBaseT* allIx = ixList->BuildIx();
 	      for( SizeT c=0; c<nCp; c++)
 		{
 // 		  SizeT cTag= (*allIx)[ c]*nTags;
@@ -577,7 +614,7 @@ void DStructGDL::InsertAt( SizeT offset, BaseGDL* srcIn,
     {
       SizeT nCp=ixList->N_Elements();
       
-      AllIxT* allIx = ixList->BuildIx();
+      AllIxBaseT* allIx = ixList->BuildIx();
       for( SizeT c=0; c<nCp; c++)
 	{
 // 	  SizeT cTag=(c+offset)*nTags;
@@ -604,7 +641,7 @@ DStructGDL* DStructGDL::CatArray( ExprListT& exprList,
   dimension     catArrDim(dim); // list contains at least one element
 
   catArrDim.MakeRank( maxIx+1);
-  catArrDim.Set(catRankIx,0);     // clear rank which is added up
+  catArrDim.SetOneDim(catRankIx,0);     // clear rank which is added up
 
   SizeT dimSum=0;
   ExprListIterT i=exprList.begin();
@@ -627,7 +664,7 @@ DStructGDL* DStructGDL::CatArray( ExprListT& exprList,
 	}
     }
   
-  catArrDim.Set(catRankIx,dimSum);
+  catArrDim.SetOneDim(catRankIx,dimSum);
   
   // the concatenated array
   DStructGDL* catArr=New( catArrDim, BaseGDL::NOZERO);
@@ -654,7 +691,7 @@ DStructGDL* DStructGDL::Index( ArrayIndexListT* ixList)
   SizeT nTags=NTags();
   
   SizeT nCp=ixList->N_Elements();
-  AllIxT* allIx = ixList->BuildIx();
+  AllIxBaseT* allIx = ixList->BuildIx();
   for( SizeT c=0; c<nCp; c++)
     {
 //       SizeT cTag=c*nTags;
@@ -672,6 +709,16 @@ DStructGDL* DStructGDL::Index( ArrayIndexListT* ixList)
   return res;
 }
 
+DStructGDL* DStructGDL::NewIx( SizeT ix)
+{
+  SizeT nTags=NTags();
+  // bug ID 3091599: Indexed struct arrays retrun always an array
+  DStructGDL* res=New( dimension(1), BaseGDL::NOZERO);
+  for( SizeT tagIx=0; tagIx<nTags; tagIx++)
+    *res->GetTag( tagIx, 0) = *GetTag( tagIx, ix);
+  return res;
+}
+
 // void DStructGDL::AddTag(const BaseGDL* data)
 // {
 // //   SizeT oldSize = dd.size();
