@@ -73,7 +73,7 @@ public:
   DStructGDL( DStructDesc* desc_, const dimension& dim_)
     : SpDStruct( desc_, dim_)
     , typeVar( desc_->NTags())
-    , dd( dim.N_Elements() * desc_->NBytes(), false) //,SpDStruct::zero)
+    , dd( dim.NDimElements() * desc_->NBytes(), false) //,SpDStruct::zero)
   {
     dim.Purge();
     
@@ -90,7 +90,7 @@ public:
   DStructGDL( DStructDesc* desc_, const dimension& dim_, BaseGDL::InitType iT)
     : SpDStruct( desc_, dim_) 
     , typeVar( desc_->NTags())
-    , dd( (iT == BaseGDL::NOALLOC) ? 0 : dim.N_Elements() * desc_->NBytes(), 
+    , dd( (iT == BaseGDL::NOALLOC) ? 0 : dim.NDimElements() * desc_->NBytes(),
 	  false)
   {
     assert( iT == BaseGDL::NOZERO || iT == BaseGDL::NOALLOC);
@@ -118,7 +118,7 @@ public:
 
   // c-i (desc defined)
   // only called from Assoc_'s c-i
-DStructGDL(const DStructGDL& d_);
+  DStructGDL(const DStructGDL& d_);
 
   // For creating new structs (always scalar)
   DStructGDL( DStructDesc* desc_)
@@ -163,9 +163,27 @@ DStructGDL(const DStructGDL& d_);
 	*GetTag( t, e) = *right.GetTag( t, e);
       }
 
-    //    dd = right.dd;
     return *this;
   }
+  
+  void InitFrom(const BaseGDL& r)
+  {
+    assert( r.Type() == STRUCT);
+    const DStructGDL& right = static_cast<const DStructGDL&>( r);
+
+    assert( *Desc() == *right.Desc());
+    assert( &right != this);
+    
+    this->dim = right.dim;
+
+    SizeT nTags = NTags();
+    SizeT nEl   = N_Elements();
+    for( SizeT e=0; e < nEl; ++e)
+		for( SizeT t=0; t < nTags; ++t)
+		{
+			GetTag( t, e)->InitFrom( *right.GetTag( t, e));
+		}
+  }
  
   inline BaseGDL* operator[] (const SizeT d1) 
   { 
@@ -412,7 +430,7 @@ public:
     //       used but never defined  
     //
     //static_cast<DataGDL&>( *GetTag( tIx)) = data; // copy data
-    *GetTag( tIx) = data; // copy data
+    GetTag( tIx)->InitFrom( data); // copy data
   }
   
   // members
@@ -420,42 +438,49 @@ public:
   // throws (datatypes.cpp)
   int Scalar2index( SizeT& st) const;
   int Scalar2RangeT( RangeT& st) const;
-  
+  RangeT LoopIndex() const;
+
   bool Scalar() const 
   { return (N_Elements() == 1);}
   
   // make a duplicate on the heap
   DStructGDL* Dup() const { return new DStructGDL(*this);}
   
-  DStructGDL* New( const dimension& dim_,
-		   BaseGDL::InitType noZero=BaseGDL::ZERO)
-  {
-// No NOZERO for structs  
-    if( noZero == BaseGDL::NOZERO)
-      {
-	DStructGDL* res = new DStructGDL( Desc(), dim_, noZero);
+DStructGDL* New ( const dimension& dim_,
+                  BaseGDL::InitType noZero=BaseGDL::ZERO ) const
+{
+// No NOZERO for structs
+	if ( noZero == BaseGDL::NOZERO )
+	{
+		DStructGDL* res = new DStructGDL ( Desc(), dim_, noZero );
+		res->MakeOwnDesc();
+		return res;
+	}
+	if ( noZero == BaseGDL::INIT )
+	{
+		DStructGDL* res =  new DStructGDL ( Desc(), dim_, BaseGDL::NOZERO );
+		res->MakeOwnDesc();
+		SizeT nEl = res->N_Elements();
+		SizeT nTags = NTags();
+		for ( SizeT t=0; t<nTags; ++t )
+		{
+			const BaseGDL& cpTag = *GetTag ( t );
+			for ( SizeT i=0; i<nEl; ++i )
+				res->GetTag ( t, i )->InitFrom( cpTag);
+			// 	      res->dd[ i] = dd[ i % nTags]->Dup();
+		}
+		return res;
+	}
+	DStructGDL* res = new DStructGDL ( Desc(), dim_ );
 	res->MakeOwnDesc();
 	return res;
-      }
-    if( noZero == BaseGDL::INIT)
-      {
-	DStructGDL* res =  new DStructGDL( Desc(), dim_, BaseGDL::NOZERO);
+}
+DStructGDL* NewResult() const
+{
+	DStructGDL* res = new DStructGDL ( Desc(), this->dim, BaseGDL::NOZERO);
 	res->MakeOwnDesc();
-	SizeT nEl = res->N_Elements();
-	SizeT nTags = NTags();
-	for( SizeT t=0; t<nTags; ++t)
-	  {
-	    BaseGDL& cpTag = *GetTag( t);
-	    for( SizeT i=0; i<nEl; ++i) 
-	      *res->GetTag( t, i) = cpTag;
-	    // 	      res->dd[ i] = dd[ i % nTags]->Dup();
-	  }
 	return res;
-      }
-    DStructGDL* res = new DStructGDL( Desc(), dim_);
-    res->MakeOwnDesc();
-    return res;
-  }
+}
 
   // used by interpreter, calls CatInsert
   DStructGDL* CatArray( ExprListT& exprList, 
@@ -538,7 +563,6 @@ public:
   DStructGDL*   Pow( BaseGDL* r);
   DStructGDL*   PowInv( BaseGDL* r);
   DStructGDL*   PowInt( BaseGDL* r);
-  DStructGDL*   PowIntNew( BaseGDL* r);
   DStructGDL*   MatrixOp( BaseGDL* r,bool,bool,bool);
 
 
@@ -561,6 +585,55 @@ public:
   DStructGDL*   PowS( BaseGDL* r);
   DStructGDL*   PowInvS( BaseGDL* r);
 
+
+  // operators returning a new value
+  DStructGDL* AndOpNew( BaseGDL* r);
+  DStructGDL* AndOpInvNew( BaseGDL* r);
+  DStructGDL* OrOpNew( BaseGDL* r);
+  DStructGDL* OrOpInvNew( BaseGDL* r);
+  DStructGDL* XorOpNew( BaseGDL* r);
+//   DStructGDL* EqOpNew( BaseGDL* r);
+//   DStructGDL* NeOpNew( BaseGDL* r);
+//   DStructGDL* LeOpNew( BaseGDL* r);
+//   DStructGDL* GeOpNew( BaseGDL* r);
+//   DStructGDL* LtOpNew( BaseGDL* r);
+//   DStructGDL* GtOpNew( BaseGDL* r);
+  DStructGDL* AddNew( BaseGDL* r);      // implemented
+  DStructGDL* AddInvNew( BaseGDL* r);      // implemented
+  DStructGDL* SubNew( BaseGDL* r);
+  DStructGDL* SubInvNew( BaseGDL* r);
+  DStructGDL* LtMarkNew( BaseGDL* r);
+  DStructGDL* GtMarkNew( BaseGDL* r);
+  DStructGDL* MultNew( BaseGDL* r);   // implemented
+  DStructGDL* DivNew( BaseGDL* r);
+  DStructGDL* DivInvNew( BaseGDL* r);
+  DStructGDL* ModNew( BaseGDL* r);
+  DStructGDL* ModInvNew( BaseGDL* r);
+  DStructGDL* PowNew( BaseGDL* r);
+  DStructGDL* PowInvNew( BaseGDL* r);
+  DStructGDL* PowIntNew( BaseGDL* r);   // implemented
+
+  // operators with scalar returning a new value
+  DStructGDL* AndOpSNew( BaseGDL* r);
+  DStructGDL* AndOpInvSNew( BaseGDL* r);
+  DStructGDL* OrOpSNew( BaseGDL* r);
+  DStructGDL* OrOpInvSNew( BaseGDL* r);
+  DStructGDL* XorOpSNew( BaseGDL* r);
+  DStructGDL* AddSNew( BaseGDL* r);         // implemented
+  DStructGDL* AddInvSNew( BaseGDL* r);    // implemented
+  DStructGDL* SubSNew( BaseGDL* r);
+  DStructGDL* SubInvSNew( BaseGDL* r);
+  DStructGDL* LtMarkSNew( BaseGDL* r);
+  DStructGDL* GtMarkSNew( BaseGDL* r);
+  DStructGDL* MultSNew( BaseGDL* r);      // implemented
+  DStructGDL* DivSNew( BaseGDL* r);
+  DStructGDL* DivInvSNew( BaseGDL* r);
+  DStructGDL* ModSNew( BaseGDL* r);
+  DStructGDL* ModInvSNew( BaseGDL* r);
+  DStructGDL* PowSNew( BaseGDL* r);
+  DStructGDL* PowInvSNew( BaseGDL* r);
+
+
   
   Data_<SpDByte>* EqOp( BaseGDL* r);
   Data_<SpDByte>* NeOp( BaseGDL* r);
@@ -575,6 +648,7 @@ public:
   void AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT offset);
   void AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList);
   void AssignAt( BaseGDL* srcIn);
+  void AssignAtIx( RangeT ixR, BaseGDL* srcIn);
 
   void DecAt( ArrayIndexListT* ixList)
   {
@@ -600,6 +674,7 @@ public:
   
   // returns (*this)[ ixList]
   DStructGDL* Index( ArrayIndexListT* ixList);
+  DStructGDL* NewIx( SizeT ix);
 
   // formatting output functions
   
