@@ -31,17 +31,80 @@ using namespace std;
 // instance of static data
 DInterpreter* EnvBaseT::interpreter;
 
-EnvBaseT::ContainerT EnvBaseT::toDestroy;
+// EnvBaseT::ContainerT EnvBaseT::toDestroy;
+
+// EnvT::new & delete 
+deque< void*> EnvT::freeList;
+const int multiAllocEnvT = 4;
+void* EnvT::operator new( size_t bytes)
+{
+  assert( bytes == sizeof( EnvT));
+  if( freeList.size() > 0)
+  {
+    void* res = freeList.back();
+    freeList.pop_back();
+    return res;	
+  }
+//   cout << "*** Resize EnvT " << endl;
+  const size_t newSize = multiAllocEnvT - 1;
+  freeList.resize( newSize);
+  char* res = static_cast< char*>( malloc( sizeof( EnvT) * multiAllocEnvT)); // one more than newSize
+  for( size_t i=0; i<newSize; ++i)
+  {
+    freeList[ i] = res;
+    res += sizeof( EnvT);
+  } 
+  // the one more
+  return res;
+}
+void EnvT::operator delete( void *ptr)
+{
+freeList.push_back( ptr);
+}
+
+// EnvUDT::new & delete 
+deque< void*> EnvUDT::freeList;
+const int multiAllocEnvUDT = 16;
+void* EnvUDT::operator new( size_t bytes)
+{
+  assert( bytes == sizeof( EnvUDT));
+  if( freeList.size() > 0)
+  {
+    void* res = freeList.back();
+    freeList.pop_back();
+    return res;	
+  }
+//   cout << "*** Resize EnvUDT " << endl;
+  const size_t newSize = multiAllocEnvUDT - 1;
+  freeList.resize( newSize);
+  char* res = static_cast< char*>( malloc( sizeof( EnvUDT) * multiAllocEnvUDT)); // one more than newSize
+  for( size_t i=0; i<newSize; ++i)
+  {
+    freeList[ i] = res;
+    res += sizeof( EnvUDT);
+  } 
+  // the one more
+  return res;
+}
+void EnvUDT::operator delete( void *ptr)
+{
+freeList.push_back( ptr);
+}
+
+
+
+
 
 EnvBaseT::EnvBaseT( ProgNodeP cN, DSub* pro_): 
   env(), 
+  toDestroy(),
   pro(pro_),
   extra(NULL),
   newEnv(NULL), 
   callingNode( cN),
   lineNumber( 0),
   obj(false)
-, toDestroyInitialIndex( toDestroy.size())
+//, toDestroyInitialIndex( toDestroy.size())
 {}
 
 EnvUDT::EnvUDT( ProgNodeP cN, DSub* pro_, bool lF): 
@@ -56,7 +119,7 @@ EnvUDT::EnvUDT( ProgNodeP cN, DSub* pro_, bool lF):
 {
   DSubUD* proUD=static_cast<DSubUD*>(pro);
 
-  forLoopInfo.resize( proUD->NForLoops());
+  forLoopInfo.InitSize( proUD->NForLoops());
   
   SizeT envSize;
   SizeT keySize;
@@ -68,22 +131,26 @@ EnvUDT::EnvUDT( ProgNodeP cN, DSub* pro_, bool lF):
   parIx=keySize; // set to first parameter
 }
 
-EnvT::EnvT( ProgNodeP cN, DSub* pro_):
-  EnvBaseT( cN, pro_)
+EnvT::EnvT ( ProgNodeP cN, DSub* pro_):
+		EnvBaseT ( cN, pro_)
 {
-  SizeT envSize;
-  SizeT keySize;
-  keySize=pro->key.size();
-  if( pro->nPar > 0)
-    envSize=pro->nPar+keySize;
-  else
-    {
-      envSize=keySize;
-      // performance optimization
-      //env.reserve(envSize+5);
-    }
-  env.resize(envSize);
-  parIx=keySize; // set to first parameter
+// 	SizeT envSize;
+// 	SizeT keySize;
+	parIx=pro->key.size();
+	if ( pro->nPar > 0 )
+	{
+		env.resize ( pro->nPar + parIx);
+// 		envSize=pro->nPar+parIx;
+	}
+	else
+	{
+		env.resize ( parIx);
+// 		envSize=parIx;
+		// performance optimization
+		//env.reserve(envSize+5);
+	}
+// 	env.resize ( envSize);
+//   parIx=keySize; // set to first parameter
 }
 
 // member pro
@@ -125,15 +192,15 @@ EnvUDT::EnvUDT( ProgNodeP cN, BaseGDL* self,
 
   DSubUD* proUD=static_cast<DSubUD*>(pro);
 
-  forLoopInfo.resize( proUD->NForLoops());
+  forLoopInfo.InitSize( proUD->NForLoops());
 
   SizeT envSize;
-  SizeT keySize;
+//   SizeT keySize;
   envSize=proUD->var.size();
-  keySize=proUD->key.size();
+  parIx=proUD->key.size();
 
   env.resize(envSize);
-  parIx=keySize; // set to first parameter
+//   parIx=keySize; // set to first parameter
   // pass by value (self must not be changed)
   env.Set( parIx++, self); //static_cast<BaseGDL*>(oStructGDL));
 }
@@ -177,19 +244,19 @@ EnvUDT::EnvUDT( BaseGDL* self, //DStructGDL* oStructGDL,
 
   DSubUD* proUD=static_cast<DSubUD*>(pro);
 
-  forLoopInfo.resize( proUD->NForLoops());
+  forLoopInfo.InitSize( proUD->NForLoops());
 
-  SizeT envSize;
-  SizeT keySize;
-  envSize=proUD->var.size();
-  keySize=proUD->key.size();
+  SizeT envSize=proUD->var.size();
+  parIx=proUD->key.size();
 
   env.resize(envSize);
-  parIx=keySize; // set to first parameter
+//   parIx=keySize; // set to first parameter
   // pass by value (self must not be changed)
   env.Set( parIx++, self); //static_cast<BaseGDL*>(oStructGDL));
 }
 
+
+
 // for obj_new, obj_destroy, call_procedure and call_function
 EnvT::EnvT( EnvT* pEnv, DSub* newPro, BaseGDL** self):
   EnvBaseT( pEnv->callingNode, newPro)
@@ -197,22 +264,24 @@ EnvT::EnvT( EnvT* pEnv, DSub* newPro, BaseGDL** self):
   obj = (self != NULL);
 
   SizeT envSize;
-  SizeT keySize;
-  keySize=pro->key.size();
-  if( pro->nPar >= 0)
-    envSize=pro->nPar+pro->key.size();
+  parIx=pro->key.size();
+  if( pro->nPar > 0)
+	{
+		envSize=pro->nPar+parIx;
+	}
   else
-    {
-      envSize=keySize;
-      // performance optimization
-      //env.reserve(envSize+5);
-    }
+	{
+		envSize=parIx;
+	}
   env.resize(envSize);
-  parIx=keySize; // set to first parameter
+//   parIx=keySize; // set to first parameter
   // pass by reference (self must not be deleted)
   if( self != NULL)
     env.Set( parIx++, self); //static_cast<BaseGDL*>(oStructGDL));
 }
+
+
+
 EnvUDT::EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self):
   EnvBaseT( pEnv->CallingNode(), newPro),
   ioError(NULL), 
@@ -227,14 +296,14 @@ EnvUDT::EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self):
 
   DSubUD* proUD=static_cast<DSubUD*>(pro);
   
-  forLoopInfo.resize( proUD->NForLoops());
+  forLoopInfo.InitSize( proUD->NForLoops());
 
   SizeT envSize;
-  SizeT keySize;
+//   SizeT keySize;
   envSize=proUD->var.size();
-  keySize=proUD->key.size();
+  parIx=proUD->key.size();
   env.resize(envSize);
-  parIx=keySize; // set to first parameter
+//   parIx=keySize; // set to first parameter
   // pass by reference (self must not be deleted)
   if( self != NULL)
     env.Set( parIx++, self); //static_cast<BaseGDL*>(oStructGDL));
@@ -338,6 +407,8 @@ void EnvBaseT::AddEnv( DPtrListT& ptrAccessible, DPtrListT& objAccessible)
 }
 void EnvBaseT::AddToDestroy( DPtrListT& ptrAccessible, DPtrListT& objAccessible)
 {
+// 	if( toDestroy == NULL)
+// 		return;
     for( SizeT i=0; i<toDestroy.size(); ++i)
       {
          Add( ptrAccessible, objAccessible, toDestroy[i]);
@@ -1029,6 +1100,34 @@ const string EnvBaseT::GetString( SizeT ix)
     return subUD->GetVarName( ix);
   }
 
+// SA: used by STRING() for VMS-compat hack
+void EnvT::ShiftParNumbering(int n)
+{
+  assert(abs(n) == 1); // currently the code below works for n = +/- 1 only
+
+  SizeT nParam = NParam();
+  SizeT oParam = pro->key.size();
+
+  if (n == 1)
+  {
+    BaseGDL* tmp = env[oParam + nParam - 1];
+    for (int i = nParam - 1; i > 0; --i) 
+    {
+      env[oParam + i] = env[oParam + i - 1];
+    }
+    env[oParam] = tmp;
+  }
+  else if (n == -1)
+  {
+    BaseGDL* tmp = env[oParam];
+    for (int i = 0; i < nParam - 1; ++i) 
+    {
+      env[oParam + i] = env[oParam + i + 1];
+    }
+    env[oParam + nParam - 1] = tmp;
+  }
+}
+
 BaseGDL*& EnvBaseT::GetParDefined(SizeT i)
 {
   SizeT ix = i + pro->key.size();
@@ -1074,7 +1173,7 @@ SizeT EnvBaseT::NParam( SizeT minPar)
   SizeT nPar = parIx - pro->key.size();
   
   if( nPar < minPar)
-    Throw(	"Incorrect number of arguments.");
+    Throw( "Incorrect number of arguments.");
   return nPar;
 }
 SizeT EnvT::NParam( SizeT minPar)
@@ -1082,6 +1181,29 @@ SizeT EnvT::NParam( SizeT minPar)
   return EnvBaseT::NParam( minPar);
 }
 
+void EnvBaseT::SetNextParUnchecked( BaseGDL* const nextP) // by value (reset loc)
+{
+  if(!( static_cast<int>(parIx - pro->key.size()) < pro->nPar))
+  assert( static_cast<int>(parIx - pro->key.size()) < pro->nPar);
+  env.Set(parIx++,nextP); // check done in parameter_def
+}
+void EnvBaseT::SetNextParUncheckedVarNum( BaseGDL* const nextP) // by reference (reset env)
+{
+  AddEnv();
+  env.Set(parIx++,nextP);
+}
+
+void EnvBaseT::SetNextParUnchecked( BaseGDL** const nextP) // by reference (reset env)
+{
+  assert( static_cast<int>(parIx - pro->key.size()) < pro->nPar);
+  env.Set(parIx++,nextP);
+}
+void EnvBaseT::SetNextParUncheckedVarNum( BaseGDL** const nextP) // by reference (reset env)
+{
+  AddEnv();
+  env.Set(parIx++,nextP);
+}
+
 void EnvBaseT::SetNextPar( BaseGDL* const nextP) // by value (reset loc)
 {
   if( pro->nPar >= 0)
@@ -1098,7 +1220,6 @@ void EnvBaseT::SetNextPar( BaseGDL* const nextP) // by value (reset loc)
     }
   env.Set(parIx++,nextP);
 }
-
 void EnvBaseT::SetNextPar( BaseGDL** const nextP) // by reference (reset env)
 {
   if( pro->nPar >= 0)
@@ -1114,11 +1235,10 @@ void EnvBaseT::SetNextPar( BaseGDL** const nextP) // by reference (reset env)
       AddEnv();
     }
   env.Set(parIx++,nextP);
-
-  //  cout << "set: " << parIx-1 << " to "; (*nextP)->ToStream( cout); 
-  //  cout << "  " << nextP << endl;
 }
 
+
+
 // returns the keyword index, used for UD functions
 int EnvBaseT::GetKeywordIx( const std::string& k)
 {
