@@ -44,39 +44,45 @@ namespace lib {
 class EnvBaseT
 {
 private:
-  typedef std::deque<BaseGDL*> ContainerT;
+//   typedef std::deque<BaseGDL*> ContainerT;
+  typedef ExprListT ContainerT;
 
-  //protected:
-  // stores all data which has to deleted upon destruction
-  static ContainerT toDestroy;
-
-  SizeT toDestroyInitialIndex;
+//   SizeT toDestroyInitialIndex;
 
   EnvBaseT(){}
   
+protected:
+  // stores all data which has to deleted upon destruction
+  ContainerT toDestroy;
+   //   static ContainerT toDestroy;
+
 public:
-  // not anmore: DEPRECATED due to poor perfomance, this does not belong into an environment
   // needed to delete temporary ptr parameters only after subroutine completion
   // 'guards' a newly created variable which should be deleted
   // upon library routines exit (normal or on error)
-  // elimates the need of auto_ptr
+  // elimates the need of auto_ptr and in some places later destruction is needed
+  
   void Guard( BaseGDL* toGuard)
-    { toDestroy.push_back( toGuard);}
+    {
+// 		if( toDestroy == NULL)
+// 			toDestroy = new ContainerT();
+		toDestroy.push_back( toGuard);
+	}
 
 protected:
   // for obj cleanup
   static std::set< DObj> inProgress;
   
   static DInterpreter* interpreter;
-  DataListT            env;
+  DataListT         env;
   SizeT                parIx;     // ix of next parameter to put
-  DSub*                pro;
-  ProgNodeP            callingNode;
-  int			  lineNumber;
+  DSub*              pro;
+  ProgNodeP      callingNode;
+  int			          lineNumber;
   bool                 obj;       // member subroutine?
-  ExtraT*              extra;
+  ExtraT*            extra;
 
-	EnvBaseT* newEnv;
+  EnvBaseT*      newEnv;
 
   // finds the local variable pp points to
   int FindLocalKW( BaseGDL** pp) { return env.FindLocal( pp);}
@@ -112,12 +118,13 @@ public:
 
   virtual ~EnvBaseT()
   {
-  delete extra;
-    for( SizeT i=toDestroyInitialIndex; i<toDestroy.size(); ++i)
+    delete extra;
+// 	delete toDestroy; // cleans up its content
+/*    for( SizeT i=toDestroyInitialIndex; i<toDestroy.size(); ++i)
       {
-	delete toDestroy[i];
+		delete toDestroy[i];
       }
-    toDestroy.resize( toDestroyInitialIndex);
+    toDestroy.resize( toDestroyInitialIndex);*/
 //      for( ContainerT::iterator i=toDestroy.begin();
 //  	 i != toDestroy.end(); ++i) 
 //        delete *i;
@@ -183,13 +190,13 @@ proUD->SetCompileOpt( cOpt);
     return s;
   }
 
-  void DelEnv()
-  {
-    env.pop_back();
-  }
+   void DelEnv()
+   {
+     env.pop_back();
+   }
 
   // the upper (calling) environment
-  // a EnvT must have always a EnvUDT caller
+  // a EnvT must have always a /*EnvUDT*/ caller
   // i.e. library functions never call each other
   // with a new EnvT environment
   EnvBaseT* Caller();
@@ -197,10 +204,15 @@ proUD->SetCompileOpt( cOpt);
   // returns environment data, by value (but that by C++ reference)
   BaseGDL*& GetKW(SizeT ix) { return env[ix];}
 
-  // used by HELP
-  SizeT EnvSize() { return env.size();}
+  // used by HELP and SetNextPar(...)
+  SizeT EnvSize() const { return env.size();}
 
-  // next four are used by interpreter
+  // next four are used by Parameter...(...) functions
+  void SetNextParUnchecked( BaseGDL* const nextP); // by value (reset loc)
+  void SetNextParUnchecked( BaseGDL** const nextP); // by reference (reset env)
+  void SetNextParUncheckedVarNum( BaseGDL* const nextP); // by value (reset loc)
+  void SetNextParUncheckedVarNum( BaseGDL** const nextP); // by reference (reset env)
+  // these are used outside Parameter functions
   void SetNextPar( BaseGDL* const nextP); // by value (reset loc)
   void SetNextPar( BaseGDL** const nextP); // by reference (reset env)
   void SetKeyword( const std::string& k, BaseGDL* const val);  // value
@@ -257,16 +269,126 @@ proUD->SetCompileOpt( cOpt);
     if( ix >= env.size()) return false;
     return ( env.Env( ix) != NULL);
   }
+    void SetNextParUnckeckedVarNum(BaseGDL** arg1);
 
   friend class DInterpreter; // gcc 4.4 compatibility
 };
 
 
+// preallocates a buffer which should never be exceeded
+// but will work with any number of elements
+template< typename T, SizeT defaultLength> class ForInfoListT
+{
+public:
+typedef T* iterator;
+
+private:
+T* eArr;
+char buf[defaultLength * sizeof(T)]; // prevent constructor calls
+SizeT sz;
+
+public:
+
+ForInfoListT(): eArr( reinterpret_cast<T*>(buf)), sz( 0)
+{
+}
+
+~ForInfoListT()
+{
+	if( eArr != reinterpret_cast<T*>(buf))
+		delete[] eArr;
+	else
+	{
+		T* pEnd = &eArr[sz];
+		for( T* p = &eArr[0]; p!=pEnd; ++p)
+			p->Clear();
+	}
+}
+
+// must be called before access
+void InitSize( SizeT s)
+{
+    assert( sz == 0);
+    if( s == 0)
+	  return;
+    sz = s;
+    if( s < defaultLength)
+    {
+	    for( SizeT i=0; i<s; ++i)
+		    eArr[ i].Init();
+	    return;
+    }
+    eArr = new T[ s]; // constructor called
+}
+// only needed for EXECUTE
+void resize( SizeT s)
+{
+    if( s == sz)
+      return;
+    if( s < sz) // shrink
+    {
+	for( SizeT i=s; i<sz; ++i)
+		eArr[ i].ClearInit(); // in case eArr was allocated
+	sz = s;
+	return;
+    }
+    // s > sz -> grow
+    if( s <= defaultLength && eArr == reinterpret_cast<T*>(buf))
+    {
+		for( SizeT i=sz; i<s; ++i)
+			eArr[ i].Init();
+		sz = s;
+		return;
+    }
+    // this should never happen (or only in extreme rarely cases)
+    // hence the performance will go down
+    // s > defaultLength
+    T* newArr = new T[ s]; // ctor called
+	if( eArr != reinterpret_cast<T*>(buf))
+	{
+		for( SizeT i=0; i<sz; ++i)
+			{
+				newArr[i] = eArr[ i];
+				eArr[ i].Init(); // prevent dtor from freeing
+			}
+		delete[] eArr;
+	}
+    else
+    {
+		for( SizeT i=0; i<s; ++i)
+			{
+				newArr[i] = eArr[ i];
+			}
+    }
+    eArr = newArr;
+    sz = s;
+}
+// T operator[]( SizeT i) const { assert( i<sz);  return eArr[i];}
+T& operator[]( SizeT i) { assert( i<sz);  return eArr[i];}
+SizeT size() const { return sz;}
+iterator begin()  { return &eArr[0];}
+iterator end()  { return &eArr[sz];}
+bool empty() const { return sz == 0;}
+T& front() { return eArr[0];}
+const T& front() const { return eArr[0];}
+T& back() { return eArr[sz-1];}
+const T& back() const { return eArr[sz-1];}
+};
+
+
 
 // for UD subroutines (written in GDL) ********************************
 class EnvUDT: public EnvBaseT
 {
-  std::vector<ForLoopInfoT> forLoopInfo;
+static std::deque< void*> freeList;
+
+public:
+static 	void* operator new( size_t bytes);
+static	void operator delete( void *ptr);
+
+private:
+ForInfoListT<ForLoopInfoT, 32> forLoopInfo;
+// std::vector<ForLoopInfoT> forLoopInfo;
 
   ProgNodeP         ioError; 
   DLong             onError; // on_error setting
@@ -277,10 +399,10 @@ class EnvUDT: public EnvBaseT
   int               lastJump; // to which label last jump went
   
 public:
-	ForLoopInfoT& GetForLoopInfo( int forIx) { return forLoopInfo[forIx];}
-    
-    int NForLoops() const { return forLoopInfo.size();}
-    void ResizeForLoops( int newSize) { forLoopInfo.resize(newSize);}
+   ForLoopInfoT& GetForLoopInfo( int forIx) { return forLoopInfo[forIx];}
+   
+  int NForLoops() const { return forLoopInfo.size();}
+  void ResizeForLoops( int newSize) { forLoopInfo.resize(newSize);}
 
   // UD pro/fun
   EnvUDT( ProgNodeP idN, DSub* pro_, bool lF = false);
@@ -333,6 +455,12 @@ public:
 // this contains the library function API ***********************
 class EnvT: public EnvBaseT
 {
+static std::deque< void*> freeList;
+
+public:
+static 	void* operator new( size_t bytes);
+static	void operator delete( void *ptr);
+
   // Please use non library API (see below) function with caution
   // (most of them can be ignored by library function authors)
 
@@ -426,6 +554,8 @@ public:
     BaseGDL*& p0 = GetNumericParDefined( ix);
     if (p0->Rank() != 0) return p0;
     Throw("Expression must be an array in this context: "+GetParString(ix));
+    assert(false);
+    throw;
   }
 
   // get i'th parameter
@@ -445,7 +575,7 @@ public:
     T* res = dynamic_cast<T*>( p);
     if( res != NULL) return res;
     res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    toDestroy.push_back( res);
+    Guard( res);
     return res;
   }
   // same as before for keywords
@@ -458,7 +588,7 @@ public:
     T* res = dynamic_cast<T*>( p);
     if( res != NULL) return res;
     res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    toDestroy.push_back( res);
+    Guard( res);
     return res;
   }
 
@@ -471,7 +601,7 @@ public:
     T* res = dynamic_cast<T*>( p);
     if( res != NULL) return res;
     res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    toDestroy.push_back( res);
+    Guard( res);
     return res;
   }
   // same as before for keywords
@@ -483,7 +613,7 @@ public:
     T* res = dynamic_cast<T*>( p);
     if( res != NULL) return res;
     res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    toDestroy.push_back( res);
+    Guard( res);
     return res;
   }
 
@@ -536,17 +666,18 @@ public:
       }
     return false;
   }
-  void StealLocalParUndefGlobal( SizeT ix)
-  {
-    if( LocalKW( ix + pro->key.size()))
-      {
-	env.Clear( ix + pro->key.size());
-      }
-      else
-      {
-      env[ ix + pro->key.size()] = NULL;
-      }
-  }
+// removed: IDL does not undefine the global parameter with OVERWRITE
+//   void StealLocalParUndefGlobal( SizeT ix)
+//   {
+//     if( LocalKW( ix + pro->key.size()))
+//       {
+// 	env.Clear( ix + pro->key.size());
+//       }
+//       else
+//       {
+//       env[ ix + pro->key.size()] = NULL;
+//       }
+//   }
   bool GlobalPar( SizeT ix) { return GlobalKW( ix + pro->key.size());}
 
   // next two to set keywords/paramters
@@ -634,10 +765,13 @@ public:
   void AssureStringScalarPar( SizeT ix, DString& scalar);
 
   // to be extended on demand for other data types  
+
+  // SA: used by STRING() for VMS-compat hack
+  void ShiftParNumbering(int n);
 };
 
 
-	typedef std::deque<EnvBaseT*> EnvStackT;
+typedef std::deque<EnvBaseT*> EnvStackT;
 
 #endif
 
