@@ -73,6 +73,7 @@ namespace lib {
       fftw_execute(p);
 
       if (direct == -1) {
+TRACEOMP( __FILE__, __LINE__)
 #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
 #pragma omp for
@@ -103,6 +104,7 @@ namespace lib {
       fftwf_execute(p_f);
 
       if (direct == -1) {
+TRACEOMP( __FILE__, __LINE__)
 #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
 #pragma omp for
@@ -183,8 +185,8 @@ namespace lib {
       {
 	  if( overwrite)
 	  {
-//		e->StealLocalPar(0);
-		e->StealLocalParUndefGlobal(0);
+		e->StealLocalPar(0);
+//		e->StealLocalParUndefGlobal(0);
 		}
        p0C = (DComplexDblGDL *) p0;
 	 }
@@ -196,7 +198,8 @@ namespace lib {
 
       //      DComplexGDL* res;
 	  if( overwrite)
-		e->StealLocalParUndefGlobal(0);
+	  	e->StealLocalPar(0);
+// 		e->StealLocalParUndefGlobal(0);
 
       return fftw_template< DComplexGDL> (p0, nEl, dbl, overwrite, direct);
 
