@@ -676,7 +676,8 @@ namespace lib {
 
 	//cout << "if 1" << endl;
 	  if( overwrite)
-		e->StealLocalParUndefGlobal(0);
+	  	e->StealLocalPar(0); // only steals if local par
+// 		e->StealLocalParUndefGlobal(0);
     
       return fft_template< DComplexDblGDL> (p0, nEl, dbl, overwrite,
 					    direct, dimension);
@@ -688,7 +689,8 @@ namespace lib {
       DComplexGDL* res;
 
 	  if( overwrite)
-		e->StealLocalParUndefGlobal(0);
+	  	e->StealLocalPar(0); // only steals if local par
+// 		e->StealLocalParUndefGlobal(0);
     
       return fft_template< DComplexGDL> (p0, nEl, dbl, overwrite, 
 					 direct, dimension);
@@ -949,7 +951,7 @@ namespace lib {
 	    r = gsl_rng_alloc (gsl_rng_mt19937);
 	    gsl_rng_set (r, seed0);
 
-	    seed0 += dim.N_Elements() * seedMul; // avoid repetition in next call
+	    seed0 += dim.NDimElements() * seedMul; // avoid repetition in next call
 	    // if called with undefined global
 
 	    seed = new DLongGDL( seed0);
@@ -967,7 +969,7 @@ namespace lib {
 	    r = gsl_rng_alloc (gsl_rng_mt19937);
 	    gsl_rng_set (r, seed0);
 
-	    seed0 += dim.N_Elements() * seedMul; // avoid repetition in next call
+	    seed0 += dim.NDimElements() * seedMul; // avoid repetition in next call
 	    // which would be defined global if used in a loop
 	    
 	    seed = new DLongGDL( seed0);
@@ -982,7 +984,7 @@ namespace lib {
 	r = gsl_rng_alloc (gsl_rng_mt19937);
 	gsl_rng_set (r, seed0);
 
-	seed0 += dim.N_Elements() * seedMul; // avoid repetition in next call
+	seed0 += dim.NDimElements() * seedMul; // avoid repetition in next call
 	// if called with undefined global
       }
     
@@ -1379,26 +1381,32 @@ namespace lib {
 
   void interpolate_linear(gsl_interp_accel *acc, gsl_interp *interp, 
 			  double *xa, SizeT nx, const double ya[], 
-			  double x[], double y[])
+			  double x[], double y[], bool use_missing, double missing)
   {
     for( SizeT i=0; i<nx; ++i)
-      y[i] = gsl_interp_eval (interp, xa, ya, x[i], acc);
+    {
+      int status = gsl_interp_eval_e (interp, xa, ya, x[i], acc, &y[i]);
+      if (status == GSL_EDOM && use_missing) y[i] = missing;
+    }
   }
 
 
   void interpolate_cubic(gsl_interp_accel *acc, gsl_spline *spline, 
 			 double *xa, SizeT nx, const double ya[], 
-			 double x[], double y[])
+			 double x[], double y[], bool use_missing, double missing)
   {
     for( SizeT i=0; i<nx; ++i)
-      y[i] = gsl_spline_eval (spline, x[i], acc);
+    {
+      int status = gsl_spline_eval_e (spline, x[i], acc, &y[i]);
+      if (status == GSL_EDOM && use_missing) y[i] = missing;
+    }
   }
 
 
   void twoD_lin_interpolate(SizeT ninterp,
 			    double *xa, bool grid, 
 			    SizeT nx, SizeT ny, SizeT nxa, SizeT nya,
-			    double *p0, double *p1, double *p2, double *res)  
+			    double *p0, double *p1, double *p2, double *res, bool use_missing, double missing)  
   {
     gsl_interp_accel *acc = gsl_interp_accel_alloc ();
     gsl_interp *interp;
@@ -1441,14 +1449,14 @@ namespace lib {
 	  if (row >= nya) row = nya - 1;
 
 	  interpolate_linear(acc, interp, xa, nx, 
-			     ya[row], dptr, &work[0][0]);
+			     ya[row], dptr, &work[0][0], use_missing, missing);
 
 	  // Correct if row out of bounds
 	  if (row < -1) row = -1;
 	  if (row >= nya-1) row = nya - 2;
 
 	  interpolate_linear(acc, interp, xa, nx, 
-			     ya[(row+1)], dptr, &work[1][0]);
+			     ya[(row+1)], dptr, &work[1][0], use_missing, missing);
 	}
 	first = false;
 	lastrow = row;
@@ -1507,11 +1515,16 @@ namespace lib {
     if( p0->Rank() < nParam-1)
       e->Throw("Number of parameters must agree with dimensions of argument.");
 
-    bool cubic = false;
-    if ( e->KeywordSet(0)) cubic = true;
+    static int cubicIx = e->KeywordIx("CUBIC");
+    bool cubic = e->KeywordSet(cubicIx);
 
-    bool grid = false;
-    if ( e->KeywordSet(1)) grid = true;
+    static int gridIx = e->KeywordIx("GRID");
+    bool grid = e->KeywordSet(gridIx);
+
+    static int missingIx = e->KeywordIx("MISSING");
+    bool use_missing = e->KeywordSet(missingIx);
+    DDouble missing;
+    e->AssureDoubleScalarKWIfPresent(missingIx, missing);
 
     if ( nParam == 3)
       if (p1->N_Elements() == 1 && p2->N_Elements() == 1)
@@ -1623,11 +1636,11 @@ namespace lib {
 	if( cubic)
 	  // cubic interpolation
 	  interpolate_cubic(acc, spline, xa, p1D->N_Elements(), 
-			    &(*p0D)[0], &(*p1D)[0], &(*res)[0]);
+			    &(*p0D)[0], &(*p1D)[0], &(*res)[0], use_missing, missing);
 	else
 	  // linear interpolation
 	  interpolate_linear(acc, interp, xa, p1D->N_Elements(), 
-			     &(*p0D)[0], &(*p1D)[0], &(*res)[0]);
+			     &(*p0D)[0], &(*p1D)[0], &(*res)[0], use_missing, missing);
       } else {
 	// Multiple Interpolation
 	for( SizeT i=0; i<ninterp; ++i) {
@@ -1640,11 +1653,11 @@ namespace lib {
 	  if( cubic)
 	    // cubic interpolation
 	    interpolate_cubic(acc, spline, xa, p1D->N_Elements(), 
-			      ya, &(*p1D)[0], &(*res)[0]);
+			      ya, &(*p1D)[0], &(*res)[0], use_missing, missing);
 	  else
 	    // linear interpolation
 	    interpolate_linear(acc, interp, xa, p1D->N_Elements(), 
-			       ya, &(*p1D)[0], y);
+			       ya, &(*p1D)[0], y, use_missing, missing);
 
 	  // Write to output array
 	  for( SizeT j=0; j<p1D->N_Elements(); ++j) (*res)[j*ninterp+i] = y[j];
@@ -1706,7 +1719,7 @@ namespace lib {
       twoD_lin_interpolate(ninterp,
 			   xa, grid, 
 			   nx, ny, nxa, nya,
-			   &(*p0D)[0], &(*p1D)[0], &(*p2D)[0], &(*res)[0]);
+			   &(*p0D)[0], &(*p1D)[0], &(*p2D)[0], &(*res)[0], use_missing, missing);
       
       delete [] xa;
 
@@ -1773,7 +1786,7 @@ namespace lib {
 			     xa, grid, 
 			     nx, ny, nxa, nya,
 			     &(*p0D)[nxa*nya*i], &(*p1D)[0], &(*p2D)[0], 
-			     &work_xy[i*nx*ny]);
+			     &work_xy[i*nx*ny], use_missing, missing);
       }
 
       double *za = new double[nza];
@@ -1794,7 +1807,7 @@ namespace lib {
 	    for( SizeT k=0; k<nza; ++k) 
 	      work_za[k] = work_xy[k*nx*ny + j*nx + i];
 	    interpolate_linear(acc, interp_z, za, nz, work_za, 
-			       &(*p3D)[0], work_z);
+			       &(*p3D)[0], work_z, use_missing, missing);
 
 	    for( SizeT k=0; k<nz; ++k) (*res)[k*nx*ny + j*nx + i] = work_z[k];
 	  }
@@ -1803,7 +1816,7 @@ namespace lib {
 	for( SizeT i=0; i<nz; ++i) {
 	  for( SizeT k=0; k<nza; ++k) work_za[k] = work_xy[k*ny + i];
 	  interpolate_linear(acc, interp_z, za, 1, work_za, 
-			     &(*p3D)[i], &(*res)[i]);
+			     &(*p3D)[i], &(*res)[i], use_missing, missing);
 	}
       }
 
@@ -2491,7 +2504,7 @@ res_guard.reset (dres);
     {
       res = new DDoubleGDL(GSL_CONST_MKSA_GRAVITATIONAL_CONSTANT);
 #ifdef USE_UDUNITS
-      if (twoparams) unit_from = ut_parse(unitsys, "N m2 kg2", UT_ASCII);
+      if (twoparams) unit_from = ut_parse(unitsys, "N m2 kg-2", UT_ASCII);
 #endif
     }
     else if (name.compare("hbar") == 0) 
@@ -2794,15 +2807,16 @@ res_guard.reset (dres);
     gsl_poly_complex_workspace* w = gsl_poly_complex_workspace_alloc(coef->N_Elements());
     gsl_poly_complex_workspace_guard w_guard(w);
 
-    vector<double> tmp(2 * (coef->N_Elements() - 1)); 
+    SizeT resultSize = coef->N_Elements()-1;
+    vector<double> tmp(2 * resultSize);
     
     if (GSL_SUCCESS != gsl_poly_complex_solve(
       &(*coef)[0], coef->N_Elements(), w, &(tmp[0]))
     )
       e->Throw("Failed to compute the roots of the polynomial");
 
-    DComplexDblGDL* result = new DComplexDblGDL(dimension(coef->N_Elements() - 1));
-    for (SizeT i = 0; i < coef->N_Elements(); ++i) 
+    DComplexDblGDL* result = new DComplexDblGDL(dimension(resultSize), BaseGDL::NOZERO);
+    for (SizeT i = 0; i < resultSize; ++i) 
       (*result)[i] = complex<double>(tmp[2 * i], tmp[2 * i + 1]);
     
     return result->Convert2(
