@@ -127,9 +127,14 @@ namespace lib {
     
     DString h5fFilename;
     e->AssureScalarPar<DStringGDL>( 0, h5fFilename);
+    WordExp( h5fFilename);
     
     h5f_id = H5Fopen(h5fFilename.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
-    if (h5f_id < 0) { string msg; e->Throw(hdf5_error_message(msg)); }
+    if (h5f_id < 0) 
+    { 
+      string msg; 
+      e->Throw(hdf5_error_message(msg)); 
+    }
     
     return new DLongGDL( h5f_id );
   }
