@@ -185,6 +185,13 @@ namespace SysVar
     return static_cast<DLongGDL*>( pStruct->GetTag( tag, 0));
   }
 
+  DLong GetPFont()
+  {
+    static DStructGDL* pStruct = SysVar::P();
+    static int tag = pStruct->Desc()->TagIndex( "FONT");
+    return (*static_cast<DLongGDL*>( pStruct->GetTag( tag)))[0];
+  }
+
   DStructGDL* X()
   {
     DVar& var = *sysVarList[ xIx];
@@ -335,7 +342,11 @@ namespace SysVar
     plt->NewTag("REGION", new DFloatGDL( dimension( &regionDim, one))); 
     plt->NewTag("SUBTITLE", new DStringGDL( "")); 
     plt->NewTag("SYMSIZE", new DFloatGDL( 0.0)); 
-    plt->NewTag("T", new DDoubleGDL( dimension( tDim, 2))); 
+    {
+      DDoubleGDL* tmp = new DDoubleGDL( dimension( tDim, 2));
+      (*tmp)[0] = (*tmp)[5] = (*tmp)[10] = (*tmp)[15] = 1;
+      plt->NewTag("T", tmp); 
+    }
     plt->NewTag("T3D", new DLongGDL( 0)); 
     plt->NewTag("THICK", new DFloatGDL( 0.0)); 
     plt->NewTag("TITLE", new DStringGDL( "")); 
@@ -355,6 +366,48 @@ namespace SysVar
     // !GDL (to allow distinguish IDL/GDL with DEFSYSV, '!gdl', exists=exists )
     DStructGDL*  gdlStruct = new DStructGDL( "!GNUDATALANGUAGE");
     gdlStruct->NewTag("RELEASE", new DStringGDL( VERSION));
+
+    // creating and Epoch entry in order to have a simple incremental number 
+    string MyDate= __DATE__;
+    int CompilationMonth =0, CompilationYear=0, CompilationDay=0;
+    string SCompilationYear;
+    SCompilationYear=MyDate.substr(7,4);
+    CompilationYear=atoi(SCompilationYear.c_str());
+    string SCompilationDay;
+    SCompilationDay=MyDate.substr(4,2);
+    CompilationDay=atoi(SCompilationDay.c_str());
+
+    // for the months, it is more difficult
+    if (MyDate.find("Jan")!=string::npos) CompilationMonth=1;
+    if (MyDate.find("Feb")!=string::npos) CompilationMonth=2;
+    if (MyDate.find("Mar")!=string::npos) CompilationMonth=3;
+    if (MyDate.find("Apr")!=string::npos) CompilationMonth=4;
+    if (MyDate.find("May")!=string::npos) CompilationMonth=5;
+    if (MyDate.find("Jun")!=string::npos) CompilationMonth=6;
+    if (MyDate.find("Jul")!=string::npos) CompilationMonth=7;
+    if (MyDate.find("Aug")!=string::npos) CompilationMonth=8;
+    if (MyDate.find("Sep")!=string::npos) CompilationMonth=9;
+    if (MyDate.find("Oct")!=string::npos) CompilationMonth=10;
+    if (MyDate.find("Nov")!=string::npos) CompilationMonth=11;
+    if (MyDate.find("Dec")!=string::npos) CompilationMonth=12;
+		     
+    //cout << SCompilationYear << " "<< CompilationMonth <<endl;
+    //cout << CompilationYear<< endl;
+    struct tm t;
+    time_t t_of_day;
+    t.tm_year = CompilationYear -1900;
+    t.tm_mon = CompilationMonth-1;           // Month, 0 - jan
+    t.tm_mday = CompilationDay;          // Day of the month
+    t.tm_hour = 0;     
+    t.tm_min = 0;
+    t.tm_sec = 0;
+    t.tm_isdst = -1;        // Is DST on? 1 = yes, 0 = no, -1 = unknown
+    t_of_day = mktime(&t);
+
+    // printing Epoch on the Command Line $   date +"%s"
+    // printf("seconds since the Epoch: %ld\n", (long) t_of_day);
+
+    gdlStruct->NewTag("EPOCH", new DLongGDL((long) t_of_day));
     DVar *gdl        = new DVar( "GDL", gdlStruct);
     sysVarList.push_back(gdl);
     sysVarRdOnlyList.push_back( gdl); // make it read only
@@ -434,9 +487,14 @@ namespace SysVar
     struct utsname uts;
     uname(&uts);
     ver->NewTag("ARCH", new DStringGDL( uts.machine)); 
-    ver->NewTag("OS", new DStringGDL( uts.sysname)); 
+
+    const char *SysName=uts.sysname;
+    if (strcmp(SysName,"Linux") ==0) SysName="linux";
+    if (strcmp(SysName,"Darwin") ==0) SysName="darwin";
+
+    ver->NewTag("OS", new DStringGDL(SysName));    
     ver->NewTag("OS_FAMILY", new DStringGDL( "unix")); 
-    ver->NewTag("OS_NAME", new DStringGDL( uts.sysname)); 
+    ver->NewTag("OS_NAME", new DStringGDL(SysName)); 
     ver->NewTag("RELEASE", new DStringGDL( "6.0")); 
     ver->NewTag("BUILD_DATE", new DStringGDL(__DATE__)); 
     ver->NewTag("MEMORY_BITS", new DIntGDL( sizeof(BaseGDL*)*8)); 
