@@ -37,6 +37,12 @@
 
 #include "gdlexception.hpp"
 
+// GGH ggh hack to implement SPAWN keyword UNIT
+#ifdef HAVE_EXT_STDIO_FILEBUF_H
+#  include <ext/stdio_filebuf.h>
+#endif
+//#include <bits/basic_ios.h>
+
 
 // the file IO system consists of 128 GDLStream objects
 
@@ -49,10 +55,12 @@ const SizeT defaultStreamWidth = 80; // used by open_lun
 
 class AnyStream
 {
+// GGH ggh made all these public
+public:
   std::fstream* fStream;
   igzstream* igzStream; // for gzip compressed input
   ogzstream* ogzStream; // for gzip compressed output
-public:
+//public:
   AnyStream()
     : fStream(NULL) 
     , igzStream(NULL) 
@@ -105,12 +113,17 @@ public:
   {
     if( fStream != NULL) 
       return fStream->good();
-    if( igzStream != NULL && ogzStream != NULL) 
+    else if( igzStream != NULL && ogzStream != NULL) 
       return igzStream->good() && ogzStream->good();
-    if( igzStream != NULL) 
+    else if( igzStream != NULL) 
       return igzStream->good();
-    if( ogzStream != NULL) 
+    else if( ogzStream != NULL) 
       return ogzStream->good();
+    else
+    {
+      assert(false);
+      throw; // getting rid of compiler warning
+    }
   }
 
   bool EofRaw()
@@ -203,21 +216,27 @@ public:
 	    ogzStream->rdbuf()->pubseekpos( cur, std::ios_base::out);
 	    return end;
 	  }
+        else
+          {
+            assert(false);
+            throw; // getting rid of compiler warning
+          }
       }
   }
 
   std::streampos Tell()
   {
-    if( fStream != NULL)
+    if (fStream != NULL)
       return( fStream->tellg());
-    if( igzStream != NULL)
-      {
-	return igzStream->rdbuf()->pubseekoff( 0, std::ios_base::cur);
-      }
-    if( ogzStream != NULL)
-      {
-	return ogzStream->rdbuf()->pubseekoff( 0, std::ios_base::cur);
-      }
+    else if(igzStream != NULL)
+      return igzStream->rdbuf()->pubseekoff( 0, std::ios_base::cur);
+    else if( ogzStream != NULL)
+      return ogzStream->rdbuf()->pubseekoff( 0, std::ios_base::cur);
+    else
+    {
+      assert(false);
+      throw; // getting rid of compiler warning
+    }
   }
 
   void SeekPad( std::streampos pos)
@@ -286,6 +305,13 @@ class GDLStream
   DDouble r_timeout;
   DDouble w_timeout;
 
+#ifdef HAVE_EXT_STDIO_FILEBUF_H
+// GGH ggh hack to implement SPAWN keyword UNIT
+  __gnu_cxx::stdio_filebuf<char> *readbuf_frb_destroy_on_close_p;
+  std::basic_streambuf<char> *readbuf_bsrb_destroy_on_close_p;
+  int fd_close_on_close;
+#endif
+
   SizeT width;
 
   std::streampos lastSeekPos;
@@ -322,7 +348,13 @@ public:
     lastSeekPos( 0),
     lastRecord( 0),
     lastRecordStart( 0)
-  {}
+    {
+#ifdef HAVE_EXT_STDIO_FILEBUF_H
+      readbuf_frb_destroy_on_close_p = NULL;
+      readbuf_bsrb_destroy_on_close_p = NULL;
+      fd_close_on_close = -1;
+#endif
+    }
 
   ~GDLStream() 
   {
@@ -431,6 +463,17 @@ public:
 
   DULong F77ReadStart();
   void   F77ReadEnd();
+
+
+#ifdef HAVE_EXT_STDIO_FILEBUF_H
+// GGH ggh hack to implement SPAWN keyword UNIT
+  std::basic_streambuf<char> *get_stream_readbuf_bsrb();
+  int set_stream_readbuf_bsrb_from_frb(__gnu_cxx::stdio_filebuf<char> *frb_p);
+  int set_readbuf_frb_destroy_on_close(__gnu_cxx::stdio_filebuf<char> *frb_p);
+  int set_readbuf_bsrb_destroy_on_close(std::basic_streambuf<char> *bsrb_p);
+  int set_fd_close_on_close(int fd);
+#endif
+
 };
 
 
