@@ -42,6 +42,7 @@
 #endif
 
 #include "grib.hpp"
+#include "semshm.hpp"
 
 // for extensions
 #include "new.hpp"
@@ -97,7 +98,7 @@ void LibInit()
   new DLibFun(lib::routine_info,string("ROUTINE_INFO"),1,routine_infoKey);
 
   const string spawnKey[]={ "COUNT","EXIT_STATUS","PID",
-			    "SH","NOSHELL",KLISTEND};
+			    "SH","NOSHELL","UNIT",KLISTEND};
   new DLibPro(lib::spawn_pro,string("SPAWN"),3,spawnKey);
 
   const string bytsclKey[]={"MIN","MAX","TOP","NAN",KLISTEND};
@@ -166,6 +167,9 @@ void LibInit()
   const string file_infoKey[]={"NOEXPAND_PATH",KLISTEND};
   new DLibFunRetNew(lib::file_info,string("FILE_INFO"),2,file_infoKey);
 
+  const string file_mkdirKey[]={"NOEXPAND_PATH",KLISTEND};
+  new DLibPro(lib::file_mkdir,string("FILE_MKDIR"),-1,file_mkdirKey);
+
   new DLibFunRetNew(lib::shift_fun,string("SHIFT"),9,NULL,NULL,true);
 
   const string sortKey[]={"L64",KLISTEND};
@@ -326,9 +330,14 @@ void LibInit()
   
   const string stringKey[]={"FORMAT","AM_PM","DAYS_OF_WEEK","MONTH",
 			    "PRINT",KLISTEND};
-  new DLibFunRetNew(lib::string_fun,string("STRING"),-1,stringKey,NULL,true);
-  new DLibFunRetNew(lib::byte_fun,string("BYTE"),10,NULL,NULL,true);
+//  new DLibFunRetNew(lib::string_fun,string("STRING"),-1,stringKey,NULL,true);
+//  new DLibFunRetNew(lib::byte_fun,string("BYTE"),10,NULL,NULL,true);
+// that's apparently the desired bahaviour, see bug no. 3151760
+  new DLibFun(lib::string_fun,string("STRING"),-1,stringKey,NULL);
+  new DLibFun(lib::byte_fun,string("BYTE"),10,NULL,NULL);
+
   const string fixKey[]={"TYPE","PRINT",KLISTEND};
+/*
   new DLibFunRetNew(lib::fix_fun,string("FIX"),10,fixKey,NULL,true);
   new DLibFunRetNew(lib::uint_fun,string("UINT"),10,NULL,NULL,true);
   new DLibFunRetNew(lib::long_fun,string("LONG"),10,NULL,NULL,true);
@@ -339,6 +348,18 @@ void LibInit()
   new DLibFunRetNew(lib::double_fun,string("DOUBLE"),10,NULL,NULL,true);
   new DLibFunRetNew(lib::complex_fun,string("COMPLEX"),MAXRANK+2,NULL,NULL,true);
   new DLibFunRetNew(lib::dcomplex_fun,string("DCOMPLEX"),MAXRANK+2,NULL,NULL,true);
+*/
+// that's apparently the desired bahaviour, see bug no. 3151760
+  new DLibFun(lib::fix_fun,string("FIX"),10,fixKey,NULL);
+  new DLibFun(lib::uint_fun,string("UINT"),10,NULL,NULL);
+  new DLibFun(lib::long_fun,string("LONG"),10,NULL,NULL);
+  new DLibFun(lib::ulong_fun,string("ULONG"),10,NULL,NULL);
+  new DLibFun(lib::long64_fun,string("LONG64"),10,NULL,NULL);
+  new DLibFun(lib::ulong64_fun,string("ULONG64"),10,NULL,NULL);
+  new DLibFun(lib::float_fun,string("FLOAT"),10,NULL,NULL);
+  new DLibFun(lib::double_fun,string("DOUBLE"),10,NULL,NULL);
+  new DLibFun(lib::complex_fun,string("COMPLEX"),MAXRANK+2,NULL,NULL);
+  new DLibFun(lib::dcomplex_fun,string("DCOMPLEX"),MAXRANK+2,NULL,NULL);
 
   new DLibFunRetNew(lib::gdl_logical_and,string("LOGICAL_AND"),2,NULL,NULL,true);
   new DLibFunRetNew(lib::gdl_logical_or,string("LOGICAL_OR"),2,NULL,NULL,true);
@@ -463,117 +484,69 @@ void LibInit()
       "CLOSE_FILE","FILENAME","LANDSCAPE","PORTRAIT",
       "DECOMPOSED","GET_DECOMPOSED","Z_BUFFERING","SET_RESOLUTION",
       "SET_CHARACTER_SIZE","GET_VISUAL_DEPTH","XSIZE","YSIZE",
-      "COLOR","GET_SCREEN_SIZE","INCHES","WINDOW_STATE",KLISTEND
+      "COLOR","GET_SCREEN_SIZE","INCHES","WINDOW_STATE","SCALE_FACTOR", 
+      "XOFFSET", "YOFFSET", "ENCAPSULATED", KLISTEND
     };
-  const string deviceWarnKey[] = {"RETAIN", "SET_FONT", KLISTEND};
+  const string deviceWarnKey[] = {"RETAIN", "SET_FONT", "HELVETICA", 
+    "AVANTGARDE", "BKMAN", "COURIER", "PALATINO", 
+    "SCHOOLBOOK", "TIMES", "ZAPFCHANCERY", "ZAPFDINGBATS", "BITS_PER_PIXEL", 
+    "ITALIC", "BOLD", KLISTEND};
   new DLibPro(lib::device,string("DEVICE"),0, deviceKey, deviceWarnKey);
 
   const string plotKey[]=
     {
-     // GRAPHIC KEYWORDS
-     // 0
      "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
-     // 4
-     "COLOR",     "DATA",    "DEVICE",   "FONT",
-     // 8
+     "COLOR",     "DATA",    "DEVICE", 
      "LINESTYLE", "NOCLIP",  "NODATA",   "NOERASE", 
-     // 12
      "NORMAL",    "POSITION","PSYM",     "SUBTITLE",
-     // 16
-     "SYMSIZE",   "T3D",     "THICK",    "TICKLEN", 
-     // 20
-     "TITLE",
-     // 21
-     "XCHARSIZE",    "XGRIDSTYLE", "XMARGIN", "XMINOR",
-     // 25
-     "XRANGE",       "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     // 29
-     "XTICKINTERVAL","XTICKLAYOUT","XTICKLEN","XTICKNAME",
-     // 33
-     "XTICKS",       "XTICKUNITS", "XTICKV",  "XTICK_GET",
-     // 37
-     "XTITLE",
-     // 38
-     "YCHARSIZE",    "YGRIDSTYLE", "YMARGIN", "YMINOR",
-     // 42
-     "YRANGE",       "YSTYLE",     "YTHICK",  "YTICKFORMAT",
-     // 46
-     "YTICKINTERVAL","YTICKLAYOUT","YTICKLEN","YTICKNAME",
-     // 50
-     "YTICKS",       "YTICKUNITS", "YTICKV",  "YTICK_GET",
-     // 51
-     "YTITLE",
-     // 55
-     "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     // 59
-     "ZRANGE",       "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     // 63
-     "ZTICKINTERVAL","ZTICKLAYOUT","ZTICKLEN","ZTICKNAME",
-     // 67
-     "ZTICKS",       "ZTICKUNITS", "ZTICKV",  "ZTICK_GET",
-     // 71
-     "ZTITLE",
-     // 72
-     "ZVALUE", 
-
-     // PLOT keywords
-     // 73
-     "ISOTROPIC",
-     "MAX_VALUE", "MIN_VALUE","NSUM",     "POLAR",
-     "XLOG",     "YLOG",     "YNOZERO", "XTYPE", "YTYPE", 
+     "SYMSIZE",   "THICK",    "TICKLEN", "TITLE",
+     "XCHARSIZE", "XMARGIN", "XMINOR",
+     "XRANGE",    "XSTYLE",     "XTHICK",  "XTICKFORMAT",
+     "XTICKLEN",  "XTICKS",     "XTITLE",
+     "YCHARSIZE", "YMARGIN", "YMINOR",
+     "YRANGE",    "YSTYLE",     "YTHICK",  "YTICKFORMAT", "YTICKLEN",
+     "YTICKS",    "YTITLE", "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
+     "ZRANGE",    "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
+     "ZTICKLEN",  "ZTICKS",    "ZTITLE", "ZVALUE", "MAX_VALUE", "MIN_VALUE",
+     "XLOG",      "YLOG",     "YNOZERO", "XTYPE", "YTYPE", 
      KLISTEND
     };
-  new DLibPro(lib::plot,string("PLOT"),2,plotKey);
+  //"POLAR",
+  const string plotWarnKey[]= {
+    "ISOTROPIC", "FONT", "T3D", "YTICKUNITS", "NSUM", "XTICKLAYOUT", "YTICKLAYOUT", 
+    "ZTICKLAYOUT", "XGRIDSTYLE", "YGRIDSTYLE", "XTICKUNITS", "XTICKV", "XTICK_GET", "YTICKV",
+    "YTICK_GET", "XTICKNAME", "ZTICKNAME", "ZTICKUNITS", "ZTICKV",  "ZTICK_GET",
+    "XTICKINTERVAL", "YTICKINTERVAL", "YTICKNAME", "ZTICKINTERVAL", 
+    KLISTEND
+  };
+  new DLibPro(lib::plot,string("PLOT"),2,plotKey,plotWarnKey);
 
   const string axisKey[]=
     {
-     // GRAPHIC KEYWORDS
-     // 0
      "CHARSIZE","CHARTHICK",
-     // 2
      "COLOR",     "DATA",    "DEVICE",   "FONT",
-     // 6
      "NODATA",    "NOERASE", 
-     // 8
      "NORMAL",    "SUBTITLE",
-     // 10
      "T3D",       "TICKLEN", 
-     // 12
      "XCHARSIZE", "XGRIDSTYLE", "XMARGIN", "XMINOR",
-     // 16
      "XRANGE",    "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     // 20
      "XTICKLEN",  "XTICKNAME",
-     // 22
      "XTICKS",    
-     // 23
      "XTITLE",
-     // 24
      "YCHARSIZE",  "YGRIDSTYLE", "YMARGIN", "YMINOR",
-     // 28
      "YRANGE",     "YSTYLE",     "YTHICK",  "YTICKFORMAT",
-     // 30
      "YTICKLEN",   "YTICKNAME",
-     // 32
      "YTICKS",
-     // 33
      "YTITLE",
-     // 34
      "ZCHARSIZE",  "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     // 38
      "ZRANGE",     "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     // 42
      "ZTICKLEN",   "ZTICKNAME",
-     // 44
      "ZTICKS",
-     // 45
      "ZTITLE",
-     // 46
      "ZVALUE", 
 
-     // AXIS keywords
-     // 47
      "SAVE", "XAXIS", "YAXIS", "XLOG", "YLOG", "XTYPE", "YTYPE", "YNOZERO", "THICK",
+     "XTICKINTERVAL", "YTICKINTERVAL",
      KLISTEND
     };
   new DLibPro(lib::axis,string("AXIS"),3,axisKey);
@@ -593,16 +566,16 @@ void LibInit()
 
   const string plotsKey[]=
     {
-      // GRAPHIC KEYWORDS
-      // 0
       "CLIP","COLOR", "LINESTYLE", "NOCLIP",
-      // 4
       "PSYM", "SYMSIZE",  "T3D",  "THICK",
-      // 8
-      "DATA","DEVICE","NORMAL",
+      "DATA","DEVICE","NORMAL","CONTINUE",
       KLISTEND
     };
-  new DLibPro(lib::plots, string("PLOTS"), 3, plotsKey);
+  const string plotsWarnKey[] = 
+  {
+    "Z", KLISTEND
+  };
+  new DLibPro(lib::plots, string("PLOTS"), 3, plotsKey, plotsWarnKey);
 
   const string surfaceKey[]=
     {
@@ -658,61 +631,40 @@ void LibInit()
      "XLOG", "XTYPE", "YLOG", "YTYPE", "ZLOG", "ZTYPE",
      KLISTEND
     };
-  new DLibPro(lib::surface,string("SURFACE"),3,surfaceKey);
+  const string surfaceWarnKey[]=
+  {
+    "SAVE", KLISTEND 
+  };
+  new DLibPro(lib::surface,string("SURFACE"),3,surfaceKey, surfaceWarnKey);
 
   const string contourKey[]=
     {
-     // GRAPHIC KEYWORDS
-     // 0
      "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
-     // 4
      "COLOR",     "DATA",    "DEVICE",   "FONT",
-     // 8
      "NOCLIP",  "NODATA",   "NOERASE", 
-     // 11
      "NORMAL",    "POSITION",     "SUBTITLE",
-     // 14
      "T3D",     "THICK",    "TICKLEN", 
-     // 17
      "TITLE",
-     // 18
      "XCHARSIZE",    "XGRIDSTYLE", "XMARGIN", "XMINOR",
-     // 22
      "XRANGE",       "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     // 26
      "XTICKLEN","XTICKNAME",
-     // 28
      "XTICKS",       "XTICKV",  "XTICK_GET",
-     // 31
      "XTITLE",
-     // 32
      "YCHARSIZE",    "YGRIDSTYLE", "YMARGIN", "YMINOR",
-     // 36
      "YRANGE",       "YSTYLE",     "YTHICK",  "YTICKFORMAT",
-     // 40
      "YTICKLEN","YTICKNAME",
-     // 42
      "YTICKS",       "YTICKV",  "YTICK_GET",
-     // 45
      "YTITLE",
-     // 46
      "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     // 50
      "ZRANGE",       "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     // 54
      "ZTICKLEN","ZTICKNAME",
-     // 56
      "ZTICKS",       "ZTICKV",  "ZTICK_GET",
-     // 59
      "ZTITLE",
-     // 60
      "ZVALUE", 
-     // CONTOUR keywords
-     // 73
      "LEVELS", "NLEVELS",
-     "MAX_VALUE", "MIN_VALUE","AX", "AZ",
+     "MAX_VALUE", "MIN_VALUE",
      "XLOG", "XTYPE", "YLOG", "YTYPE", "ZLOG", "ZTYPE", "FILL",  "ISOTROPIC",
-     "FOLLOW", "C_CHARSIZE","OVERPLOT",
+     "FOLLOW", "C_CHARSIZE","OVERPLOT","C_COLORS","C_LINESTYLE",
      KLISTEND
     };
   new DLibPro(lib::contour,string("CONTOUR"),3,contourKey);
@@ -808,6 +760,13 @@ void LibInit()
   const string gdl_erfinvKey[] = {"DOUBLE", KLISTEND };
   new DLibFun(lib::gdl_erfinv_fun, string("GDL_ERFINV"), 1, gdl_erfinvKey);
 
+  // by Mateusz Turcza
+  const string sem_createKey[] = {"DESTROY_SEMAPHORE", KLISTEND };
+  new DLibFun(lib::sem_create, string("SEM_CREATE"), 1, sem_createKey);
+  new DLibPro(lib::sem_delete, string("SEM_DELETE"), 1);
+  new DLibFun(lib::sem_lock, string("SEM_LOCK"), 1);
+  new DLibPro(lib::sem_release, string("SEM_RELEASE"), 1);
+
   // sort lists
   sort( libFunList.begin(), libFunList.end(), CompLibFunName());
   sort( libProList.begin(), libProList.end(), CompLibProName());
