@@ -92,9 +92,8 @@ void LibInit_jmg()
 			       "LOCATIONS",KLISTEND};
   new DLibFun(lib::histogram_fun,string("HISTOGRAM"),1,histogramKey);
 
-  const string interpolateKey[]={"CUBIC","GRID",KLISTEND};
-  const string interpolateWarnKey[]={"MISSING",KLISTEND};
-  new DLibFun(lib::interpolate_fun,string("INTERPOLATE"),4,interpolateKey,interpolateWarnKey);
+  const string interpolateKey[]={"CUBIC","GRID","MISSING",KLISTEND};
+  new DLibFun(lib::interpolate_fun,string("INTERPOLATE"),4,interpolateKey);
 
   const string la_triredKey[]={"DOUBLE","UPPER",KLISTEND};
   new DLibPro(lib::la_trired_pro,string("LA_TRIRED"),3,la_triredKey);
@@ -166,7 +165,7 @@ void LibInit_jmg()
   new DLibPro(lib::wait,string("WAIT"),1);
 
 #if defined(USE_HDF)
-  const string hdf_openKey[]={"READ","RDWR","CREATE",KLISTEND};
+  const string hdf_openKey[]={"READ","RDWR","CREATE","ALL","NUM_DD","WRITE",KLISTEND};
   new DLibFun(lib::hdf_open_fun,string("HDF_OPEN"),2,hdf_openKey);
 
   new DLibFun(lib::hdf_vg_getid_fun,string("HDF_VG_GETID"),2);
@@ -250,7 +249,7 @@ void LibInit_jmg()
  
 #endif
 
-  const string tvKey[]={"TRUE","NORMAL","CHANNEL","XSIZE","YSIZE","ORDER",KLISTEND};
+  const string tvKey[]={"TRUE","NORMAL","CHANNEL","XSIZE","YSIZE","ORDER","DEVICE",KLISTEND};
   new DLibPro(lib::tv,string("TV"),4,tvKey);
 
   const string tvrdKey[]={"CHANNEL","ORDER","TRUE","WORDS",KLISTEND};
@@ -259,8 +258,6 @@ void LibInit_jmg()
   const string loadctKey[]={"GET_NAMES",KLISTEND};
   new DLibPro(lib::loadct,string("LOADCT_INTERNALGDL"),1,loadctKey);
 
-
-#ifdef HAVE_LIBWXWIDGETS
   const string widget_baseKey[] = {"ALIGN_BOTTOM","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","ALIGN_TOP","MBAR","MODAL","BASE_ALIGN_BOTTOM","BASE_ALIGN_CENTER","BASE_ALIGN_LEFT","BASE_ALIGN_RIGHT","BASE_ALIGN_TOP","COLUMN","ROW","CONTEXT_EVENTS","CONTEXT_MENU","EVENT_FUNC","EVENT_PRO","EXCLUSIVE","NONEXCLUSIVE","FLOATING","FRAME","FUNC_GET_VALUE","GRID_LAYOUT","GROUP_LEADER","KBRD_FOCUS_EVENTS","KILL_NOTIFY","MAP","NO_COPY","NOTIFY_REALIZE","PRO_SET_VALUE","SCR_XSIZE","SCR_YSIZE","SCROLL","SENSITIVE","SPACE","TITLE","TLB_FRAME_ATTR","TLB_ICONIFY_EVENTS","TLB_KILL_REQUEST_EVENTS","TLB_MOVE_EVENTS","TLB_SIZE_EVENTS","TOOLBAR","TRACKING_EVENTS","UNITS","UNAME","UVALUE","XOFFSET","XPAD","XSIZE","X_SCROLL_SIZE","YOFFSET","YPAD","YSIZE","Y_SCROLL_SIZE","DISPLAY_NAME","RESOURCE_NAME","RNAME_MBAR",KLISTEND};
   new DLibFun(lib::widget_base,string("WIDGET_BASE"),1,widget_baseKey);
 
@@ -293,7 +290,6 @@ void LibInit_jmg()
 				      "GET_VALUE",KLISTEND};
   new DLibPro(lib::widget_control,string("WIDGET_CONTROL"),1, 
 	      widget_controlKey);
-#endif
 
   // call_external (by Christoph Fuchs)
   const string call_externalKey[] = {"VALUE", "ALL_VALUE", "RETURN_TYPE",
