@@ -14,9 +14,11 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#else
+#include "includefirst.hpp"
+
+#ifndef HAVE_CONFIG_H
+// #include <config.h>
+// #else
 // default: assume we have ImageMagick
 #define USE_MAGICK 1
 #define USE_MAGICK6 1
@@ -25,8 +27,6 @@
 
 #ifdef USE_MAGICK
 
-#include "includefirst.hpp"
-
 #include <string>
 #include <fstream>
 #include <memory>
@@ -126,7 +126,9 @@ namespace lib {
       e->AssureScalarPar<DStringGDL>(0,filename);
 
       WordExp( filename);
-      
+
+      if (filename.length() == 0) e->Throw("Void file Name");
+
       Image a;
       try 
       {
@@ -386,6 +388,7 @@ namespace lib {
 	IndexPacket* index;
 	pixel=image.getPixels(0,0,columns,rows);
 	index=image.getIndexes();
+        if (index == NULL) e->Throw("(FIXME!) Magick's getIndexes() returned NULL for: " +e->GetParString(0));
 	unsigned int cx, cy;
 	for (cy=0;cy<rows;++cy)
 	  for (cx=0;cx<columns;++cx)
@@ -498,10 +501,6 @@ namespace lib {
 	
 	columns=image.columns();
 	rows=image.rows();
-	SizeT c[3];
-	if(image.matte()) c[0]=4;
-	else c[0]=3;
-
 	
 	string map="BGR";
 	if(e->GetKW(0) != NULL)//RGB
@@ -522,8 +521,8 @@ namespace lib {
 		Message(s);		
 		map="BGR";
 	      }
-	    if(image.matte()) map=map+"A";
 	  }
+	if(image.matte()) map=map+"A";
 
 	if(e->KeywordSet(2)) //MAP
 	  e->AssureScalarPar<DStringGDL>(0,map);    
@@ -545,14 +544,12 @@ namespace lib {
 
 	    if(ly+wy > rows) 
 	      e->Throw("Requested height exceeds number of rows. Either reduce the height or the Y origin.");
-
-
-	    
 	  }	
 
-	c[0]=map.length();
-	c[1]=wx;
-	c[2]=wy;
+	SizeT c[3];
+	c[0] = map.length();
+	c[1] = wx;
+	c[2] = wy;
 	dimension dim(c,3);	  
 
 	if(image.depth() == 8)
@@ -993,7 +990,7 @@ namespace lib {
     PixelPacket* pixels;
     IndexPacket* index;
 
-    unsigned int columns, rows, cx, cy;
+    unsigned int columns, rows;
     columns=image.columns();
     rows=image.rows();
 
@@ -1001,12 +998,12 @@ namespace lib {
     index=image.getIndexes();
 
 SizeT nEl = columns*rows;
-#pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
+// #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
-#pragma omp for
-    for(cx=0;cx<nEl;++cx)
+// #pragma omp for
+    for(SizeT cx=0;cx<nEl;++cx)
       {
-	    index[cx]=(unsigned int)(*bImage)[cx];
+	    index[cx]=static_cast<unsigned int>((*bImage)[cx]);
 /*	    *index=(unsigned int)(*bImage)[cx];
 	    index++;*/
       }
