@@ -493,7 +493,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
     {
     case GDLTokenTypes::FCALL_LIB_RETNEW:
       {
-	ProgNodeP c = new DefaultNode( refNode);
+	ProgNodeP c = new FCALL_LIB_RETNEWNode( refNode);
 
 	if( !static_cast<DLibFunRetNew*>(c->libFun)->RetConstant()
 	    || !ConstantPar( c->getFirstChild())) return c;
@@ -534,6 +534,14 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
       {
 	return new ARRAYEXPRNode( refNode);
       }
+    case GDLTokenTypes::EXPR:
+      {
+	return new EXPRNode( refNode);
+      }
+    case GDLTokenTypes::DOT:
+      {
+	return new DOTNode( refNode);
+      }
     
     case GDLTokenTypes::VAR:
       {
@@ -719,6 +727,14 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
       {
 	return new  PCALLNode( refNode);
       }
+    case GDLTokenTypes::POSTDEC:
+      {
+	return new  POSTDECNode( refNode);
+      }
+    case GDLTokenTypes::POSTINC:
+      {
+	return new  POSTINCNode( refNode);
+      }
     case GDLTokenTypes::DEC:
       {
 	return new  DECNode( refNode);
@@ -728,97 +744,148 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	return new  INCNode( refNode);
       }
       // the following must change their type if precalculation strikes
-		case GDLTokenTypes::KEYDEF_REF:
-		{
-		  ProgNodeP nn = new KEYDEF_REFNode( refNode);
+    case GDLTokenTypes::KEYDEF_REF:
+    {
+      ProgNodeP nn = new KEYDEF_REFNode( refNode);
 
-		  if( !nn->getFirstChild()->getNextSibling()->ConstantNode())
-		    return nn;
+      if( !nn->getFirstChild()->getNextSibling()->ConstantNode())
+	return nn;
 
 // 		  nn->setType( GDLTokenTypes::KEYDEF);
-		  delete nn;
-		  nn = new KEYDEFNode( refNode);
-		  return nn;
-		}
-		case GDLTokenTypes::KEYDEF_REF_EXPR:
-		{
-		  ProgNodeP nn = new KEYDEF_REF_EXPRNode( refNode);
+      delete nn;
+      nn = new KEYDEFNode( refNode);
+      return nn;
+    }
+    case GDLTokenTypes::KEYDEF_REF_EXPR:
+    {
+      ProgNodeP nn = new KEYDEF_REF_EXPRNode( refNode);
 
-		  if( !nn->getFirstChild()->getNextSibling()->ConstantNode())
-		    return nn;
+      if( !nn->getFirstChild()->getNextSibling()->ConstantNode())
+	return nn;
 
-		  delete nn;
-		  nn = new KEYDEFNode( refNode);
+      delete nn;
+      nn = new KEYDEFNode( refNode);
 // 		  nn->setType( GDLTokenTypes::KEYDEF);
-		  return nn;
-		}
-		case  GDLTokenTypes::PARAEXPR:
-		  {
-		  ProgNodeP firstChild = 
-		    new ParameterNode( refNode);
-		  return firstChild;
-		  }
-		case  GDLTokenTypes::KEYDEF:
-		  {
-		  ProgNodeP nn = new KEYDEFNode( refNode);
-		  return nn;  
-		  }
-		case GDLTokenTypes::REF:
-		{
-		  ProgNodeP nn = new REFNode( refNode);
+      return nn;
+    }
+    case  GDLTokenTypes::PARAEXPR:
+      {
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);
+      return firstChild;
+      }
+    case  GDLTokenTypes::PARAEXPR_VN:
+      {
+      ProgNodeP firstChild = 
+	new ParameterVNNode( refNode);
+      return firstChild;
+      }
+    case  GDLTokenTypes::KEYDEF:
+      {
+      ProgNodeP nn = new KEYDEFNode( refNode);
+      return nn;  
+      }
+    case GDLTokenTypes::REF:
+    {
+      ProgNodeP nn = new REFNode( refNode);
 
-		  if( !nn->getFirstChild()->ConstantNode())
-		    return nn;
-		  
-		  delete nn;
-		  
-		  ProgNodeP firstChild = 
-		    new ParameterNode( refNode);//->GetFirstChild());
-		  return firstChild;
-		}
-		case GDLTokenTypes::REF_EXPR:
-		{
-		  ProgNodeP nn = new REF_EXPRNode( refNode);
+      if( !nn->getFirstChild()->ConstantNode())
+	return nn;
+      
+      delete nn;
+      
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);//->GetFirstChild());
+      return firstChild;
+    }
+    case GDLTokenTypes::REF_VN:
+    {
+      ProgNodeP nn = new REFVNNode( refNode);
 
-		  if( !nn->getFirstChild()->ConstantNode())
-		    return nn;
-		  
-		  delete nn;
-		  ProgNodeP firstChild = 
-		    new ParameterNode( refNode);//->GetFirstChild());
-		  return firstChild;
+      if( !nn->getFirstChild()->ConstantNode())
+	return nn;
+      
+      delete nn;
+      
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);//->GetFirstChild());
+      return firstChild;
+    }
+    case GDLTokenTypes::REF_EXPR:
+    {
+      ProgNodeP nn = new REF_EXPRNode( refNode);
+
+      if( !nn->getFirstChild()->ConstantNode())
+	return nn;
+      
+      delete nn;
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);//->GetFirstChild());
+      return firstChild;
 
 // 		  auto_ptr<ProgNode> guard(nn);
 // 		  return nn->StealFirstChild();
-		}
- 		case GDLTokenTypes::KEYDEF_REF_CHECK:
-		{
-		  ProgNodeP nn = new KEYDEF_REF_CHECKNode( refNode);
+    }
+    case GDLTokenTypes::REF_EXPR_VN:
+    {
+      ProgNodeP nn = new REF_EXPRVNNode( refNode);
 
-		  if( nn->getFirstChild()->getNextSibling()->getType() != 
-		      GDLTokenTypes::CONSTANT) return nn;
+      if( !nn->getFirstChild()->ConstantNode())
+	return nn;
+      
+      delete nn;
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);//->GetFirstChild());
+      return firstChild;
 
-		  delete nn;
-		  nn = new KEYDEFNode( refNode);
-		  return nn;
-		}
-		case GDLTokenTypes::REF_CHECK:
-		{
-		  ProgNodeP nn = new REF_CHECKNode( refNode);
+// 		  auto_ptr<ProgNode> guard(nn);
+// 		  return nn->StealFirstChild();
+    }
+    case GDLTokenTypes::KEYDEF_REF_CHECK:
+    {
+      ProgNodeP nn = new KEYDEF_REF_CHECKNode( refNode);
 
-		  if( !nn->getFirstChild()->ConstantNode())
-		    return nn;
-		  
-		  delete nn;
-		  ProgNodeP firstChild = 
-		    new ParameterNode( refNode);
-		  return firstChild;
+      if( nn->getFirstChild()->getNextSibling()->getType() != 
+	  GDLTokenTypes::CONSTANT) return nn;
+
+      delete nn;
+      nn = new KEYDEFNode( refNode);
+      return nn;
+    }
+    case GDLTokenTypes::REF_CHECK:
+    {
+      ProgNodeP nn = new REF_CHECKNode( refNode);
+
+      if( !nn->getFirstChild()->ConstantNode())
+	return nn;
+      
+      delete nn;
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);
+      return firstChild;
 
 // 		  auto_ptr<ProgNode> guard(nn);
 // 		  ProgNodeP firstChild = nn->StealFirstChild();
 // 		  firstChild->SetNextSibling( nn->StealNextSibling());
 // 		  return firstChild;
-		}
+    }
+    case GDLTokenTypes::REF_CHECK_VN:
+    {
+      ProgNodeP nn = new REF_CHECKVNNode( refNode);
+
+      if( !nn->getFirstChild()->ConstantNode())
+	return nn;
+      
+      delete nn;
+      ProgNodeP firstChild = 
+	new ParameterNode( refNode);
+      return firstChild;
+
+// 		  auto_ptr<ProgNode> guard(nn);
+// 		  ProgNodeP firstChild = nn->StealFirstChild();
+// 		  firstChild->SetNextSibling( nn->StealNextSibling());
+// 		  return firstChild;
+    }
     }
 
   // default
