@@ -27,6 +27,20 @@ email                : m_schellens@users.sf.net
 
 using namespace std;
 
+bool* GetNonCopyNodeLookupArray()
+{
+static bool nonCopyNodeLookupArray[ GDLTokenTypes::MAX_TOKEN_NUMBER];
+for( int i=0; i<GDLTokenTypes::MAX_TOKEN_NUMBER; ++i)
+	nonCopyNodeLookupArray[ i] = false;
+nonCopyNodeLookupArray[ GDLTokenTypes::VAR] = true;
+nonCopyNodeLookupArray[ GDLTokenTypes::VARPTR] = true;
+nonCopyNodeLookupArray[ GDLTokenTypes::DEREF] = true;
+nonCopyNodeLookupArray[ GDLTokenTypes::CONSTANT] = true;
+nonCopyNodeLookupArray[ GDLTokenTypes::SYSVAR] = true;
+return nonCopyNodeLookupArray;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // c-tor d-tor
 /////////////////////////////////////////////////////////////////////////////
@@ -46,6 +60,7 @@ ProgNode::ProgNode( const RefDNode& refNode):
   libFun( refNode->libFun),
   var( refNode->var),
   arrIxList( refNode->StealArrIxList()),
+  arrIxListNoAssoc( refNode->StealArrIxNoAssocList()),
 //   arrIxList( refNode->CloneArrIxList()),
   labelStart( refNode->labelStart),
   labelEnd( refNode->labelEnd)
@@ -65,6 +80,7 @@ ProgNode::~ProgNode()
   if( (getType() == GDLTokenTypes::ARRAYIX))
     {
       delete arrIxList;
+      delete arrIxListNoAssoc;
     }
   if( !keepDown) delete down;
   if( !keepRight) delete right;
@@ -113,6 +129,133 @@ bool ProgNode::ConstantNode()
 /////////////////////////////////////////////////////////
 // Eval 
 /////////////////////////////////////////////////////////
+BaseGDL* ASSIGNNode::Eval()
+{
+    ProgNodeP _t = this->getFirstChild();
+
+    auto_ptr<BaseGDL> r_guard;
+	BaseGDL* res;
+    if( _t->getType() == GDLTokenTypes::FCALL_LIB)
+        {
+            res=interpreter->lib_function_call(_t);
+            _t = interpreter->GetRetTree();
+            if( !interpreter->CallStackBack()->Contains( res))
+                r_guard.reset( res);
+        }
+    else
+        {
+            res=interpreter->tmp_expr(_t);
+            _t = interpreter->GetRetTree();
+            r_guard.reset( res);
+        }
+
+    BaseGDL** l=_t->LExpr( res); //l_expr(_t, res);
+    //_t = _retTree;
+    if( r_guard.get() == res) // owner
+      r_guard.release();
+    else
+      res = res->Dup();
+    return res;
+}
+BaseGDL* ASSIGN_ARRAYEXPR_MFCALLNode::Eval()
+{
+    ProgNodeP _t = this->getFirstChild();
+
+    auto_ptr<BaseGDL> r_guard;
+
+	BaseGDL* res;
+    if( _t->getType() == GDLTokenTypes::FCALL_LIB)
+        {
+            res=interpreter->lib_function_call(_t);
+            _t = interpreter->GetRetTree();
+            if( !interpreter->CallStackBack()->Contains( res))
+                r_guard.reset( res);
+        }
+    else
+        {
+            res=interpreter->tmp_expr(_t);
+            _t = interpreter->GetRetTree();
+            r_guard.reset( res);
+        }
+
+    ProgNodeP mark = _t;
+    // try MFCALL
+    try
+	{
+	  BaseGDL** l=interpreter->l_arrayexpr_mfcall_as_mfcall( mark);
+
+	  if( res != (*l))
+	    {
+	      delete *l;
+	      *l = res->Dup();
+
+	      if( r_guard.get() == res) // owner
+		  {
+		      r_guard.release();
+		  }
+	      else
+			  res = res->Dup();
+	    }
+	}
+    catch( GDLException& ex)
+	{
+	  // try ARRAYEXPR
+	  try
+	    {
+		    BaseGDL** l=interpreter->l_arrayexpr_mfcall_as_arrayexpr(mark, res);
+		    if( r_guard.get() == res) // owner
+				r_guard.release();
+		    else
+			res = res->Dup();
+	    }
+	  catch( GDLException& ex2)
+	    {
+		throw GDLException(ex.toString() + " or "+ex2.toString());
+	    }
+	}
+	return res;
+}
+BaseGDL* ASSIGN_REPLACENode::Eval()
+{
+    ProgNodeP _t = this->getFirstChild();
+
+    auto_ptr<BaseGDL> r_guard;
+
+	BaseGDL* res;
+    if( _t->getType() == GDLTokenTypes::FCALL_LIB)
+        {
+            res=interpreter->lib_function_call(_t);
+            _t = interpreter->GetRetTree();
+            if( !interpreter->CallStackBack()->Contains( res))
+                r_guard.reset( res);
+        }
+    else
+        {
+            res=interpreter->tmp_expr(_t);
+            _t = interpreter->GetRetTree();
+            r_guard.reset( res);
+        }
+
+    BaseGDL** l=_t->LEval();
+    //_t = _t->getNextSibling();
+    //_t = _retTree;
+
+    if( res != (*l))
+    {
+      delete *l;
+      *l = res->Dup();
+
+      if( r_guard.get() == res) // owner
+      {
+		r_guard.release();
+      }
+      else
+	res = res->Dup();
+	}
+	return res;
+}
+
+
 BaseGDL* ARRAYDEFNode::Eval()
 {
   // GDLInterpreter::
@@ -195,7 +338,7 @@ BaseGDL* ARRAYDEFNode::Eval()
 	
   BaseGDL* res=cTypeData->CatArray(exprList,this->arrayDepth,maxRank);
 	
-  ProgNode::interpreter->_retTree = _t;
+//   ProgNode::interpreter->_retTree = _t;
   return res;
 }
 
@@ -220,7 +363,7 @@ BaseGDL* STRUCNode::Eval()
     ProgNodeP si = _t;
     // 			match(antlr::RefAST(_t),IDENTIFIER);
     _t = _t->getNextSibling();
-    BaseGDL* e=_t->Eval();//expr(_t);
+    BaseGDL* e=_t->Eval(); //interpreter->expr(_t);
     _t = _t->getNextSibling();
     //WRONG    _t = ProgNode::interpreter->_retTree;
 			
@@ -231,7 +374,7 @@ BaseGDL* STRUCNode::Eval()
   instance_guard.release();
   BaseGDL* res=instance;
 	
-  ProgNode::interpreter->_retTree = rTree;
+//   ProgNode::interpreter->_retTree = rTree;
   return res;
 }
 
@@ -382,7 +525,7 @@ BaseGDL* NSTRUCNode::Eval()
   instance_guard.release();
   BaseGDL* res=instance;
 	
-  ProgNode::interpreter->_retTree = this->getNextSibling();
+//   ProgNode::interpreter->_retTree = this->getNextSibling();
   return res;
 }
 BaseGDL* NSTRUC_REFNode::Eval()
@@ -404,7 +547,7 @@ BaseGDL* NSTRUC_REFNode::Eval()
 
   BaseGDL* res = new DStructGDL( dStruct, dimension(1));
 	
-  ProgNode::interpreter->_retTree = this->getNextSibling();
+//   ProgNode::interpreter->_retTree = this->getNextSibling();
   return res;
 }
 
@@ -416,13 +559,18 @@ void KEYDEF_REFNode::Parameter( EnvBaseT* actEnv)
 //   ProgNodeP knameR = _t;
   // 			match(antlr::RefAST(_t),IDENTIFIER);
 //   _t = _t->getNextSibling();
-  BaseGDL** kvalRef=ProgNode::interpreter->ref_parameter(_t->getNextSibling(), actEnv);
+  BaseGDL** kvalRef=_t->getNextSibling()->LEval();
+  //ProgNode::interpreter->ref_parameter(_t->getNextSibling(), actEnv);
 
   // pass reference
   actEnv->SetKeyword( _t->getText(), kvalRef); 
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+// void KEYDEF_REFNode::ParameterVarNum( EnvBaseT* actEnv)
+// {
+//   return Parameter( actEnv);
+// }
 
 void KEYDEF_REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 {
@@ -432,16 +580,22 @@ void KEYDEF_REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 //   _t = _t->getNextSibling();
   BaseGDL* kval= _t->getNextSibling()->Eval();//expr(_t);
   delete kval;
-
 //   _t = ProgNode::interpreter->_retTree;
-  BaseGDL** kvalRef=ProgNode::interpreter->
-    ref_parameter(ProgNode::interpreter->_retTree, actEnv);
+  BaseGDL** kvalRef=_t->getNextSibling()->getNextSibling()->LEval();
+//   BaseGDL** kvalRef=ProgNode::interpreter->_retTree->LEval();
+//   ProgNode::interpreter->
+//     ref_parameter(ProgNode::interpreter->_retTree, actEnv);
 
   // pass reference
   actEnv->SetKeyword( _t->getText(), kvalRef); 
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+// void KEYDEF_REF_EXPRNode::ParameterVarNum( EnvBaseT* actEnv)
+// {
+//   return Parameter( actEnv);
+// }
+
 void KEYDEFNode::Parameter( EnvBaseT* actEnv)
 {
    ProgNodeP _t = this->getFirstChild();
@@ -454,16 +608,29 @@ void KEYDEFNode::Parameter( EnvBaseT* actEnv)
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+// void KEYDEFNode::ParameterVarNum( EnvBaseT* actEnv)
+// {
+//   return Parameter( actEnv);
+// }
+
 void REFNode::Parameter( EnvBaseT* actEnv)
 {
 //   ProgNodeP _t = this->getFirstChild();
-  BaseGDL** pvalRef=ProgNode::interpreter->ref_parameter(this->getFirstChild(), actEnv);
+  BaseGDL** pvalRef=this->getFirstChild()->LEval();
+//   ProgNode::interpreter->ref_parameter(this->getFirstChild(), actEnv);
 //   _t = ProgNode::interpreter->_retTree;
   // pass reference
-  actEnv->SetNextPar(pvalRef); 
+  actEnv->SetNextParUnchecked(pvalRef); 
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+void REFVNNode::Parameter( EnvBaseT* actEnv)
+{
+  BaseGDL** pvalRef=this->getFirstChild()->LEval();
+  actEnv->SetNextParUncheckedVarNum(pvalRef); 
+  ProgNode::interpreter->_retTree = this->getNextSibling();
+}
+
 void REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 {
   // 			match(antlr::RefAST(_t),REF_EXPR);
@@ -471,14 +638,24 @@ void REF_EXPRNode::Parameter( EnvBaseT* actEnv)
   BaseGDL* pval= this->getFirstChild()->Eval();//expr(_t);
   delete pval;
 //   _t = ProgNode::interpreter->_retTree;
-  BaseGDL** pvalRef=ProgNode::interpreter->
-    ref_parameter( ProgNode::interpreter->_retTree, actEnv);
+  BaseGDL** pvalRef=this->getFirstChild()->getNextSibling()->LEval();
+//   ProgNode::interpreter->
+//     ref_parameter( ProgNode::interpreter->_retTree, actEnv);
 
   // pass reference
-  actEnv->SetNextPar(pvalRef); 
+  actEnv->SetNextParUnchecked(pvalRef); 
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+void REF_EXPRVNNode::Parameter( EnvBaseT* actEnv)
+{
+  BaseGDL* pval= this->getFirstChild()->Eval();//expr(_t);
+  delete pval;
+  BaseGDL** pvalRef=this->getFirstChild()->getNextSibling()->LEval();
+  actEnv->SetNextParUncheckedVarNum(pvalRef); 
+  ProgNode::interpreter->_retTree = this->getNextSibling();
+}
+
 void KEYDEF_REF_CHECKNode::Parameter( EnvBaseT* actEnv)
 {
 //   ProgNodeP _t = this->getFirstChild();
@@ -500,6 +677,11 @@ void KEYDEF_REF_CHECKNode::Parameter( EnvBaseT* actEnv)
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+// void KEYDEF_REF_CHECKNode::ParameterVarNum( EnvBaseT* actEnv)
+// {
+//   return Parameter( actEnv);
+// }
+
 void REF_CHECKNode::Parameter( EnvBaseT* actEnv)
 {
   BaseGDL* pval=ProgNode::interpreter->lib_function_call(this->getFirstChild());
@@ -507,26 +689,46 @@ void REF_CHECKNode::Parameter( EnvBaseT* actEnv)
   BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
   if( pvalRef != NULL)
     {   // pass reference
-      actEnv->SetNextPar( pvalRef); 
+      actEnv->SetNextParUnchecked( pvalRef); 
     }
   else 
     {   // pass value
-      actEnv->SetNextPar( pval); 
+      actEnv->SetNextParUnchecked( pval); 
     }
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
+void REF_CHECKVNNode::Parameter( EnvBaseT* actEnv)
+{
+  BaseGDL* pval=ProgNode::interpreter->lib_function_call(this->getFirstChild());
+  BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
+  if( pvalRef != NULL)
+    {   // pass reference
+      actEnv->SetNextParUncheckedVarNum( pvalRef); 
+    }
+  else 
+    {   // pass value
+      actEnv->SetNextParUncheckedVarNum( pval); 
+    }
+  ProgNode::interpreter->_retTree = this->getNextSibling();
+}
 
 void ParameterNode::Parameter( EnvBaseT* actEnv)
 {
 //   BaseGDL* pval=this->Eval();//expr(this);
 			
   // pass value
-  actEnv->SetNextPar(this->getFirstChild()->Eval()); 
+  actEnv->SetNextParUnchecked(this->getFirstChild()->Eval()); 
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
 
+void ParameterVNNode::Parameter( EnvBaseT* actEnv)
+{
+  actEnv->SetNextParUncheckedVarNum(this->getFirstChild()->Eval()); 
+  ProgNode::interpreter->_retTree = this->getNextSibling();
+}
+
 
 
 RetCode  ASSIGNNode::Run()
@@ -537,41 +739,60 @@ RetCode  ASSIGNNode::Run()
 	
   //     match(antlr::RefAST(_t),ASSIGN);
   ProgNodeP _t = this->getFirstChild();
+  if( NonCopyNode(_t->getType()))
   {
-    switch ( _t->getType()) {
-    case GDLTokenTypes::CONSTANT:
-    case GDLTokenTypes::DEREF:
-    case GDLTokenTypes::SYSVAR:
-    case GDLTokenTypes:: VAR:
-    case GDLTokenTypes::VARPTR:
-      {
-	r= ProgNode::interpreter->indexable_expr(_t);
-	_t = ProgNode::interpreter->_retTree;
-	break;
-      }
-    case GDLTokenTypes::FCALL_LIB:
-      {
-	r=ProgNode::interpreter->lib_function_call(_t);
-	_t = ProgNode::interpreter->_retTree;
-			
-	if( !ProgNode::interpreter->callStack.back()->Contains( r)) 
-	  r_guard.reset( r); // guard if no global data
-			
-	break;
-      }
-    default:
-      {
-	r=ProgNode::interpreter->indexable_tmp_expr(_t);
-	_t = ProgNode::interpreter->_retTree;
-	r_guard.reset( r);
-	break;
-      }
-    }//switch
+      r= _t->EvalNC(); //ProgNode::interpreter->indexable_expr(_t);
+      _t = _t->getNextSibling();
+      l=_t->LExpr( r); //ProgNode::interpreter->l_expr(_t, r);
   }
-  l=ProgNode::interpreter->l_expr(_t, r);
+  else if( _t->getType() == GDLTokenTypes::FCALL_LIB)
+  {
+      r=ProgNode::interpreter->lib_function_call(_t);
+      _t = ProgNode::interpreter->_retTree;		      
+      if( !ProgNode::interpreter->callStack.back()->Contains( r)) 
+ 	r_guard.reset( r); // guard if no global data
+      l=_t->LExpr( r); //ProgNode::interpreter->l_expr(_t, r);
+  }
+  else
+  {
+      r=_t->Eval(); //ProgNode::interpreter->indexable_tmp_expr(_t);
+      _t = _t->getNextSibling();
+//       _t = ProgNode::interpreter->_retTree;
+      r_guard.reset( r);
+      l=_t->LExpr( r); //ProgNode::interpreter->l_expr(_t, r);
+  }
+//     switch ( _t->getType()) {
+//     case GDLTokenTypes::CONSTANT:
+//     case GDLTokenTypes::DEREF:
+//     case GDLTokenTypes::SYSVAR:
+//     case GDLTokenTypes::VAR:
+//     case GDLTokenTypes::VARPTR:
+//       {
+// 	r= ProgNode::interpreter->indexable_expr(_t);
+// 	_t = ProgNode::interpreter->_retTree;
+// 	break;
+//       }
+//     case GDLTokenTypes::FCALL_LIB:
+//       {
+// 	r=ProgNode::interpreter->lib_function_call(_t);
+// 	_t = ProgNode::interpreter->_retTree;
+// 			
+// 	if( !ProgNode::interpreter->callStack.back()->Contains( r)) 
+// 	  r_guard.reset( r); // guard if no global data
+// 			
+// 	break;
+//       }
+//     default:
+//       {
+// 	r=ProgNode::interpreter->indexable_tmp_expr(_t);
+// 	_t = ProgNode::interpreter->_retTree;
+// 	r_guard.reset( r);
+// 	break;
+//       }
+//     }//switch
+//   l=_t->LExpr( r); //ProgNode::interpreter->l_expr(_t, r);
 
   ProgNode::interpreter->_retTree = this->getNextSibling();
-
   return RC_OK;
 }
 
@@ -589,70 +810,83 @@ RetCode  ASSIGN_ARRAYEXPR_MFCALLNode::Run()
     // BOTH
     if( _t->getType() ==  GDLTokenTypes::FCALL_LIB)
       {
-		r=ProgNode::interpreter->lib_function_call(_t);
+	r=ProgNode::interpreter->lib_function_call(_t);
 
-		if( r == NULL) // ROUTINE_NAMES
-			ProgNode::interpreter->callStack.back()->Throw( "Undefined return value");
+	if( r == NULL) // ROUTINE_NAMES
+		ProgNode::interpreter->callStack.back()->Throw( "Undefined return value");
+
+	_t = ProgNode::interpreter->_retTree;
 	
-		_t = ProgNode::interpreter->_retTree;
-		
-		if( !ProgNode::interpreter->callStack.back()->Contains( r)) 
-			r_guard.reset( r);
+	if( !ProgNode::interpreter->callStack.back()->Contains( r)) 
+		r_guard.reset( r);
 			
       }
     else
       {
-			// ASSIGN
-			switch ( _t->getType()) {
-				case GDLTokenTypes::CONSTANT:
-				case GDLTokenTypes::DEREF:
-				case GDLTokenTypes::SYSVAR:
-				case GDLTokenTypes:: VAR:
-				case GDLTokenTypes::VARPTR:
-				{
-				r= ProgNode::interpreter->indexable_expr(_t);
-				_t = ProgNode::interpreter->_retTree;
-				break;
-				}
-				default:
-				{
-				r=ProgNode::interpreter->indexable_tmp_expr(_t);
-				_t = ProgNode::interpreter->_retTree;
-				r_guard.reset( r);
-				break;
-			}
-			}//switch
-		}
+	// ASSIGN
+	if( NonCopyNode(_t->getType()))
+	{
+	  r= _t->EvalNC(); //ProgNode::interpreter->indexable_expr(_t);
+	  _t = _t->getNextSibling(); // ProgNode::interpreter->_retTree;
+	}
+	else
+	{
+	  r= _t->Eval(); //ProgNode::interpreter->indexable_tmp_expr(_t);
+	  _t = _t->getNextSibling();
+// 	  _t = ProgNode::interpreter->_retTree;
+	  r_guard.reset( r);
+	}
+
+// 	switch ( _t->getType()) {
+// 	  case GDLTokenTypes::CONSTANT:
+// 	  case GDLTokenTypes::DEREF:
+// 	  case GDLTokenTypes::SYSVAR:
+// 	  case GDLTokenTypes:: VAR:
+// 	  case GDLTokenTypes::VARPTR:
+// 	  {
+// 	    r= ProgNode::interpreter->indexable_expr(_t);
+// 	    _t = ProgNode::interpreter->_retTree;
+// 	    break;
+// 	  }
+// 	  default:
+// 	  {
+// 	    r=ProgNode::interpreter->indexable_tmp_expr(_t);
+// 	    _t = ProgNode::interpreter->_retTree;
+// 	    r_guard.reset( r);
+// 	    break;
+// 	  }
+// 	}//switch
+    }
   }
 
-	ProgNodeP lExpr = _t;
+//     ProgNodeP lExpr = _t;
     
-	// try MFCALL
-	try
+    // try MFCALL
+    try
     {
-    l=ProgNode::interpreter->l_arrayexpr_mfcall_as_mfcall(_t);
-    
-	if( r != (*l))
-		{
-		delete *l;
+      l=ProgNode::interpreter->l_arrayexpr_mfcall_as_mfcall(_t);
+  
+      if( r != (*l))
+	{
+	  delete *l;
 
-		if( r_guard.get() == r)
-		*l = r_guard.release();
-		else
-		*l = r->Dup();
-		}
+	  if( r_guard.get() == r)
+	    *l = r_guard.release();
+	  else
+	    *l = r->Dup();
+	}
     }
     catch( GDLException& e)
     {
-		// try ARRAYEXPR
-		try
-		{
-			l=ProgNode::interpreter->l_arrayexpr_mfcall_as_arrayexpr(lExpr, r);
-		}
-		catch( GDLException& e2)
-		{
-			throw GDLException(e.toString() + " or "+e2.toString());
-		}
+      // try ARRAYEXPR
+      try
+      {
+	l=ProgNode::interpreter->l_arrayexpr_mfcall_as_arrayexpr(_t/*lExpr*/, r);
+      }
+      catch( GDLException& e2)
+      {
+	throw GDLException(e.toString() + " or "+e2.toString());
+      }
     }
 
   ProgNode::interpreter->_retTree = this->getNextSibling();
@@ -664,68 +898,58 @@ RetCode  ASSIGN_ARRAYEXPR_MFCALLNode::Run()
 RetCode  ASSIGN_REPLACENode::Run()
 {
   BaseGDL*  r;
-  BaseGDL** l;
   auto_ptr<BaseGDL> r_guard;
 
   //match(antlr::RefAST(_t),ASSIGN_REPLACE);
   ProgNodeP _t = this->getFirstChild();
   {
-    if( _t->getType() ==  GDLTokenTypes::FCALL_LIB)
+//     if( _t->getType() ==  GDLTokenTypes::FCALL_LIB)
+//       {
+// 	r=_t->Eval();//different: ProgNode::interpreter->lib_function_call(_t);
+// 	_t = _t->getNextSibling(); //ProgNode::interpreter->_retTree;
+// 	assert(_t != NULL);
+// 	r_guard.reset( r);
+// // 	if( !ProgNode::interpreter->callStack.back()->Contains( r))
+// // 		r_guard.reset( r);
+// // 	else
+// // 		r_guard.reset( r->Dup());
+//       }
+//     else
       {
-		r=_t->Eval();//ProgNode::interpreter->lib_function_call(_t);
-		r_guard.reset( r);
-
-		if( r == NULL) // ROUTINE_NAMES
-			throw GDLException( this, "Undefined return value", true, false);
-		
-		_t = ProgNode::interpreter->_retTree;
-		
-/*		if( !ProgNode::interpreter->callStack.back()->Contains( r))
-			r_guard.reset( r);
-		else
-			r_guard.reset( r->Dup());*/
-      }
-    else
-      {
-			r=ProgNode::interpreter->tmp_expr(_t);
-			_t = ProgNode::interpreter->_retTree;
-					
-			r_guard.reset( r);
+	//r=ProgNode::interpreter->tmp_expr(_t);
+ 	r = _t->Eval();
+	r_guard.reset( r);
+ 	_t = _t->getNextSibling();
+	assert(_t != NULL);
       }
   }
-
-  switch ( _t->getType()) {
-  case GDLTokenTypes::VAR:
-  case GDLTokenTypes::VARPTR:
-    {
-      l=ProgNode::interpreter->l_simple_var(_t);
-//       _t = ProgNode::interpreter->_retTree;
-      break;
-    }
-  case GDLTokenTypes::DEREF:
-    {
-      l=ProgNode::interpreter->l_deref(_t);
-//       _t = ProgNode::interpreter->_retTree;
-      break;
-    }
-  default:
-//   case GDLTokenTypes::FCALL:
-//   case GDLTokenTypes::FCALL_LIB:
-//   case GDLTokenTypes::MFCALL:
-//   case GDLTokenTypes::MFCALL_PARENT:
-    {
-      l=ProgNode::interpreter->l_function_call(_t);
-//       _t = ProgNode::interpreter->_retTree;
-      break;
-    }
-  }
-//  if( r != (*l))
+//   switch ( _t->getType()) {
+//   case GDLTokenTypes::VAR:
+//   case GDLTokenTypes::VARPTR:
+//   case GDLTokenTypes::DEREF:
 //     {
-//      if( r_guard.get() == r)
-	*l = r_guard.release();
-//       else
-// 	*l = r->Dup();
+//       l=_t->LEval(); //ProgNode::interpreter->l_simple_var(_t);
+// //       _t = ProgNode::interpreter->_retTree;
+//       break;
 //     }
+//   default:
+// //   case GDLTokenTypes::FCALL:
+// //   case GDLTokenTypes::FCALL_LIB:
+// //   case GDLTokenTypes::MFCALL:
+// //   case GDLTokenTypes::MFCALL_PARENT:
+//     {
+//       l=ProgNode::interpreter->l_function_call(_t);
+// //       _t = ProgNode::interpreter->_retTree;
+//       break;
+//     }
+//   } // switch
+  
+  BaseGDL** l=_t->LEval();
+
+  if( r != (*l))
+    delete *l;
+  
+  *l = r_guard.release();
 
   ProgNode::interpreter->SetRetTree( this->getNextSibling());
   return RC_OK;
@@ -747,7 +971,7 @@ RetCode  PCALL_LIBNode::Run()
 		
   EnvT* newEnv=new EnvT( pl, pl->libPro);//libProList[pl->proIx]);
 		
-  ProgNode::interpreter->parameter_def(_t, newEnv);
+  ProgNode::interpreter->parameter_def_nocheck(_t, newEnv);
   //   _t = _retTree;
   //if( this->getLine() != 0) ProgNode::interpreter->callStack.back()->SetLineNumber( this->getLine());
 		
@@ -773,8 +997,8 @@ RetCode  MPCALLNode::Run()
 	
   // 			match(antlr::RefAST(_t),MPCALL);
   ProgNodeP _t = this->getFirstChild();
-  self=ProgNode::interpreter->expr(_t);
-  _t = ProgNode::interpreter->_retTree;
+  self=_t->Eval(); //ProgNode::interpreter->expr(_t);
+  _t = _t->getNextSibling(); // ProgNode::interpreter->_retTree;
   ProgNodeP mp = _t;
   // 			match(antlr::RefAST(_t),IDENTIFIER);
   _t = _t->getNextSibling();
@@ -810,8 +1034,10 @@ RetCode  MPCALL_PARENTNode::Run()
 
   // 			match(antlr::RefAST(_t),MPCALL_PARENT);
   ProgNodeP _t = this->getFirstChild();
-  self=ProgNode::interpreter->expr(_t);
-  _t = ProgNode::interpreter->_retTree;
+  self=_t->Eval(); //ProgNode::interpreter->expr(_t);
+  _t = _t->getNextSibling(); // ProgNode::interpreter->_retTree;
+//   self=ProgNode::interpreter->expr(_t);
+//   _t = ProgNode::interpreter->_retTree;
   ProgNodeP parent = _t;
   // 			match(antlr::RefAST(_t),IDENTIFIER);
   _t = _t->getNextSibling();
@@ -839,6 +1065,7 @@ RetCode  MPCALL_PARENTNode::Run()
   //  ProgNode::interpreter->_retTree = this->getNextSibling();
   return RC_OK;
 }
+
 RetCode  PCALLNode::Run()
 {
   // better than auto_ptr: auto_ptr wouldn't remove newEnv from the stack
@@ -870,197 +1097,224 @@ RetCode  PCALLNode::Run()
   return RC_OK;
 }
 
+BaseGDL* POSTDECNode::Eval()
+{
+  BaseGDL* res=interpreter->l_decinc_expr( this->getFirstChild(), GDLTokenTypes::POSTDEC);
+//   interpreter->SetRetTree(this->getNextSibling());
+  return res;
+}
 
+BaseGDL* DECNode::Eval()
+{
+  BaseGDL* res=interpreter->l_decinc_expr( this->getFirstChild(), GDLTokenTypes::DEC);
+//   interpreter->SetRetTree(this->getNextSibling());
+  return res;
+}
 
 RetCode  DECNode::Run()
 {
   //		match(antlr::RefAST(_t),DEC);
   ProgNodeP _t = this->getFirstChild();
   ProgNode::interpreter->l_decinc_expr(_t, GDLTokenTypes::DECSTATEMENT);
-
   ProgNode::interpreter->SetRetTree( this->getNextSibling());
   return RC_OK;
 }
 
 
+BaseGDL* POSTINCNode::Eval()
+{
+  BaseGDL* res=interpreter->l_decinc_expr( this->getFirstChild(), GDLTokenTypes::POSTINC);
+//   interpreter->SetRetTree(this->getNextSibling());
+  return res;
+}
+
+BaseGDL* INCNode::Eval()
+{
+  BaseGDL* res=interpreter->l_decinc_expr( this->getFirstChild(), GDLTokenTypes::INC);
+//   interpreter->SetRetTree(this->getNextSibling());
+  return res;
+}
 
 RetCode  INCNode::Run()
 {
   //		match(antlr::RefAST(_t),INC);
   ProgNodeP _t = this->getFirstChild();
   ProgNode::interpreter->l_decinc_expr(_t, GDLTokenTypes::INCSTATEMENT);
-
   ProgNode::interpreter->SetRetTree( this->getNextSibling());
   return RC_OK;
 }
 
 RetCode   FORNode::Run()//for_statement(ProgNodeP _t) {
 {
-		EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
-		
-		ForLoopInfoT& loopInfo = callStack_back->GetForLoopInfo( this->forLoopIx);
-	
-		ProgNodeP vP = this->GetNextSibling()->GetFirstChild();
-		
-		BaseGDL** v=ProgNode::interpreter->l_simple_var(vP);
-		
-		BaseGDL* s=ProgNode::interpreter->expr( this->GetFirstChild());
-		auto_ptr<BaseGDL> s_guard(s);
-		
-		delete loopInfo.endLoopVar;
-		loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling());
-		
-		s->ForCheck( &loopInfo.endLoopVar);
-		
-		// ASSIGNMENT used here also
-		delete (*v);
-		(*v)= s_guard.release(); // s held in *v after this
-		
-		if( (*v)->ForCondUp( loopInfo.endLoopVar))
-		{
-			ProgNode::interpreter->_retTree = vP->GetNextSibling();
-			return RC_OK;
-		}
-		else
-		{
-			// skip if initial test fails
-			ProgNode::interpreter->_retTree = this->GetNextSibling()->GetNextSibling();
-			return RC_OK;
-		}
+  EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
+  
+  ForLoopInfoT& loopInfo = callStack_back->GetForLoopInfo( this->forLoopIx);
+
+  ProgNodeP vP = this->GetNextSibling()->GetFirstChild();
+  
+  BaseGDL** v=vP->LEval(); //ProgNode::interpreter->l_simple_var(vP);
+  
+  BaseGDL* s=this->GetFirstChild()->Eval(); 
+//   BaseGDL* s=ProgNode::interpreter->expr( this->GetFirstChild());
+  auto_ptr<BaseGDL> s_guard(s);
+  
+  delete loopInfo.endLoopVar;
+  loopInfo.endLoopVar=this->GetFirstChild()->GetNextSibling()->Eval();
+//   loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling());
+  
+  s->ForCheck( &loopInfo.endLoopVar);
+  
+  // ASSIGNMENT used here also
+  delete (*v);
+  (*v)= s_guard.release(); // s held in *v after this
+  
+  if( (*v)->ForCondUp( loopInfo.endLoopVar))
+  {
+	  ProgNode::interpreter->_retTree = vP->GetNextSibling();
+	  return RC_OK;
+  }
+  else
+  {
+	  // skip if initial test fails
+	  ProgNode::interpreter->_retTree = this->GetNextSibling()->GetNextSibling();
+	  return RC_OK;
+  }
 }
 
 
-	RetCode   FOR_LOOPNode::Run()
-	{
-		EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
- 		ForLoopInfoT& loopInfo = 	callStack_back->GetForLoopInfo( this->forLoopIx);
-		BaseGDL* endLoopVar = 	loopInfo.endLoopVar;
-		if( endLoopVar == NULL)
-		{
-			// non-initialized loop (GOTO)
-			ProgNode::interpreter->_retTree = this->GetNextSibling();
-			return RC_OK;
-
-		}
-
-		// // problem:
-		// // EXECUTE may call DataListT.loc.resize(), as v points to the
-		// // old sequence v might be invalidated -> segfault
-		// // note that the value (*v) is preserved by resize()
-		
-		BaseGDL** v=this->getFirstChild()->LEval();//ProgNode::interpreter->l_simple_var(this->getFirstChild());
-
-// shortCut:;
-		
-		//(*v)->ForAdd();
-		if( (*v)->ForAddCondUp( endLoopVar))
-		{
-			ProgNode::interpreter->_retTree = this->statementList; //GetFirstChild()->GetNextSibling();
-// 			if( ProgNode::interpreter->_retTree == this) goto shortCut;
-		}
-		else
-		{
-			delete loopInfo.endLoopVar;
-			loopInfo.endLoopVar = NULL;
-			ProgNode::interpreter->_retTree = this->GetNextSibling();
-		}
-		return RC_OK;
-	}
-
-	
- RetCode   FOR_STEPNode::Run()//for_statement(ProgNodeP _t) {
+RetCode   FOR_LOOPNode::Run()
 {
 	EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
-
-	ForLoopInfoT& loopInfo = callStack_back->GetForLoopInfo( this->forLoopIx);
-
-	ProgNodeP vP = this->GetNextSibling()->GetFirstChild();
-
-	BaseGDL** v=ProgNode::interpreter->l_simple_var(vP);
-
-	BaseGDL* s=ProgNode::interpreter->expr( this->GetFirstChild());
-	auto_ptr<BaseGDL> s_guard(s);
-
-	delete loopInfo.endLoopVar;
-	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling());
-
-	delete loopInfo.loopStepVar;
-	loopInfo.loopStepVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling()->GetNextSibling());
-
-	s->ForCheck( &loopInfo.endLoopVar, &loopInfo.loopStepVar);
-
-	// ASSIGNMENT used here also
-	delete (*v);
-	(*v)= s_guard.release(); // s held in *v after this
-
-	if( loopInfo.loopStepVar->Sgn() == -1)
-	{
-		if( (*v)->ForCondDown( loopInfo.endLoopVar))
-		{
-			ProgNode::interpreter->_retTree = vP->GetNextSibling();
-  return RC_OK;
-
-		}
-	}
-	else
-	{
-		if( (*v)->ForCondUp( loopInfo.endLoopVar))
-		{
-			ProgNode::interpreter->_retTree = vP->GetNextSibling();
-  return RC_OK;
-
-		}
-	}
-	// skip if initial test fails
-	ProgNode::interpreter->_retTree = this->GetNextSibling()->GetNextSibling();
-  return RC_OK;
-}
-	
-RetCode   FOR_STEP_LOOPNode::Run()
-{
-	EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
-	
 	ForLoopInfoT& loopInfo = 	callStack_back->GetForLoopInfo( this->forLoopIx);
+// 		BaseGDL* endLoopVar = 	loopInfo.endLoopVar;
 	if( loopInfo.endLoopVar == NULL)
 	{
 		// non-initialized loop (GOTO)
 		ProgNode::interpreter->_retTree = this->GetNextSibling();
-        return RC_OK;
+		return RC_OK;
 	}
 
 	// // problem:
 	// // EXECUTE may call DataListT.loc.resize(), as v points to the
 	// // old sequence v might be invalidated -> segfault
 	// // note that the value (*v) is preserved by resize()
+	
+	BaseGDL** v=this->getFirstChild()->LEval();//ProgNode::interpreter->l_simple_var(this->getFirstChild());
 
-	BaseGDL** v=ProgNode::interpreter->l_simple_var(this->GetFirstChild());
-
-	(*v)->ForAdd(loopInfo.loopStepVar);
-	if( loopInfo.loopStepVar->Sgn() == -1)
+// shortCut:;
+	
+	//(*v)->ForAdd();
+	if( (*v)->ForAddCondUp( loopInfo.endLoopVar))
 	{
-		if( (*v)->ForCondDown( loopInfo.endLoopVar))
-		{
-			ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling();
-  return RC_OK;
-		}
+		ProgNode::interpreter->_retTree = this->statementList; //GetFirstChild()->GetNextSibling();
+// 			if( ProgNode::interpreter->_retTree == this) goto shortCut;
 	}
 	else
 	{
-		if( (*v)->ForCondUp( loopInfo.endLoopVar))
-		{
-			ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling();
-  return RC_OK;
-		}
+		delete loopInfo.endLoopVar;
+		loopInfo.endLoopVar = NULL;
+		ProgNode::interpreter->_retTree = this->GetNextSibling();
 	}
-	
-	delete loopInfo.endLoopVar;
-	loopInfo.endLoopVar = NULL;
-	delete loopInfo.loopStepVar;
-	loopInfo.loopStepVar = NULL;
-	ProgNode::interpreter->_retTree = this->GetNextSibling();
 	return RC_OK;
 }
 
+	
+RetCode   FOR_STEPNode::Run()//for_statement(ProgNodeP _t) {
+{
+  EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
+
+  ForLoopInfoT& loopInfo = callStack_back->GetForLoopInfo( this->forLoopIx);
+
+  ProgNodeP vP = this->GetNextSibling()->GetFirstChild();
+
+  BaseGDL** v=vP->LEval(); //ProgNode::interpreter->l_simple_var(vP);
+
+  BaseGDL* s=this->GetFirstChild()->Eval(); 
+//  BaseGDL* s=ProgNode::interpreter->expr( this->GetFirstChild());
+  auto_ptr<BaseGDL> s_guard(s);
+
+  delete loopInfo.endLoopVar;
+  loopInfo.endLoopVar=this->GetFirstChild()->GetNextSibling()->Eval();
+//  loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling());
+
+  delete loopInfo.loopStepVar;
+  loopInfo.loopStepVar=this->GetFirstChild()->GetNextSibling()->GetNextSibling()->Eval();
+//   loopInfo.loopStepVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling()->GetNextSibling());
+
+  s->ForCheck( &loopInfo.endLoopVar, &loopInfo.loopStepVar);
+
+  // ASSIGNMENT used here also
+  delete (*v);
+  (*v)= s_guard.release(); // s held in *v after this
+
+  if( loopInfo.loopStepVar->Sgn() == -1)
+  {
+	  if( (*v)->ForCondDown( loopInfo.endLoopVar))
+	  {
+		  ProgNode::interpreter->_retTree = vP->GetNextSibling();
+		  return RC_OK;
+
+	  }
+  }
+  else
+  {
+	  if( (*v)->ForCondUp( loopInfo.endLoopVar))
+	  {
+		  ProgNode::interpreter->_retTree = vP->GetNextSibling();
+		  return RC_OK;
+
+	  }
+  }
+  // skip if initial test fails
+  ProgNode::interpreter->_retTree = this->GetNextSibling()->GetNextSibling();
+  return RC_OK;
+}
+	
+RetCode   FOR_STEP_LOOPNode::Run()
+{
+  EnvUDT* callStack_back = static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
+  
+  ForLoopInfoT& loopInfo = callStack_back->GetForLoopInfo( this->forLoopIx);
+  if( loopInfo.endLoopVar == NULL)
+  {
+    // non-initialized loop (GOTO)
+    ProgNode::interpreter->_retTree = this->GetNextSibling();
+    return RC_OK;
+  }
+
+  // // problem:
+  // // EXECUTE may call DataListT.loc.resize(), as v points to the
+  // // old sequence v might be invalidated -> segfault
+  // // note that the value (*v) is preserved by resize()
+
+  BaseGDL** v=this->GetFirstChild()->LEval(); //ProgNode::interpreter->l_simple_var(this->GetFirstChild());
+
+  (*v)->ForAdd(loopInfo.loopStepVar);
+  if( loopInfo.loopStepVar->Sgn() == -1)
+  {
+    if( (*v)->ForCondDown( loopInfo.endLoopVar))
+    {
+	    ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling();
+	    return RC_OK;
+    }
+  }
+  else
+  {
+    if( (*v)->ForCondUp( loopInfo.endLoopVar))
+    {
+	    ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling();
+	    return RC_OK;
+    }
+  }
+  
+  delete loopInfo.endLoopVar;
+  loopInfo.endLoopVar = NULL;
+  delete loopInfo.loopStepVar;
+  loopInfo.loopStepVar = NULL;
+  ProgNode::interpreter->_retTree = this->GetNextSibling();
+  return RC_OK;
+}
+
 RetCode   FOREACHNode::Run()
 {
 	EnvUDT* callStack_back = 	static_cast<EnvUDT*>(GDLInterpreter::CallStack().back());
@@ -1068,10 +1322,11 @@ RetCode   FOREACHNode::Run()
 
 	ProgNodeP vP = this->GetNextSibling()->GetFirstChild();
 
-	BaseGDL** v=ProgNode::interpreter->l_simple_var(vP);
+	BaseGDL** v=vP->LEval(); // ProgNode::interpreter->l_simple_var(vP);
 
 	delete loopInfo.endLoopVar;
-	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild());
+	loopInfo.endLoopVar=this->GetFirstChild()->Eval();
+//	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild());
 
 	loopInfo.foreachIx = 0;
 
@@ -1095,10 +1350,10 @@ RetCode   FOREACH_LOOPNode::Run()
 	{
 	// non-initialized loop (GOTO)
 	ProgNode::interpreter->_retTree = this->GetNextSibling();
-  return RC_OK;
+	return RC_OK;
 	}
 
-	BaseGDL** v=ProgNode::interpreter->l_simple_var(this->GetFirstChild());
+	BaseGDL** v=this->GetFirstChild()->LEval(); //ProgNode::interpreter->l_simple_var(this->GetFirstChild());
 	
 	++loopInfo.foreachIx;
 
@@ -1111,14 +1366,14 @@ RetCode   FOREACH_LOOPNode::Run()
 		(*v) = loopInfo.endLoopVar->NewIx( loopInfo.foreachIx);
 
 		ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling();
-  return RC_OK;
+		return RC_OK;
 	}
 
 	delete loopInfo.endLoopVar;
 	loopInfo.endLoopVar = NULL;
 	// 	loopInfo.foreachIx = -1;
 	ProgNode::interpreter->SetRetTree( this->GetNextSibling());
-  return RC_OK;
+	return RC_OK;
 }
 
 
@@ -1131,11 +1386,12 @@ RetCode   FOREACH_INDEXNode::Run()
 	ProgNodeP vP = this->GetNextSibling()->GetFirstChild();
 	ProgNodeP indexP = vP->GetNextSibling();
 
-	BaseGDL** v=ProgNode::interpreter->l_simple_var(vP);
-	BaseGDL** index=ProgNode::interpreter->l_simple_var(indexP);
+	BaseGDL** v=vP->LEval(); //ProgNode::interpreter->l_simple_var(vP);
+	BaseGDL** index=indexP->LEval(); //ProgNode::interpreter->l_simple_var(indexP);
 
 	delete loopInfo.endLoopVar;
-	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild());
+	loopInfo.endLoopVar=this->GetFirstChild()->Eval(); 
+// 	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild());
 
 	loopInfo.foreachIx = 0;
 
@@ -1163,11 +1419,11 @@ RetCode   FOREACH_INDEX_LOOPNode::Run()
 	{
 	// non-initialized loop (GOTO)
 	ProgNode::interpreter->_retTree = this->GetNextSibling();
-  return RC_OK;
+	return RC_OK;
 	}
 
-	BaseGDL** v=ProgNode::interpreter->l_simple_var(this->GetFirstChild());
-	BaseGDL** index=ProgNode::interpreter->l_simple_var(this->GetFirstChild()->GetNextSibling());
+	BaseGDL** v=this->GetFirstChild()->LEval(); //ProgNode::interpreter->l_simple_var(this->GetFirstChild());
+	BaseGDL** index=this->GetFirstChild()->GetNextSibling()->LEval(); //ProgNode::interpreter->l_simple_var(this->GetFirstChild()->GetNextSibling());
 	
 	++loopInfo.foreachIx;
 
@@ -1198,11 +1454,11 @@ RetCode   FOREACH_INDEX_LOOPNode::Run()
 
 RetCode   REPEATNode::Run()
 {
-	// _t is REPEAT_LOOP, GetFirstChild() is expr, GetNextSibling is first loop statement
-	if( this->GetFirstChild()->GetFirstChild()->GetNextSibling() == NULL)
-		ProgNode::interpreter->SetRetTree( this->GetFirstChild());
-	else	
-		ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetFirstChild()->GetNextSibling());     // statement
+  // _t is REPEAT_LOOP, GetFirstChild() is expr, GetNextSibling is first loop statement
+  if( this->GetFirstChild()->GetFirstChild()->GetNextSibling() == NULL)
+	  ProgNode::interpreter->SetRetTree( this->GetFirstChild());
+  else	
+	  ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetFirstChild()->GetNextSibling());     // statement
   return RC_OK;
 }
 
@@ -1210,16 +1466,17 @@ RetCode   REPEATNode::Run()
 
 RetCode   REPEAT_LOOPNode::Run()
 {
-	auto_ptr<BaseGDL> eVal( ProgNode::interpreter->expr(this->GetFirstChild()));
-	if( eVal.get()->False())
-	{
-	ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling());     // 1st loop statement
-	if(  this->GetFirstChild()->GetNextSibling() == NULL)
-		throw GDLException(this,	"Empty REPEAT loop entered (infinite loop).",true,false);
+  auto_ptr<BaseGDL> eVal( this->GetFirstChild()->Eval());
+//  auto_ptr<BaseGDL> eVal( ProgNode::interpreter->expr(this->GetFirstChild()));
+  if( eVal.get()->False())
+  {
+  ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling());     // 1st loop statement
+  if(  this->GetFirstChild()->GetNextSibling() == NULL)
+	  throw GDLException(this,	"Empty REPEAT loop entered (infinite loop).",true,false);
   return RC_OK;
-	}
-	
-	ProgNode::interpreter->SetRetTree( this->GetNextSibling());     // statement
+  }
+  
+  ProgNode::interpreter->SetRetTree( this->GetNextSibling());     // statement
   return RC_OK;
 }
 
@@ -1232,24 +1489,24 @@ RetCode   WHILENode::Run()
   ProgNodeP evalExpr = this->getFirstChild();
   if( NonCopyNode( evalExpr->getType()))
   {
-	e1 = evalExpr->EvalNC();
+    e1 = evalExpr->EvalNC();
   }
   else
   {
-	e1 = evalExpr->Eval();
+    e1 = evalExpr->Eval();
     e1_guard.reset(e1);
   }
 // 	auto_ptr<BaseGDL> eVal( ProgNode::interpreter->expr( this->GetFirstChild()));
-	if( e1->True()) 
-	{
-		ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling());
-		if( this->GetFirstChild()->GetNextSibling() == NULL)
-			throw GDLException(this,"Empty WHILE loop entered (infinite loop).",true,false);
-	}
-	else
-	{
-		ProgNode::interpreter->SetRetTree( this->GetNextSibling());
-	}
+  if( e1->True()) 
+  {
+    ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling());
+    if( this->GetFirstChild()->GetNextSibling() == NULL)
+	    throw GDLException(this,"Empty WHILE loop entered (infinite loop).",true,false);
+  }
+  else
+  {
+    ProgNode::interpreter->SetRetTree( this->GetNextSibling());
+  }
   return RC_OK;
 }
 
@@ -1293,17 +1550,17 @@ RetCode   IF_ELSENode::Run()
   else
   {
 	e1 = evalExpr->Eval();
-    e1_guard.reset(e1);
+	e1_guard.reset(e1);
   }
 // 	auto_ptr<BaseGDL> eVal( ProgNode::interpreter->expr( this->GetFirstChild()));
-	if( e1->True()) 
-	{
-		ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling()->GetFirstChild());
-	}
-	else
-	{
-		ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling()->GetNextSibling());
-	}
+  if( e1->True()) 
+  {
+	  ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling()->GetFirstChild());
+  }
+  else
+  {
+	  ProgNode::interpreter->SetRetTree( this->GetFirstChild()->GetNextSibling()->GetNextSibling());
+  }
   return RC_OK;
 }
 
@@ -1525,8 +1782,8 @@ RetCode   RETFNode::Run()
 	assert( _t != NULL);
 	if ( !static_cast<EnvUDT*>(GDLInterpreter::CallStack().back())->LFun())
 		{
-			BaseGDL* e=ProgNode::interpreter->expr(_t);
-
+			BaseGDL* e=_t->Eval(); //ProgNode::interpreter->expr(_t);
+			interpreter->SetRetTree( _t->getNextSibling()); // ???
 			delete ProgNode::interpreter->returnValue;
 			ProgNode::interpreter->returnValue=e;
 
@@ -1535,7 +1792,6 @@ RetCode   RETFNode::Run()
 	else
 		{
 			BaseGDL** eL=ProgNode::interpreter->l_ret_expr(_t);
-
 			// returnValueL is otherwise owned
 			ProgNode::interpreter->returnValueL=eL;
 		}
