@@ -33,13 +33,31 @@
 class ProgNode;
 typedef ProgNode* ProgNodeP;
 
+// inline bool* GetNonCopyNodeLookupArray()
+// {
+// static bool nonCopyNodeLookupArray[ GDLTokenTypes::MAX_TOKEN_NUMBER];
+// for( int i=0; i<GDLTokenTypes::MAX_TOKEN_NUMBER; ++i)
+// 	nonCopyNodeLookupArray[ i] = false;
+// nonCopyNodeLookupArray[ GDLTokenTypes::VAR] = true;
+// nonCopyNodeLookupArray[ GDLTokenTypes::VARPTR] = true;
+// nonCopyNodeLookupArray[ GDLTokenTypes::DEREF] = true;
+// nonCopyNodeLookupArray[ GDLTokenTypes::CONSTANT] = true;
+// nonCopyNodeLookupArray[ GDLTokenTypes::SYSVAR] = true;
+// return nonCopyNodeLookupArray;
+// }
+bool* GetNonCopyNodeLookupArray();
+
 inline bool NonCopyNode( int type)
 {
-  return (type == GDLTokenTypes::DEREF) ||
-    (type == GDLTokenTypes::VAR) ||
-    (type == GDLTokenTypes::VARPTR) ||
-    (type == GDLTokenTypes::CONSTANT) ||
-    (type == GDLTokenTypes::SYSVAR) ;
+  static bool* nonCopyNodeLookupArray = GetNonCopyNodeLookupArray();
+  return nonCopyNodeLookupArray[ type];
+//   return 
+//     (type == GDLTokenTypes::VAR) ||
+//     (type == GDLTokenTypes::VARPTR) ||
+//     (type == GDLTokenTypes::DEREF) ||
+//     (type == GDLTokenTypes::CONSTANT) ||
+//     (type == GDLTokenTypes::SYSVAR) ;
+
 // are always copy nodes:
 //     (type == GDLTokenTypes::ARRAYDEF)
 //     (type == GDLTokenTypes::STRUC)
@@ -81,6 +99,7 @@ protected:
     int        initInt;    // for c-i not actually used
     int        numBranch;  // number of branches in switch/case statements
     int        nDot;       // nesting level for tag access
+    int        nParam;     // number of parameters in this parameter list (stored in 1st par)
     int        arrayDepth; // dimension to cat
     int        proIx;      // Index into proList
     int        funIx;      // Index into funList
@@ -88,17 +107,18 @@ protected:
     int        targetIx;   // Index into label list
     int        structDefined; // struct contains entry with no tag name
     int        compileOpt; // for PRO and FUNCTION nodes
-	int        forLoopIx; // acessing loop variables
+    int        forLoopIx; // acessing loop variables
   };
 
-	void SetType( int tt, const std::string& txt) { ttype = tt; text = txt;} 
+  void SetType( int tt, const std::string& txt) { ttype = tt; text = txt;} 
 
-	static ProgNodeP GetNULLProgNodeP(); 
+  static ProgNodeP GetNULLProgNodeP(); 
 
 private:
   // from DNode (see there)
   int lineNumber;
   ArrayIndexListT* arrIxList; // ptr to array index list
+  ArrayIndexListT* arrIxListNoAssoc; // ptr to array index list
   int labelStart; // for loops to determine if to bail out
   int labelEnd; // for loops to determine if to bail out
 
@@ -139,6 +159,8 @@ public:
   
   void SetRightDown( const ProgNodeP right, const ProgNodeP down);
 
+  virtual BaseGDL** LExpr( BaseGDL* right);
+//   virtual BaseGDL** LExprGrab( BaseGDL* right); // take ownership of right
   virtual BaseGDL** LEval();
   virtual BaseGDL* Eval(); // caller receives ownership
   virtual BaseGDL* EvalNC(); // non-copy used by all operators (and in other places)
@@ -245,6 +267,8 @@ public:
   void setLine( int l) { lineNumber = l;}
   void SetGotoIx( int ix) { targetIx=ix;}
 
+  int GetNParam() const { return nParam;}
+
   ProgNodeP BreakTarget() const { return breakTarget;}
   
   bool LabelInRange( const int lIx)
@@ -273,6 +297,8 @@ public:
 	friend class REF_EXPRNode;
 	friend class ParameterNode;
 	friend class ARRAYEXPRNode;
+	friend class EXPRNode;
+	friend class SYSVARNode;
 };
 
 
@@ -1321,6 +1347,15 @@ class ParameterNode: public DefaultNode
 public:
   ParameterNode( const RefDNode& refNode): DefaultNode( refNode) {}
   virtual void Parameter( EnvBaseT* actEnv);
+//   virtual void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
+};
+
+class ParameterVNNode: public ParameterNode
+{
+public:
+  ParameterVNNode( const RefDNode& refNode): ParameterNode( refNode) {}
+  void Parameter( EnvBaseT* actEnv);
+//   virtual void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class KEYDEF_REFNode: public ParameterNode
@@ -1328,6 +1363,7 @@ class KEYDEF_REFNode: public ParameterNode
 public:
   KEYDEF_REFNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class KEYDEF_REF_EXPRNode: public ParameterNode
@@ -1335,6 +1371,7 @@ class KEYDEF_REF_EXPRNode: public ParameterNode
 public:
   KEYDEF_REF_EXPRNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class KEYDEF_REF_CHECKNode: public ParameterNode
@@ -1342,6 +1379,7 @@ class KEYDEF_REF_CHECKNode: public ParameterNode
 public:
   KEYDEF_REF_CHECKNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class KEYDEFNode: public ParameterNode
@@ -1349,6 +1387,7 @@ class KEYDEFNode: public ParameterNode
 public:
   KEYDEFNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class REF_EXPRNode: public ParameterNode
@@ -1356,6 +1395,14 @@ class REF_EXPRNode: public ParameterNode
 public:
   REF_EXPRNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
+};
+class REF_EXPRVNNode: public ParameterNode
+{
+public:
+  REF_EXPRVNNode( const RefDNode& refNode): ParameterNode( refNode) {}
+  void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class REF_CHECKNode: public ParameterNode
@@ -1363,6 +1410,14 @@ class REF_CHECKNode: public ParameterNode
 public:
   REF_CHECKNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
+};
+class REF_CHECKVNNode: public ParameterNode
+{
+public:
+  REF_CHECKVNNode( const RefDNode& refNode): ParameterNode( refNode) {}
+  void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 class REFNode: public ParameterNode
@@ -1370,6 +1425,14 @@ class REFNode: public ParameterNode
 public:
   REFNode( const RefDNode& refNode): ParameterNode( refNode) {}
   void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
+};
+class REFVNNode: public ParameterNode
+{
+public:
+  REFVNNode( const RefDNode& refNode): ParameterNode( refNode) {}
+  void Parameter( EnvBaseT* actEnv);
+//   void ParameterVarNum( EnvBaseT* actEnv); // for variable number of parameters
 };
 
 
@@ -1384,18 +1447,27 @@ class ASSIGNNode: public CommandNode
 public:
   ASSIGNNode( const RefDNode& refNode): CommandNode( refNode) {}
   RetCode Run();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);
+  BaseGDL* Eval();
 };
 class ASSIGN_ARRAYEXPR_MFCALLNode: public CommandNode
 {
 public:
   ASSIGN_ARRAYEXPR_MFCALLNode( const RefDNode& refNode): CommandNode( refNode) {}
   RetCode Run();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);
+  BaseGDL* Eval();
 };
 class ASSIGN_REPLACENode: public CommandNode
 {
 public:
   ASSIGN_REPLACENode( const RefDNode& refNode): CommandNode( refNode) {}
   RetCode Run();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);
+  BaseGDL* Eval();
 };
 class PCALL_LIBNode: public CommandNode
 {
@@ -1424,19 +1496,49 @@ public:
 class DECNode: public CommandNode
 { public:
   DECNode( const RefDNode& refNode): CommandNode( refNode){}
+  BaseGDL* Eval();
   RetCode Run();
 };
 class INCNode: public CommandNode
 { public:
   INCNode( const RefDNode& refNode): CommandNode( refNode){}
+  BaseGDL* Eval();
   RetCode Run();
 };
+class POSTDECNode: public DECNode
+{ public:
+  POSTDECNode( const RefDNode& refNode): DECNode( refNode){}
+  BaseGDL* Eval();
+};
+class POSTINCNode: public INCNode
+{ public:
+  POSTINCNode( const RefDNode& refNode): INCNode( refNode){}
+  BaseGDL* Eval();
+};
 
 class ARRAYEXPRNode: public DefaultNode
 {
 public:
  ARRAYEXPRNode( const RefDNode& refNode): DefaultNode( refNode) {}
  BaseGDL* Eval(); // caller receives ownership
+ BaseGDL** LExpr(BaseGDL* r);
+ //BaseGDL** LEval(); 
+};
+
+class EXPRNode: public DefaultNode
+{
+public:
+ EXPRNode( const RefDNode& refNode): DefaultNode( refNode) {}
+ BaseGDL** LEval();
+};
+
+class DOTNode: public DefaultNode
+{
+public:
+ DOTNode( const RefDNode& refNode): DefaultNode( refNode) {}
+ BaseGDL* Eval();
+ BaseGDL** LExpr( BaseGDL* right);
+
 };
 
 #endif
