@@ -112,6 +112,9 @@ public:
   {}
   BaseGDL** LEval();
   BaseGDL* Eval();
+  BaseGDL* EvalFCALL_LIB(); // might return a non new value
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 
 
@@ -123,6 +126,8 @@ public:
   {}
   BaseGDL** LEval();
   BaseGDL* Eval();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 
 
@@ -134,6 +139,8 @@ public:
   {}
   BaseGDL** LEval();
   BaseGDL* Eval();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 
 
@@ -143,8 +150,10 @@ class FCALLNode: public LeafNode
 public:
   FCALLNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** LExpr( BaseGDL* right);
   BaseGDL** LEval();
   BaseGDL* Eval();
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 
 
@@ -154,8 +163,10 @@ class ARRAYEXPR_MFCALLNode: public LeafNode
 public:
   ARRAYEXPR_MFCALLNode( const RefDNode& refNode): LeafNode( refNode)
   {}
-  BaseGDL** LEval();
+  BaseGDL** LEval(); //{ ThrowGDLException( "Internal errorr: ARRAYEXPR_MFCALLNode called.");}
   BaseGDL* Eval();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 
 
@@ -167,6 +178,9 @@ public:
   {}
   BaseGDL** LEval();
   BaseGDL* EvalNC();
+  BaseGDL* Eval();
+  BaseGDL** LExpr( BaseGDL* right);  
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 class VARPTRNode: public LeafNode
 {
@@ -175,20 +189,31 @@ public:
   {}
   BaseGDL** LEval();
   BaseGDL* EvalNC();
+  BaseGDL* Eval();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 class SYSVARNode: public LeafNode
 {
 public:
   SYSVARNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** LEval();
   BaseGDL* EvalNC();
+  BaseGDL* Eval();
+  BaseGDL** LExpr(BaseGDL* r);
+  //BaseGDL** LExprGrab( BaseGDL* right);    
 };
 class DEREFNode: public LeafNode
 {
 public:
   DEREFNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** LEval();
   BaseGDL* EvalNC();
+  BaseGDL* Eval();
+  BaseGDL** LExpr( BaseGDL* right);
+//   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 class CONSTANTNode: public LeafNode
 {
@@ -202,6 +227,7 @@ public:
   CONSTANTNode( const RefDNode& refNode): LeafNode( refNode)
   {}
   BaseGDL* EvalNC();
+  BaseGDL* Eval();
 };
 class ARRAYDEFNode: public DefaultNode
 { public:
@@ -213,7 +239,7 @@ class ARRAYDEFNode: public DefaultNode
     while(  _t != NULL) {
 
       if( _t->getType() != GDLTokenTypes::CONSTANT)
-	return false;
+		return false;
 
       _t = _t->getNextSibling();
     }
@@ -245,6 +271,8 @@ class QUESTIONNode: public TrinaryExpr
 { public:
   QUESTIONNode( const RefDNode& refNode): TrinaryExpr( refNode){}
   BaseGDL* Eval();
+  BaseGDL** LExpr(BaseGDL* right);
+//   BaseGDL** LExprGrab(BaseGDL* right);
 };
 class UMINUSNode: public UnaryExpr
 { public:
